/******************************************************************************/
/*   Copyright (C) 2005 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
#include "teq1type.h"
#include "teq1ext.h"
/******************************************************************************/
/*                     Do processing for decoder 2.
*******************************************************************************/
int Dec2_Process()
{
 Init_Alpha_Beta_Dec2();
 Compute_Alpha_Beta_Dec2();
 if(Inner_Cnt < (MAX_INNER-ONE))
  Compute_APPLes_Dec2();
 else
  Compute_Final_APPs_Dec2();
 return(0);
}
/******************************************************************************/
/*                          Initialize alpha and beta.
*******************************************************************************/
int Init_Alpha_Beta_Dec2()
{
 int state;
 for(state=ZERO;state<NUM_STATES;state++)
  Alpha[ZERO][state]=Beta[FRAME_SIZE][state]=1.0;
 return(0);
}
/******************************************************************************/
/*                   Compute the alpha and beta values.
*******************************************************************************/
int Compute_Alpha_Beta_Dec2()
{
 int time,state;
 for(time=ZERO;time<=(FRAME_SIZE-TWO);time++)
 {
  for(state=ZERO;state<NUM_STATES;state++)
   Get_New_Alpha_Beta_Dec2(time,state);
  Normalize_Alpha_Beta(time);
 }
 return(0);
}
/******************************************************************************/
/*     Compute the new alpha and beta values for a given state and time.
*******************************************************************************/
int Get_New_Alpha_Beta_Dec2(time,state)
int time,state;
{
 int input,prv_st,nxt_st,iv_inxa,iv_inxb,opa,opb;
 double gea,geb,gia,gib,proba,probb,prv_alpha,nxt_beta,alpha,beta;
 alpha=beta=0.0;
 iv_inxa=De_Iv_Map[time];
 iv_inxb=De_Iv_Map[FRAME_SIZE-ONE-time];
 for(input=ZERO;input<NUM_INPUTS;input++)                       /* Data bits. */
 {
  prv_st=Trel_PS[state][input];
  nxt_st=Trel_NS[state][input];
  opa=Trel_Op[prv_st][input];                             /* Get encoded bit. */
  opb=Trel_Op[state][input];                              /* Get encoded bit. */
  gea=Gamma_E_P2[time][opa];                        /* Corresponds to parity. */
  geb=Gamma_E_P2[FRAME_SIZE-ONE-time][opb];         /* Corresponds to parity. */
  gia=Gamma_I[iv_inxa][input];                        /* Corresponds to data. */
  gib=Gamma_I[iv_inxb][input];                        /* Corresponds to data. */
  proba=Prob[iv_inxa][input];
  probb=Prob[iv_inxb][input];
  prv_alpha=Alpha[time][prv_st];
  nxt_beta=Beta[FRAME_SIZE-ONE-time+ONE][nxt_st];
  alpha=alpha+gea*gia*proba*prv_alpha;
  beta=beta+geb*gib*probb*nxt_beta;
 }
 Alpha[time+ONE][state]=alpha;
 Beta[FRAME_SIZE-ONE-time][state]=beta;
 return(0);
}
/******************************************************************************/
/*           Compute the APPs due to extrinsic information alone.
*******************************************************************************/
int Compute_APPLes_Dec2()
{
 int time;
 double app[NUM_INPUTS];
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  Get_APPLe_Dec2(time,app);
  Get_Prob_Dec2(time,app);
 }
 return(0);
}
/******************************************************************************/
/*   Get a posteriori probability of +1 and -1 due to extrinsic info alone.
*******************************************************************************/
int Get_APPLe_Dec2(time,app)
int time;
double app[];
{
 int state,input,nxt_state,op;
 double ge,prv_alpha,nxt_beta;
 app[ZERO]=app[ONE]=0.0;
 for(state=ZERO;state<NUM_STATES;state++)
 {
  for(input=ZERO;input<NUM_INPUTS;input++)
  {
   nxt_state=Trel_NS[state][input];
   op=Trel_Op[state][input];
   ge=Gamma_E_P2[time][op];                         /* Corresponds to parity. */
   prv_alpha=Alpha[time][state];
   nxt_beta=Beta[time+ONE][nxt_state];
   app[input]=app[input]+ge*prv_alpha*nxt_beta;
  }
 }
 return(0);
}
/******************************************************************************/
/*          Get the normalized probabilities in deinterleaved format.
*******************************************************************************/
int Get_Prob_Dec2(time,app)
int time;
double app[];
{
 int input,iv_inx;
 double sum;
 iv_inx=De_Iv_Map[time];
 sum=0.0;
 for(input=ZERO;input<NUM_INPUTS;input++)
  sum=sum+app[input];
 for(input=ZERO;input<NUM_INPUTS;input++)
  Prob[iv_inx][input]=app[input]/sum;
 return(0);
}
/******************************************************************************/
