/******************************************************************************/
/*   Copyright (C) 2005 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
#include "teq1type.h"
#include "teq1ext.h"
/******************************************************************************/
/*         Compute the final APPs in the last iteration for decoder 1.
*******************************************************************************/
int Compute_Final_APPs_Dec1()
{
 int time;
 double fapp[NUM_INPUTS],parity[NUM_INPUTS];
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  Get_Final_APP_Dec1(time,fapp,parity);
  Get_Final_Prob_Dec1(time,fapp,parity);
 }
 return(0);
}
/******************************************************************************/
/*  Get the final unnormalized probabilities for the systematic bit and parity
 *  for decoder 1.
*******************************************************************************/
int Get_Final_APP_Dec1(time,fapp,parity)
int time;
double fapp[],parity[];
{
 int state,input,nxt_state,op;
 double ge,gi,prob,nxt_beta,prv_alpha;
 fapp[ZERO]=fapp[ONE]=parity[ZERO]=parity[ONE]=0.0;
 for(state=ZERO;state<NUM_STATES;state++)
 {
  for(input=ZERO;input<NUM_INPUTS;input++)
  {
   nxt_state=Trel_NS[state][input];
   op=Trel_Op[state][input];
   ge=Gamma_E_P1[time][op];                         /* Corresponds to parity. */
   gi=Gamma_I[time][input];                           /* Corresponds to data. */
   prob=Prob[time][input];
   prv_alpha=Alpha[time][state];
   nxt_beta=Beta[time+ONE][nxt_state];
   fapp[input]=fapp[input]+ge*gi*prv_alpha*nxt_beta*prob;
   parity[op]=parity[op]+ge*gi*prv_alpha*nxt_beta*prob;
  }
 }
 return(0);
}
/******************************************************************************/
/*   Get the final normalized probabilities at the channel interleaver output.
*******************************************************************************/
int Get_Final_Prob_Dec1(time,fapp,parity)
int time;
double fapp[],parity[];
{
 int input,ch_iv_op_inx;
 double sum;
 ch_iv_op_inx=Ch_Iv_Map[TWO*time];
 sum=0.0;
 for(input=ZERO;input<NUM_INPUTS;input++)
  sum=sum+fapp[input];
 for(input=ZERO;input<NUM_INPUTS;input++)
  Ch_Prob[ch_iv_op_inx][input]=fapp[input]/sum;
 if((time%TWO) == ZERO)
 {
  ch_iv_op_inx=Ch_Iv_Map[TWO*time+ONE];
  sum=0.0;
  for(input=ZERO;input<NUM_INPUTS;input++)
   sum=sum+parity[input];
  for(input=ZERO;input<NUM_INPUTS;input++)
   Ch_Prob[ch_iv_op_inx][input]=parity[input]/sum;
 }
 return(0);
}
/******************************************************************************/
/*         Compute the final APPs in the last iteration for decoder 2.
*******************************************************************************/
int Compute_Final_APPs_Dec2()
{
 int time;
 double parity[NUM_INPUTS];
 for(time=ONE;time<FRAME_SIZE;time=time+TWO)
 {
  Get_Final_APP_Dec2(time,parity);
  Get_Final_Prob_Dec2(time,parity);
 }
 return(0);
}
/******************************************************************************/
/*  Get the final unnormalized probabilities for the systematic bit and parity
 *  for decoder 2.
*******************************************************************************/
int Get_Final_APP_Dec2(time,parity)
int time;
double parity[];
{
 int state,input,nxt_state,op,iv_inx;
 double ge,gi,prob,nxt_beta,prv_alpha;
 parity[ZERO]=parity[ONE]=0.0;
 iv_inx=De_Iv_Map[time];
 for(state=ZERO;state<NUM_STATES;state++)
 {
  for(input=ZERO;input<NUM_INPUTS;input++)
  {
   nxt_state=Trel_NS[state][input];
   op=Trel_Op[state][input];
   ge=Gamma_E_P2[time][op];
   gi=Gamma_I[iv_inx][input];
   prob=Prob[iv_inx][input];
   prv_alpha=Alpha[time][state];
   nxt_beta=Beta[time+ONE][nxt_state];
   parity[op]=parity[op]+ge*gi*prv_alpha*nxt_beta*prob;
  }
 }
 return(0);
}
/******************************************************************************/
/*   Get the final normalized probabilities at the channel interleaver output.
*******************************************************************************/
int Get_Final_Prob_Dec2(time,parity)
int time;
double parity[];
{
 int input,ch_iv_op_inx;
 double sum;
 sum=0.0;
 ch_iv_op_inx=Ch_Iv_Map[TWO*time+ONE];
 for(input=ZERO;input<NUM_INPUTS;input++)
  sum=sum+parity[input];
 for(input=ZERO;input<NUM_INPUTS;input++)
  Ch_Prob[ch_iv_op_inx][input]=parity[input]/sum;
 return(0);
}
/******************************************************************************/
