/******************************************************************************/
/*   Copyright (C) 2005 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
#include "teq1type.h"
#include "teq1ext.h"
/******************************************************************************/
int Simulation()
{
 for(Frame_Cnt=ZERO;Frame_Cnt<MAX_FRAMES;Frame_Cnt++)
 {
  Get_Frame();
  Get_Channel_Op();
  Add_Noise();
  Receive();
 }
 return(0);
}
/******************************************************************************/
/*                  Get the samples at the channel output.
*******************************************************************************/
int Get_Channel_Op()
{
 int time,iv_ip_time,bit,ch_state;
 ch_state=ZERO;
 for(time=ZERO;time<CH_IV_SIZE;time++)
 {
  iv_ip_time=Ch_De_Iv_Map[time];
  bit=Enc_Op[iv_ip_time];
  Chan_Op[time]=Ch_Trel_Op[ch_state][bit];
  ch_state=Ch_Trel_NS[ch_state][bit];
 }
 Chan_F_State=ch_state;
 return(0);
}
/******************************************************************************/
/*                        Add white Gaussian noise.
*******************************************************************************/
int Add_Noise()
{
 int time;
 double erand48();
 for(time=ZERO;time<CH_IV_SIZE;time++)
 {
  N1_Val[URADIUS]=erand48(N1_Store);
  N1_Val[UTHETA]=erand48(N2_Store);
  My_Gauss(Variance,N1_Val);
  Chan_Op[time]=Chan_Op[time]+N1_Val[GRAND_0];
 }
 return(0);
}
/******************************************************************************/
/*                         Receiver routines.
*******************************************************************************/
int Receive()
{
 Init_Chan_Dec();
 Init_Chan_Prob();
 for(Outer_Cnt=ZERO;Outer_Cnt<MAX_OUTER;Outer_Cnt++)
 {
  Compute_Ch_APP();
  Init_Gamma_I_E_Turbo_Dec();
  Init_App();
  Bcjr();
//  Get_Frame_Error();
 }
 Get_Bit_Errors();
 return(0);
}
/******************************************************************************/
/*                      Initialize the channel decoder.
*******************************************************************************/
int Init_Chan_Dec()
{
 int time,state,ip;
 double x;
 for(time=ZERO;time<CH_IV_SIZE;time++)
 {
  for(state=ZERO;state<NUM_CH_STATES;state++)
  {
   for(ip=ZERO;ip<NUM_INPUTS;ip++)
   {
    x=Chan_Op[time]-Ch_Trel_Op[state][ip];
    x= -x*x*Inv_Twice_Noise_Var;
    Ch_Gamma[time][state][ip]=exp(x);
   }
  }
 }
 return(0);
}
/******************************************************************************/
/*            Get the a priori probabilities at the precoder input.
*******************************************************************************/
int Init_Chan_Prob()
{
 int time;
 for(time=ZERO;time<CH_IV_SIZE;time++)
  Ch_Prob[time][ZERO]=Ch_Prob[time][ONE]=0.5;
 return(0);
}
/******************************************************************************/
/*          Initialize the intrinsic and extrinsic values of gamma.
*******************************************************************************/
int Init_Gamma_I_E_Turbo_Dec()
{
 Init_Gamma_I_E();
// Init_Noisy_Gamma_I_E();
// Get_Ch_MAP_Op_Err();
 return(0);
}
/******************************************************************************/
/*  Initialize the intrinsic and extrinsic values of gamma for decoder 1 and
 *  two.
*******************************************************************************/
int Init_Gamma_I_E()
{
 int time,bit,ip_inx,p_inx;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  for(bit=ZERO;bit<NUM_INPUTS;bit++)
  {
   ip_inx=Ch_Iv_Map[TWO*time];
   p_inx=Ch_Iv_Map[TWO*time+ONE];
   Gamma_I[time][bit]=Ch_Prob[ip_inx][bit];
   if((time%TWO) == ZERO)
   {
    Gamma_E_P1[time][bit]=Ch_Prob[p_inx][bit];
    Gamma_E_P2[time][bit]=1.0;
   }
   else
   {
    Gamma_E_P1[time][bit]=1.0;
    Gamma_E_P2[time][bit]=Ch_Prob[p_inx][bit];
   }
  }
 }
 return(0);
}
/******************************************************************************/
/*        Get the errors in a frame at the channel MAP detector output.
*******************************************************************************/
int Get_Ch_MAP_Op_Err()
{
 int time,bit_err,dect_bit,p1_err,p2_err,p1_bit,p2_bit;
 p1_err=p2_err=bit_err=ZERO;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  p1_bit=p2_bit=dect_bit=ZERO;
  if(Gamma_I[time][ONE] > Gamma_I[time][ZERO])
   dect_bit=ONE;
  if(dect_bit != Bits[time])
   bit_err++;
  if((time%TWO) == ZERO)
  {
   if(Gamma_E_P1[time][ONE] > Gamma_E_P1[time][ZERO])
    p1_bit=ONE;
   if(p1_bit != Enc_Op[2*time+1])
    p1_err++;
  }
  else
  {
   if(Gamma_E_P2[time][ONE] > Gamma_E_P2[time][ZERO])
    p2_bit=ONE;
   if(p2_bit != Enc_Op[2*time+1])
    p2_err++;
  }
 }
 printf("fr:%d iperr:%d p1err:%d p2err:%d\n",Frame_Cnt,bit_err,p1_err,p2_err);
 return(0);
}
/******************************************************************************/
/*              Initialize the MAP decoder with noisy inputs.
*******************************************************************************/
int Init_Noisy_Gamma_I_E()
{
 int time;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  Init_Gamma_I(time);
  if((time%TWO) == ZERO)
   Init_Gamma_P1(time);
  else
   Init_Gamma_P2(time);
 }
 return(0);
}
/******************************************************************************/
/*                         Get noisy intrinsic inputs.
*******************************************************************************/
int Init_Gamma_I(time)
int time;
{
 int bit;
 double sample,x;
 double erand48();
 sample=Bit_Map[Enc_Op[TWO*time]];
 N1_Val[URADIUS]=erand48(N1_Store);
 N1_Val[UTHETA]=erand48(N2_Store);
 My_Gauss(Variance,N1_Val);
 sample=sample+N1_Val[GRAND_0];
 for(bit=ZERO;bit<NUM_INPUTS;bit++)
 {
  x=sample-Bit_Map[bit];
  x= -x*x*Inv_Twice_Noise_Var;
  Gamma_I[time][bit]=exp(x);
 }
 return(0);
}
/******************************************************************************/
/*                         Get noisy parity 1 inputs.
*******************************************************************************/
int Init_Gamma_P1(time)
int time;
{
 int bit;
 double sample,x;
 double erand48();
 sample=Bit_Map[Enc_Op[TWO*time+ONE]];
 N1_Val[URADIUS]=erand48(N1_Store);
 N1_Val[UTHETA]=erand48(N2_Store);
 My_Gauss(Variance,N1_Val);
 sample=sample+N1_Val[GRAND_0];
 for(bit=ZERO;bit<NUM_INPUTS;bit++)
 {
  x=sample-Bit_Map[bit];
  x= -x*x*Inv_Twice_Noise_Var;
  Gamma_E_P1[time][bit]=exp(x);
  Gamma_E_P2[time][bit]=1.0;
 }
 return(0);
}
/******************************************************************************/
/*                         Get noisy parity 2 inputs.
*******************************************************************************/
int Init_Gamma_P2(time)
int time;
{
 int bit;
 double sample,x;
 double erand48();
 sample=Bit_Map[Enc_Op[TWO*time+ONE]];
 N1_Val[URADIUS]=erand48(N1_Store);
 N1_Val[UTHETA]=erand48(N2_Store);
 My_Gauss(Variance,N1_Val);
 sample=sample+N1_Val[GRAND_0];
 for(bit=ZERO;bit<NUM_INPUTS;bit++)
 {
  x=sample-Bit_Map[bit];
  x= -x*x*Inv_Twice_Noise_Var;
  Gamma_E_P2[time][bit]=exp(x);
  Gamma_E_P1[time][bit]=1.0;
 }
 return(0);
}
/******************************************************************************/
/*                Initialize the a posteriori probabilities.
*******************************************************************************/
int Init_App()
{
 int time,input;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  for(input=ZERO;input<NUM_INPUTS;input++)
   Prob[time][input]=0.5;
 }
 return(0);
}
/******************************************************************************/
/*                          The BCJR algorithm.
*******************************************************************************/
int Bcjr()
{
 for(Inner_Cnt=ZERO;Inner_Cnt<MAX_INNER;Inner_Cnt++)
 {
  Dec2_Process();
  Dec1_Process();
 }
 return(0);
}
/******************************************************************************/
/*               Get the number of errors in the current frame.
*******************************************************************************/
int Get_Frame_Error()
{
 int time,bit,ip_inx,p_inx,bit_err,dect_bit,p1_err,p2_err,p1_bit,p2_bit;
 p1_err=p2_err=bit_err=ZERO;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  ip_inx=Ch_Iv_Map[TWO*time];
  p_inx=Ch_Iv_Map[TWO*time+ONE];
  p1_bit=p2_bit=dect_bit=ZERO;
  if(Ch_Prob[ip_inx][ONE] > Ch_Prob[ip_inx][ZERO])
   dect_bit=ONE;
  if(dect_bit != Enc_Op[TWO*time])
   bit_err++;
  if((time%TWO) == ZERO)
  {
   if(Ch_Prob[p_inx][ONE] > Ch_Prob[p_inx][ZERO])
    p1_bit=ONE;
   if(p1_bit != Enc_Op[TWO*time+ONE])
    p1_err++;
  }
  else
  {
   if(Ch_Prob[p_inx][ONE] > Ch_Prob[p_inx][ZERO])
    p2_bit=ONE;
   if(p2_bit != Enc_Op[TWO*time+ONE])
    p2_err++;
  }
 }
 printf("br:%d iperr:%d p1err:%d p2err:%d\n",Frame_Cnt,bit_err,p1_err,p2_err);
 return(0);
}
/******************************************************************************/
/*           Finally compute the number of bit errors in the frame.
*******************************************************************************/
int Get_Bit_Errors()
{
 int time,detected_bit,ch_iv_inx;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  ch_iv_inx=Ch_Iv_Map[TWO*time];
  detected_bit=ONE;
  if(Ch_Prob[ch_iv_inx][ZERO] > Ch_Prob[ch_iv_inx][ONE])
   detected_bit=ZERO;
  if(detected_bit != Bits[time])
   Ber++;
 }
 return(0);
}
/******************************************************************************/
/*                       Print out the probabilities.
*******************************************************************************/
int Print_Prob()
{
 int time,act_bit;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  act_bit=Bits[time];
  printf("Print prob Frame:%d time:%d tx bit:%d ",Frame_Cnt,time,act_bit);
  printf("Prob 0:%lf  ",Prob[time][0]);
  printf("Prob 1:%lf\n",Prob[time][1]);
 }
 return(0);
}
/******************************************************************************/
