/******************************************************************************/
/*   Copyright (C) 2005 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
#include "teq10type.h"
#include "teq10ext.h"
/******************************************************************************/
/*    Compute the a posteriori probabilities for the predictive MAP decoder.
*******************************************************************************/
int Compute_PID_APP()
{
 Init_Alpha_Beta_PID();
 Compute_Alpha_Beta_PID();
 Compute_APPLes_PID();
 return(0);
}
/******************************************************************************/
/*                          Initialize alpha and beta.
*******************************************************************************/
int Init_Alpha_Beta_PID()
{
 int state;
 for(state=ZERO;state<NUM_STATES;state++)
 {
  Ch_Alpha[ZERO][state]=1.0;
  Ch_Beta[CH_IV_SIZE][state]=1.0;
 }
 return(0);
}
/******************************************************************************/
/*                   Compute the alpha and beta values.
*******************************************************************************/
int Compute_Alpha_Beta_PID()
{
 int time,state;
 for(time=ZERO;time<=CH_IV_SIZE-TWO;time++)
 {
  for(state=ZERO;state<NUM_STATES;state++)
   Get_New_PID_Alpha_Beta(time,state);
  Normalize_PID_Alpha_Beta(time);
 }
 return(0);
}
/******************************************************************************/
/*      Compute the new alpha and beta values for a given state and time.
*******************************************************************************/
int Get_New_PID_Alpha_Beta(time,state)
int time,state;
{
 int input,prv_st,nxt_st,inxa,inxb;
 double ga,gb,proba,probb,prv_alpha,nxt_beta,alpha,beta;
 alpha=beta=0.0;
 for(input=ZERO;input<NUM_INPUTS;input++)                       /* Data bits. */
 {
  prv_st=Trel_PS[state][input];
  nxt_st=Trel_NS[state][input];
  prv_alpha=Ch_Alpha[time][prv_st];
  nxt_beta=Ch_Beta[CH_IV_SIZE-ONE-time+ONE][nxt_st];
  inxa=Get_Max_Gamma(prv_alpha,time,prv_st,input);
  inxb=Get_Max_Gamma(nxt_beta,CH_IV_SIZE-ONE-time,state,input);
  Ch_Alpha_Inx[time][prv_st][input]=inxa;
  ga=Ch_Gamma[time][prv_st][input][inxa];                 /* Gamma for alpha. */
  gb=Ch_Gamma[CH_IV_SIZE-ONE-time][state][input][inxb];    /* Gamma for beta. */
  proba=Ch_Prob[time][input];
  probb=Ch_Prob[CH_IV_SIZE-ONE-time][input];
  alpha=alpha+ga*proba*prv_alpha;
  beta=beta+gb*probb*nxt_beta;
 }
 Ch_Alpha[time+ONE][state]=alpha;
 Ch_Beta[CH_IV_SIZE-ONE-time][state]=beta;
 return(0);
}
/******************************************************************************/
/*           Compute the APPs due to extrinsic information alone.
*******************************************************************************/
int Compute_APPLes_PID()
{
 int time;
 double app[NUM_INPUTS];
 for(time=ZERO;time<CH_IV_SIZE;time++)
 {
  Get_APPLe_PID(time,app);
  Get_Prob_PID(time,app);
 }
 return(0);
}
/******************************************************************************/
/*   Get a posteriori probability of +1 and -1 due to extrinsic info alone.
*******************************************************************************/
int Get_APPLe_PID(time,app)
int time;
double app[];
{
 int state,input,nxt_state,inxa;
 double prv_alpha,nxt_beta,gamma;
 app[ZERO]=app[ONE]=0.0;
 for(state=ZERO;state<NUM_STATES;state++)
 {
  for(input=ZERO;input<NUM_INPUTS;input++)
  {
   nxt_state=Trel_NS[state][input];
   prv_alpha=Ch_Alpha[time][state];
   nxt_beta=Ch_Beta[time+ONE][nxt_state];
   inxa=Ch_Alpha_Inx[time][state][input];
   gamma=Ch_Gamma[time][state][input][inxa];
   app[input]=app[input]+prv_alpha*nxt_beta*gamma;
  }
 }
 return(0);
}
/******************************************************************************/
/*                     Get the normalized probabilities.
*******************************************************************************/
int Get_Prob_PID(time,app)
int time;
double app[];
{
 int input;
 double sum;
 sum=0.0;
 for(input=ZERO;input<NUM_INPUTS;input++)
  sum=sum+app[input];
 for(input=ZERO;input<NUM_INPUTS;input++)
  Ch_Prob[time][input]=app[input]/sum;
 return(0);
}
/******************************************************************************/
