/******************************************************************************/
/*   Copyright (C) 2005 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
#include "teq10type.h"
#include "teq10ext.h"
/******************************************************************************/
int Simulation()
{
 for(Frame_Cnt=ZERO;Frame_Cnt<MAX_FRAMES;Frame_Cnt++)
 {
  Get_Frame();
  if(Frame_Cnt > TWO)
   Step=0.05;
  Init_FB_Reg();
  ISI_Procedures();
  if(Frame_Cnt >= TRN_FRAMES)
   Receive();
 }
 return(0);
}
/******************************************************************************/
/*                     Initialize the feedback register.
*******************************************************************************/
int Init_FB_Reg()
{
 int cnt;
 for(cnt=ZERO;cnt<FB_SIZE;cnt++)
  Re_FB_Reg[cnt]=Im_FB_Reg[cnt]=1.0;
 return(0);
}
/******************************************************************************/
/*                             ISI procedures.
*******************************************************************************/
int ISI_Procedures()
{
 int time;
 Enc1_State=ZERO;
 for(time=ZERO;time<RX_DATA_END;time++)
 {
  Get_Channel_Op(time);
  Add_Noise();
  Get_Eq_Op();
  if((time>=RX_DATA_START) && (Frame_Cnt<TRN_FRAMES))
   Eq_Training();
  else if((time >= RX_DATA_START) && (Frame_Cnt >= TRN_FRAMES))
   Get_Noisy_Symbols(time);
 }
 return(0);
}
/******************************************************************************/
/*                  Get the samples at the channel output.
*******************************************************************************/
int Get_Channel_Op(time)
int time;
{
 int iv_ip_time,bit;
 if(time < PREAMBLE)
  bit=ZERO;
 else if(time < TX_DATA_END)
 {
  iv_ip_time=Ch_De_Iv_Map[time-PREAMBLE];
  bit=Enc_Op[iv_ip_time];
 }
 else
  bit=ZERO;
 Right_Shift_Reg(Re_Ch_Reg,Im_Ch_Reg,CHAN_SIZE);
 Shift_In_New_Ch_Inp(bit,time);
 Compute_Ch_Op();
 return(0);
}
/******************************************************************************/
/*                        Shift in the new channel input.
*******************************************************************************/
int Shift_In_New_Ch_Inp(bit,time)
int bit,time;
{
 int pty;
 pty=Trel_Op[Enc1_State][bit];
 Enc1_State=Trel_NS[Enc1_State][bit];
 Re_Ch_Reg[ZERO]=Bit_Map[bit];
 Im_Ch_Reg[ZERO]=Bit_Map[pty];
 if((time >= PREAMBLE) && (time < TX_DATA_END) && (Frame_Cnt < TRN_FRAMES))
 {
  if(Eq_Buf_Inx == EQ_BUF_SIZE)
   Error_Mes(13);
  Eq_Buf[Eq_Buf_Inx]=TWO*pty+bit;
  Eq_Buf_Inx++;
 }
 return(0);
}
/******************************************************************************/
/*                       Compute the channel output.
*******************************************************************************/
int Compute_Ch_Op()
{
 int cnt;
 Re_Ch_Op=Im_Ch_Op=0.0;
 for(cnt=ZERO;cnt<CHAN_SIZE;cnt++)
 {
  Re_Ch_Op=Re_Ch_Op+Re_Ch[cnt]*Re_Ch_Reg[cnt];
  Re_Ch_Op=Re_Ch_Op-Im_Ch[cnt]*Im_Ch_Reg[cnt];
  Im_Ch_Op=Im_Ch_Op+Re_Ch[cnt]*Im_Ch_Reg[cnt];
  Im_Ch_Op=Im_Ch_Op+Im_Ch[cnt]*Re_Ch_Reg[cnt];
 }
 return(0);
}
/******************************************************************************/
/*                        Add white Gaussian noise.
*******************************************************************************/
int Add_Noise()
{
 double erand48();
 N1_Val[URADIUS]=erand48(N1_Store);
 N1_Val[UTHETA]=erand48(N2_Store);
 My_Gauss(Variance,N1_Val);
 Re_Ch_Op=Re_Ch_Op+N1_Val[GRAND_0];
 Im_Ch_Op=Im_Ch_Op+N1_Val[GRAND_1];
 return(0);
}
/******************************************************************************/
/*                          Get equalizer output.
*******************************************************************************/
int Get_Eq_Op()
{
 int cnt;
 Right_Shift_Reg(Re_Eq_Reg,Im_Eq_Reg,EQ_SIZE);
 Re_Eq_Reg[ZERO]=Re_Ch_Op;
 Im_Eq_Reg[ZERO]=Im_Ch_Op;
 Re_Eq_Op=Im_Eq_Op=0.0;
 for(cnt=ZERO;cnt<EQ_SIZE;cnt++)
 {
  Re_Eq_Op=Re_Eq_Op+Re_Eq_Reg[cnt]*Re_Eq[cnt]-Im_Eq_Reg[cnt]*Im_Eq[cnt];
  Im_Eq_Op=Im_Eq_Op+Re_Eq_Reg[cnt]*Im_Eq[cnt]+Im_Eq_Reg[cnt]*Re_Eq[cnt];
 }
 return(0);
}
/******************************************************************************/
/*                 Get noisy symbols from equalizer output.
*******************************************************************************/
int Get_Noisy_Symbols(time)
int time;
{
 Rx_Re_Sym[time-RX_DATA_START]=Re_Eq_Op;
 Rx_Im_Sym[time-RX_DATA_START]=Im_Eq_Op;
 return(0);
}
/******************************************************************************/
/*                         Receiver routines.
*******************************************************************************/
int Receive()
{
 Init_PID_Dec();
 Init_PID_Prob();
 for(Outer_Cnt=ZERO;Outer_Cnt<MAX_OUTER;Outer_Cnt++)
 {
  Compute_PID_APP();
  Init_Gamma_Turbo_Dec();
  Get_Ch_MAP_Op_Err();
  Dec1_Process();
 }
 Get_Bit_Errors();
 return(0);
}
/******************************************************************************/
/*                      Initialize the channel decoder.
*******************************************************************************/
int Init_PID_Dec()
{
 int time,state,ip;
 for(time=ZERO;time<CH_IV_SIZE;time++)
 {
  for(state=ZERO;state<NUM_STATES;state++)
  {
   for(ip=ZERO;ip<NUM_INPUTS;ip++)
    Compute_PID_Gamma(time,state,ip);
  }
 }
 return(0);
}
/******************************************************************************/
/*             Compute the gammas for the predictive MAP decoder.
*******************************************************************************/
int Compute_PID_Gamma(time,state,ip)
int time,state,ip;
{
 int cnt,pty,mod_inp,bit;
 double x,re_sym,im_sym,re_sum,im_sum,re_err,im_err;
 for(mod_inp=ZERO;mod_inp<MOD_INPUTS;mod_inp++)
 {
  re_sum=im_sum=0.0;
  for(cnt=ZERO;cnt<FB_SIZE;cnt++)
  {
   bit=Trel_Ip_Array[state][mod_inp][cnt];
   pty=Trel_Pt_Array[state][mod_inp][cnt];
   re_sym=Bit_Map[bit];
   im_sym=Bit_Map[pty];
   re_sum=re_sum+re_sym*Re_FB[cnt]-im_sym*Im_FB[cnt];
   im_sum=im_sum+im_sym*Re_FB[cnt]+re_sym*Im_FB[cnt];
  }
  pty=Trel_Op[state][ip];
  re_err=Bit_Map[ip] -re_sum-Rx_Re_Sym[time];
  im_err=Bit_Map[pty]-im_sum-Rx_Im_Sym[time];
  x=re_err*re_err+im_err*im_err;
  x= -x*Inv_Twice_Eq_Var;
  Ch_Gamma[time][state][ip][mod_inp]=exp(x);
 }
 return(0);
}
/******************************************************************************/
/*            Get the a priori probabilities at the precoder input.
*******************************************************************************/
int Init_PID_Prob()
{
 int time;
 for(time=ZERO;time<CH_IV_SIZE;time++)
  Ch_Prob[time][ZERO]=Ch_Prob[time][ONE]=0.5;
 return(0);
}
/******************************************************************************/
/*          Initialize the intrinsic and extrinsic values of gamma.
*******************************************************************************/
int Init_Gamma_Turbo_Dec()
{
 Init_Gamma_I_E();
// Get_Ch_MAP_Op_Err();
 return(0);
}
/******************************************************************************/
/*  Initialize the intrinsic and extrinsic values of gamma for decoder 1 and
 *  two.
*******************************************************************************/
int Init_Gamma_I_E()
{
 int time,bit,d_inx,p_inx;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  d_inx=Ch_Iv_Map[TWO*time];
  p_inx=Ch_Iv_Map[TWO*time+ONE];
  for(bit=ZERO;bit<NUM_INPUTS;bit++)
  {
   Gamma_I[time][bit]=Ch_Prob[d_inx][bit];
   Gamma_P[time][bit]=Ch_Prob[p_inx][bit];
  }
 }
 return(0);
}
/******************************************************************************/
/*        Get the errors in a frame at the channel MAP detector output.
*******************************************************************************/
int Get_Ch_MAP_Op_Err()
{
 int time,d_err,p_err,d_bit,p_bit;
 d_err=p_err=ZERO;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  d_bit=p_bit=ZERO;
  if(Gamma_I[time][ONE] > Gamma_I[time][ZERO])
   d_bit=ONE;
  if(d_bit != Enc_Op[TWO*time])
   d_err++;
  if(Gamma_P[time][ONE] > Gamma_P[time][ZERO])
   p_bit=ONE;
  if(p_bit != Enc_Op[TWO*time+ONE])
   p_err++;
 }
 printf("fr:%d derr:%d perr:%d\n",Frame_Cnt,d_err,p_err);
 return(0);
}
/******************************************************************************/
/*               Get the number of errors in the current frame.
*******************************************************************************/
int Get_Frame_Error()
{
 int time,d_inx,p_inx,d_err,p_err,d_bit,p_bit;
 d_err=p_err=ZERO;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  d_inx=Ch_Iv_Map[TWO*time];
  p_inx=Ch_Iv_Map[TWO*time+ONE];
  d_bit=p_bit=ZERO;
  if(Ch_Prob[d_inx][ONE] > Ch_Prob[d_inx][ZERO])
   d_bit=ONE;
  if(d_bit != Enc_Op[TWO*time])
   d_err++;
  if(Ch_Prob[p_inx][ONE] > Ch_Prob[p_inx][ZERO])
   p_bit=ONE;
  if(p_bit != Enc_Op[TWO*time+ONE])
   p_err++;
 }
 printf("br:%d derr:%d perr:%d\n",Frame_Cnt,d_err,p_err);
 return(0);
}
/******************************************************************************/
/*           Finally compute the number of bit errors in the frame.
*******************************************************************************/
int Get_Bit_Errors()
{
 int time,detected_bit;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  detected_bit=ONE;
  if(Prob[time][ZERO] > Prob[time][ONE])
   detected_bit=ZERO;
  if(detected_bit != Enc_Op[TWO*time])
   Ber++;
 }
 return(0);
}
/******************************************************************************/
