/******************************************************************************/
/*   Copyright (C) 2005 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
#include "teq2type.h"
#include "teq2ext.h"
/******************************************************************************/
int Get_Channel_Iv_Map()
{
 int cnt;
 Init_0();
 First_Unmapped_Inx=ONE;                         /* The next unalloted index. */
 for(cnt=ZERO;cnt<S_VAL;cnt++)
 {
  Left_Process();
  Right_Process();
 }
 Print_Ch_Iv_Map();
 Get_Min_Cycle_Length();
 return(0);
}
/******************************************************************************/
/*                          Initialize index zero.
*******************************************************************************/
int Init_0()
{
 int mid_inx;
 mid_inx=(int)(CH_IV_SIZE/TWO);
 Ch_Iv_Map[ZERO]=mid_inx;
 Ch_De_Iv_Map[mid_inx]=ZERO;
 Left_Start_Inx=mid_inx-S_VAL;
 Right_Start_Inx=mid_inx+ONE;
 return(0);
}
/******************************************************************************/
/*                         Process the left hand side.
*******************************************************************************/
int Left_Process()
{
 int num_indices;
 if(First_Unmapped_Inx < CH_IV_SIZE)
 {
  num_indices=(int)(Left_Start_Inx/S_VAL)+ONE;
  Map_Left(num_indices);
  First_Unmapped_Inx=First_Unmapped_Inx+num_indices;
  Left_Start_Inx++;
 }
 return(0);
}
/******************************************************************************/
/*                 Map to the left part of the interleaver.
*******************************************************************************/
int Map_Left(num_indices)
int num_indices;
{
 int cnt,ck_iv_inx[CH_IV_SIZE],iv_map[CH_IV_SIZE];
 int inx,iv_inx;
 double iv_noise_val[CH_IV_SIZE];
 Get_Ch_Uniform_Noise(num_indices,iv_noise_val);
 Init_Ch_Ck_Iv_Inx(ck_iv_inx);
 Get_Ch_Iv_Elements(num_indices,iv_map,iv_noise_val,ck_iv_inx);
 for(cnt=ZERO;cnt<num_indices;cnt++)
 {
  inx=First_Unmapped_Inx+cnt;
  iv_inx=Left_Start_Inx-iv_map[cnt]*S_VAL;
  if(inx >= CH_IV_SIZE)
   Error_Mes(1);
  if(iv_inx < ZERO)
   Error_Mes(4);
  Ch_Iv_Map[inx]=iv_inx;
  Ch_De_Iv_Map[iv_inx]=inx;
 }
 return(0);
}
/******************************************************************************/
/*                         Process the right hand side.
*******************************************************************************/
int Right_Process()
{
 int num_indices;
 if(First_Unmapped_Inx < CH_IV_SIZE)
 {
  num_indices=(int)((CH_IV_SIZE-ONE-Right_Start_Inx)/S_VAL)+ONE;
  Map_Right(num_indices);
  First_Unmapped_Inx=First_Unmapped_Inx+num_indices;
  Right_Start_Inx++;
 }
 return(0);
}
/******************************************************************************/
/*                 Map to the right part of the interleaver.
*******************************************************************************/
int Map_Right(num_indices)
int num_indices;
{
 int cnt,ck_iv_inx[CH_IV_SIZE],iv_map[CH_IV_SIZE];
 int inx,iv_inx;
 double iv_noise_val[CH_IV_SIZE];
 Get_Ch_Uniform_Noise(num_indices,iv_noise_val);
 Init_Ch_Ck_Iv_Inx(ck_iv_inx);
 Get_Ch_Iv_Elements(num_indices,iv_map,iv_noise_val,ck_iv_inx);
 for(cnt=ZERO;cnt<num_indices;cnt++)
 {
  inx=First_Unmapped_Inx+cnt;
  iv_inx=Right_Start_Inx+iv_map[cnt]*S_VAL;
  if(inx >= CH_IV_SIZE)
   Error_Mes(1);
  if(iv_inx < ZERO)
   Error_Mes(4);
  Ch_Iv_Map[inx]=iv_inx;
  Ch_De_Iv_Map[iv_inx]=inx;
 }
 return(0);
}
/******************************************************************************/
/*                       Print the interleaver map.
*******************************************************************************/
int Print_Ch_Iv_Map()
{
 int cnt;
 printf("\t Channel interleaver map.\n\n");
 for(cnt=ZERO;cnt<CH_IV_SIZE;cnt++)
 {
  printf("\t\t Ch. Iv. map Index:%5d Map:%5d\n",cnt,Ch_Iv_Map[cnt]);
  printf("\t\t Ch. Dv. map Index:%5d Map:%5d\n",cnt,Ch_De_Iv_Map[cnt]);
 }
 printf("\n");
 fflush(stdout);
 return(0);
}
/******************************************************************************/
/*                        Get the minimum cycle length.
*******************************************************************************/
int Get_Min_Cycle_Length()
{
 int cnt1,cnt2,diff,min,cycle_len;
 min=10*CH_IV_SIZE;
 for(cnt1=ZERO;cnt1<CH_IV_SIZE;cnt1++)
 {
  for(cnt2=cnt1+ONE;cnt2<CH_IV_SIZE;cnt2++)
  {
   cycle_len=abs(cnt1-cnt2)+abs(Ch_Iv_Map[cnt1]-Ch_Iv_Map[cnt2]);
   if(cycle_len < min)
    min=cycle_len;
  }
 }
 printf("Min. cycle length:%d.\n\n",min);
 fflush(stdout);
 return(0);
}
/******************************************************************************/
