/******************************************************************************/
/*   Copyright (C) 2005 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
#include "teq2type.h"
#include "teq2ext.h"
/******************************************************************************/
/*                          Get the trellis.
*******************************************************************************/
int Get_Channel_Trellis()
{
 int p_state,p_input,index[NUM_CH_STATES];
 Init_Ch_Index(index);
 printf("\t Channel trellis:\n\n");
 for(p_state=ZERO;p_state<NUM_CH_STATES;p_state++)
 {
  for(p_input=ZERO;p_input<NUM_INPUTS;p_input++)
  {
   Get_Ch_Op(p_state,p_input);
   Get_Ch_Next_State(p_state,p_input,index);
  }
 }
 Print_Ch_Trellis();
 return(0);
}
/******************************************************************************/
/*                   Initialize the indices to each state.
*******************************************************************************/
int Init_Ch_Index(index)
int index[];
{
 int cnt;
 for(cnt=ZERO;cnt<NUM_CH_STATES;cnt++)
  index[cnt]=ZERO;
 return(0);
}
/******************************************************************************/
/*                          Get the parity bit.
*******************************************************************************/
int Get_Ch_Op(p_state,p_input)
int p_state,p_input;
{
 int bin_state[CHAN_MEM_SIZE],cnt,bit;
 double op;
 Decimal_to_Binary(p_state,bin_state,CHAN_MEM_SIZE);
 bit=p_input^bin_state[ZERO];
// bit=p_input;                                              /* No precoding. */
 op=Chan_Tap[ZERO]*Bit_Map[bit];
 for(cnt=ZERO;cnt<CHAN_MEM_SIZE;cnt++)
 {
  bit=bin_state[cnt];
  op=op+Chan_Tap[cnt+ONE]*Bit_Map[bit];
 }
 Ch_Trel_Op[p_state][p_input]=op;
 return(0);
}
/******************************************************************************/
/*                           Get the next state.
*******************************************************************************/
int Get_Ch_Next_State(p_state,p_input,index)
int p_state,p_input;
int index[];
{
 int bit,bin_state[CHAN_MEM_SIZE],cnt,new_state,inx;
 Decimal_to_Binary(p_state,bin_state,CHAN_MEM_SIZE);
 bit=p_input^bin_state[ZERO];
// bit=p_input;                                              /* No precoding. */
 for(cnt=(CHAN_MEM_SIZE-ONE);cnt>ZERO;cnt--)
  bin_state[cnt]=bin_state[cnt-ONE];
 bin_state[ZERO]=bit;
 new_state=Binary_to_Decimal(bin_state,CHAN_MEM_SIZE);
 inx=index[new_state]++;
 if(inx >= NUM_INPUTS)
  Error_Mes(1);
 Ch_Trel_NS[p_state][p_input]=new_state;
 Ch_Trel_PS[new_state][inx]=p_state;
 Ch_Trel_Ip[new_state][inx]=p_input;
 return(0);
}
/******************************************************************************/
/*                          Print the channel trellis.
*******************************************************************************/
int Print_Ch_Trellis()
{
 int p_state,p_input;
 for(p_state=ZERO;p_state<NUM_CH_STATES;p_state++)
 {
  for(p_input=ZERO;p_input<NUM_INPUTS;p_input++)
  {
   printf("\t St:%d Inp:%d ",p_state,p_input);
   printf("N. St:%d ",Ch_Trel_NS[p_state][p_input]);
   printf("Prv. St:%d ",Ch_Trel_PS[p_state][p_input]);
   printf("Input to st:%d ",Ch_Trel_Ip[p_state][p_input]);
   printf("Op:%+lf\n",Ch_Trel_Op[p_state][p_input]);
  }
 }
 printf("\n");
 return(0);
}
/******************************************************************************/
