/******************************************************************************/
#include "teq3type.h"
#include "teq3ext.h"
/******************************************************************************/
/*         Allocate memory for arrays used for defining the trellis.
*******************************************************************************/
int Trellis_Alloc()
{
 Nxt_State_Alloc();
 Re_Rx_Op_Alloc();
 Im_Rx_Op_Alloc();
 Sym_Map_Alloc();
 return(0);
}
/******************************************************************************/
/* Allocate memory for the array '**N_State' which defines the next state,
   given the current state and the current input.
*******************************************************************************/
int Nxt_State_Alloc()
{
 int cnt;
 void *tmp;
 void *calloc();
 tmp=calloc(Rx_States,sizeof(N_State));
 Ck_Calloc(tmp);
 N_State=tmp;
 for(cnt=ZERO;cnt<Rx_States;cnt++)
 {
  tmp=calloc(C_Size,sizeof(int));
  Ck_Calloc(tmp);
  N_State[cnt]=(int *)tmp;
 }
 return(0);
}
/******************************************************************************/
/*            Allocate memory for Re. part of resultant pulse output.
*******************************************************************************/
int Re_Rx_Op_Alloc()
{
 int samp_cnt,state_cnt;
 void *tmp;
 void *calloc();
 for(samp_cnt=ZERO;samp_cnt<S_PER_SYM;samp_cnt++)
 {
  tmp=calloc(Rx_States,sizeof(Re_Rx_Op[samp_cnt]));
  Ck_Calloc(tmp);
  Re_Rx_Op[samp_cnt]=tmp;
  for(state_cnt=ZERO;state_cnt<Rx_States;state_cnt++)
  {
   tmp=calloc(C_Size,sizeof(double));
   Ck_Calloc(tmp);
   Re_Rx_Op[samp_cnt][state_cnt]=(double *)tmp;
  }
 }
 return(0);
}
/******************************************************************************/
/*            Allocate memory for Im. part of resultant pulse output.
*******************************************************************************/
int Im_Rx_Op_Alloc()
{
 int samp_cnt,state_cnt;
 void *tmp;
 void *calloc();
 for(samp_cnt=ZERO;samp_cnt<S_PER_SYM;samp_cnt++)
 {
  tmp=calloc(Rx_States,sizeof(Im_Rx_Op[samp_cnt]));
  Ck_Calloc(tmp);
  Im_Rx_Op[samp_cnt]=tmp;
  for(state_cnt=ZERO;state_cnt<Rx_States;state_cnt++)
  {
   tmp=calloc(C_Size,sizeof(double));
   Ck_Calloc(tmp);
   Im_Rx_Op[samp_cnt][state_cnt]=(double *)tmp;
  }
 }
 return(0);
}
/******************************************************************************/
/* Allocate memory for storing symbols. Mapping is done by associating a
   number to a symbol value.
*******************************************************************************/
int Sym_Map_Alloc()
{
 void *tmp;
 void *calloc();
 tmp=calloc(C_Size,sizeof(double));
 Ck_Calloc(tmp);
 Re_Map=(double *)tmp;
 tmp=calloc(C_Size,sizeof(double));
 Ck_Calloc(tmp);
 Im_Map=(double *)tmp;
 return(0);
}
/******************************************************************************/
/*                 Check for memory allocation failure.
*******************************************************************************/
int Ck_Calloc(tmp)
void *tmp;
{
 if(tmp==NULL)
  Error_Mes(10);
 return(0);
}
/******************************************************************************/
