/******************************************************************************/
#include "teq3type.h"
#include "teq3ext.h"
/******************************************************************************/
/*                            Simulation.
*******************************************************************************/
int Simulate()
{
 for(Sym_Cnt=ZERO;Sym_Cnt<MAX_SYM;Sym_Cnt++)
 {
  Get_Sym();
  for(Ip_Cnt=ZERO;Ip_Cnt<S_PER_SYM;Ip_Cnt++)
  {
   Transmit();
   Add_Noise();
   Receive();
   Pad_Zeros();
  }
 }
 return(0);
}
/******************************************************************************/
/*                       Introduce channel distortion.
*******************************************************************************/
int Transmit()
{
 int cnt;
 Re_Ch_Op=Im_Ch_Op=0.0;
 Right_Shift(Re_Ch_Reg,Im_Ch_Reg,CHAN_SIZE);
 Re_Ch_Reg[ZERO]=Re_Sym;
 Im_Ch_Reg[ZERO]=Im_Sym;
 for(cnt=Ip_Cnt;cnt<CHAN_SIZE;cnt=cnt+S_PER_SYM)
 {
  Re_Ch_Op=Re_Ch_Op+Re_Ch_Tap[cnt]*Re_Ch_Reg[cnt];
  Re_Ch_Op=Re_Ch_Op-Im_Ch_Tap[cnt]*Im_Ch_Reg[cnt];
  Im_Ch_Op=Im_Ch_Op+Re_Ch_Tap[cnt]*Im_Ch_Reg[cnt];
  Im_Ch_Op=Im_Ch_Op+Im_Ch_Tap[cnt]*Re_Ch_Reg[cnt];
 }
 /*printf("Re_Ch_Op:%lf Im_Ch_Op:%lf\n",Re_Ch_Op,Im_Ch_Op);*/
 return(0);
}
/******************************************************************************/
/*                   Add complex gaussian noise to the sample.
*******************************************************************************/
int Add_Noise()
{
 double erand48();
 N_Val[URADIUS]=erand48(N1_Store);
 N_Val[UTHETA]=erand48(N2_Store);
 My_Gauss(Ch_Var,N_Val);
 if(Sym_Sync)                              /* Add noise only if synchronized. */
 {
  Re_Ch_Op=Re_Ch_Op+N_Val[GRAND_0];
  Im_Ch_Op=Im_Ch_Op+N_Val[GRAND_1];
 }
 /*printf("rgauss:%lf igauss:%lf\n",Re_N_Val[GRAND],Im_N_Val[GRAND]);*/
 return(0);
}
/******************************************************************************/
/*                       Process the received sample.
*******************************************************************************/
int Receive()
{
 Update_Counters();
 Re_VA_Ip[Samp_Cnt]=Re_Ch_Op;                  /* Re. sample input to the VA. */
 Im_VA_Ip[Samp_Cnt]=Im_Ch_Op;                  /* Im. sample input to the VA. */
 Get_VA_Output();
 return(0);
}
/******************************************************************************/
/*             Update sample count mod-S_PER_SYM and symbol count.
*******************************************************************************/
int Update_Counters()
{
 Samp_Cnt++;
 if(Samp_Cnt==S_PER_SYM)
 {
  Samp_Cnt=ZERO;
  if(Sym_Delay<TRN_PERIOD)
   Sym_Delay++;
 }
 /*printf("sampcnt:%d interpolation cnt:%d\n",Samp_Cnt,Ip_Cnt);*/
 return(0);
}
/******************************************************************************/
/*                     Get the output of the VA, if any.
*******************************************************************************/
int Get_VA_Output()
{
 if((Samp_Cnt==(S_PER_SYM-ONE))&&(Sym_Delay>=SYS_DELAY))
 {
  Viterbi();
  if(!Sym_Sync)
   Sync_Sym();
  if(Sym_Sync)                    /* Symbols at Tx. and Rx. are synchronized. */
   Sym_Error_Update();
 }
 return(0);
}
/******************************************************************************/
/*         Synchronize the symbols at the transmitter and receiver.
*******************************************************************************/
int Sync_Sym()
{
 int cnt;
 Sym_Sync=TRUE;
 Left_Shift(Sync_Buf,SYNC_SIZE);
 Sync_Buf[SYNC_SIZE-ONE]=VA_Op_Sym_Num;
 for(cnt=ZERO;cnt<SYNC_SIZE;cnt++)
 {
  if(Sync_Buf[cnt] != Err_Buf[cnt])
   Sym_Sync=FALSE;
 }
 if(Sym_Sync)
  printf("\t Sync established at symbol %d.\n\n",Sym_Cnt);
 return(0);
}
/******************************************************************************/
/*                        Update symbol error count.
*******************************************************************************/
int Sym_Error_Update()
{
 int cnt;
 if(Sym_Cnt >= TRN_PERIOD)
 {
  V_Sym_Cnt++;
  if(Err_Buf[SYNC_SIZE-ONE]!=VA_Op_Sym_Num)
   Sym_Err++;
 }
 for(cnt=ZERO;cnt<(Err_Buf_Inx-1);cnt++)
  Err_Buf[cnt]=Err_Buf[cnt+1];
 Err_Buf_Inx--;
 if(Err_Buf_Inx<ZERO)
  Error_Mes(18);
 return(0);
}
/******************************************************************************/
/*                        Pad zeros for interpolation.
*******************************************************************************/
int Pad_Zeros()
{
 Re_Sym=Im_Sym=0.0;                         /* Set the symbols equal to zero. */
 return(0);
}
/******************************************************************************/
