/******************************************************************************/
#include "teq3type.h"
#include "teq3ext.h"
/******************************************************************************/
/*                             Get the trellis.
*******************************************************************************/
int Get_Trellis()
{
 printf("Trellis structure:\n\n");
 Get_Constell_Size();
 Get_Num_States();
 Print_States();
 Trellis_Alloc();
 Read_Constellation();
 Populate_Trellis();
 return(0);
}
/******************************************************************************/
/*                 Get the size of the constellation.
*******************************************************************************/
int Get_Constell_Size()
{
 int cnt;
 C_Size=ONE;
 for(cnt=ZERO;cnt<B_PER_SYM;cnt++)
  C_Size=C_Size*2;
 if(C_Size==ONE)
  Error_Mes(9);
 printf("\t The constellation has %d symbols.\n\n",C_Size);
 return(0);
}
/******************************************************************************/
/* Get the number of states corresponding to transmitter and receiver trellis.
*******************************************************************************/
int Get_Num_States()
{
 int cnt;
 if(CHAN_SIZE%S_PER_SYM)
  Rx_Mem=(int)(CHAN_SIZE/S_PER_SYM);
 else
  Rx_Mem=(int)(CHAN_SIZE/S_PER_SYM)-ONE;
 if(Rx_Mem > MAX_MEM_SIZE)
  Error_Mes(22);
 VA_Depth=5*Rx_Mem;                                       /* @Remove the ONE. */
 if(Rx_Mem==ZERO)
  Error_Mes(8);
 Rx_States=ONE;
 for(cnt=ZERO;cnt<Rx_Mem;cnt++)
  Rx_States=Rx_States*C_Size;
 return(0);
}
/******************************************************************************/
/*         Print the number of states in transmitter and receiver.
*******************************************************************************/
int Print_States()
{
 printf("\t Received pulse has a memory of %d.\n\n",Rx_Mem);
 printf("\t The receiver trellis has %d states.\n\n",Rx_States);
 printf("\t Delay of the Viterbi Algorithm is %d symbols.\n\n",VA_Depth);
 return(0);
}
/******************************************************************************/
/*              Read the constellation from 'constell.dat'.
*******************************************************************************/
int Read_Constellation()
{
 int cnt;
 double retmp,imtmp;
 cnt=ZERO;
 printf("Symbol values in the constellation:\n\n");
 while(fscanf(fp3,"%lf %lf",&imtmp,&retmp)!=EOF)
 {
  if(cnt==C_Size)
   Error_Mes(12);
  Re_Map[cnt]=retmp;
  Im_Map[cnt]=imtmp;
  printf("\t Im:%+14.10lf Re:%+14.10lf\n",Im_Map[cnt],Re_Map[cnt]);
  cnt++;
 }
 if(cnt<C_Size)
  Error_Mes(13);
 printf("\n");
 return(0);
}
/******************************************************************************/
/*                        Populate the trellis.
*******************************************************************************/
int Populate_Trellis()
{
 int stcnt,ipcnt;
 int stmary[MAX_MEM_SIZE];
 for(stcnt=ZERO;stcnt<Rx_States;stcnt++)
 {
  Decimal_To_M_ary(stcnt,stmary);
  for(ipcnt=ZERO;ipcnt<C_Size;ipcnt++)
  {
   Get_Rx_Op(stcnt,ipcnt,stmary);
   Get_N_State(stcnt,ipcnt,stmary);
  }
 }
 return(0);
}
/******************************************************************************/
/* Convert the state given in decimal into modulo-C_Size and return
   the result in the array 'stmary[]'. The state is defined when the input
   is 'normal'.
*******************************************************************************/
int Decimal_To_M_ary(stcnt,stmary)
int stcnt,stmary[];
{
 int index;
 for(index=ZERO;index<Rx_Mem;index++)
  stmary[index]=ZERO;
 /*printf("State mod 10:%4d mod C_Size:",stcnt);*/
 index=ZERO;
 while(stcnt>ZERO)
 {
  stmary[index]=stcnt%C_Size;
  stcnt=stcnt/C_Size;
  index++;
 }
 /*for(index=(RXMEMSIZE-1);index>=ZERO;index--)
  printf("%2d",stmary[index]);
 printf("\n");*/
 return(0);
}
/******************************************************************************/
/*         Get the outputs of transmit filter and resultant pulse.
*******************************************************************************/
int Get_Rx_Op(stcnt,ipcnt,stmary)
int stcnt,ipcnt;
int stmary[];
{
 int samp_cnt;
 for(samp_cnt=ZERO;samp_cnt<S_PER_SYM;samp_cnt++)
 {
  Get_Rx_Samp_Op(stcnt,ipcnt,samp_cnt,stmary);
  Print_Op(stcnt,ipcnt,samp_cnt);
 }
 return(0);
}
/******************************************************************************/
/*              Get output of resultant pulse for each sample.
*******************************************************************************/
int Get_Rx_Samp_Op(stcnt,ipcnt,samp_cnt,stmary)
int stcnt,ipcnt,samp_cnt;
int stmary[];
{
 int cnt,map_inx,ch_inx;
 Re_Rx_Op[samp_cnt][stcnt][ipcnt]=0.0;
 Im_Rx_Op[samp_cnt][stcnt][ipcnt]=0.0;
 for(cnt=ZERO;cnt<Rx_Mem;cnt++)
 {
  map_inx=stmary[cnt];
  ch_inx=samp_cnt+(cnt+ONE)*S_PER_SYM;
  Get_Rx_Samp_Op1(stcnt,ipcnt,samp_cnt,map_inx,ch_inx);
 }
 Get_Rx_Samp_Op1(stcnt,ipcnt,samp_cnt,ipcnt,samp_cnt);
 return(0);
}
/******************************************************************************/
/*                      Get output due to a symbol.
*******************************************************************************/
int Get_Rx_Samp_Op1(stcnt,ipcnt,samp_cnt,sym_num,ch_inx)
int stcnt,ipcnt,samp_cnt,sym_num,ch_inx;
{
 double reop,imop;
 if(ch_inx < CHAN_SIZE)
 {
  reop=Re_Rx_Op[samp_cnt][stcnt][ipcnt];
  imop=Im_Rx_Op[samp_cnt][stcnt][ipcnt];
  reop=reop+Re_Map[sym_num]*Re_Ch_Tap[ch_inx];
  reop=reop-Im_Map[sym_num]*Im_Ch_Tap[ch_inx];
  imop=imop+Re_Map[sym_num]*Im_Ch_Tap[ch_inx];
  imop=imop+Im_Map[sym_num]*Re_Ch_Tap[ch_inx];
  Re_Rx_Op[samp_cnt][stcnt][ipcnt]=reop;
  Im_Rx_Op[samp_cnt][stcnt][ipcnt]=imop;
 }
 return(0);
}
/******************************************************************************/
/*       Print the outputs of transmit and receive (resultant) filter.
*******************************************************************************/
Print_Op(stcnt,ipcnt,samp_cnt)
int stcnt,ipcnt,samp_cnt;
{
 printf("\t Samp.:%-3d  ",samp_cnt);
 printf("Re. Rx. Op.:%+14.8lf  ",Re_Rx_Op[samp_cnt][stcnt][ipcnt]);
 printf("Im. Rx. Op.:%+14.8lf\n",Im_Rx_Op[samp_cnt][stcnt][ipcnt]);
 return(0);
}
/******************************************************************************/
/* Given the current state and input get the next state without disturbing
   the input array 'stmary[]'.
*******************************************************************************/
int Get_N_State(stcnt,ipcnt,stmary)
int stcnt,ipcnt;
int stmary[];
{
 int cnt,decimal,factor;
 decimal=ipcnt;
 factor=C_Size;
 for(cnt=ZERO;cnt<(Rx_Mem-1);cnt++)
 {
  decimal=decimal+stmary[cnt]*factor;
  factor=factor*C_Size;
 }
 N_State[stcnt][ipcnt]=decimal;
 printf("\t Pre. state:%-3d  input:%-3d ",stcnt,ipcnt);
 printf("   Nxt. state:%-3d\n\n",N_State[stcnt][ipcnt]);
 return(0);
}
/******************************************************************************/
