/******************************************************************************/
#include <stdio.h>
#include <math.h>
/******************************************************************************/
/*   TIME (IN SYMBOLS) REQUIRED FOR THE SYSTEM TO GIVE STEADY-STATE OUTPUT.
*******************************************************************************/
#define SYS_DELAY 50
/******************************************************************************/
/*    TIME (IN SYMBOLS) REQUIRED TO TRAIN AND SYNCHRONIZE THE RECEIVER.
*******************************************************************************/
#define SEG1 (2*SYS_DELAY)
#define TRN_PERIOD (SEG1+SYS_DELAY)
#define SYNC_SIZE 16
/******************************************************************************/
/*                             SIMULATION RUN.
*******************************************************************************/
#define MAX_SYM (TRN_PERIOD+10000000)
/******************************************************************************/
/*                    PARAMETERS RELATED TO THE SOURCE.
*******************************************************************************/
#define B_PER_SYM 1                             /* Number of bits per symbol. */
#define S_PER_SYM 1                                    /* Samples per symbol. */
/******************************************************************************/
/*                     PARAMETERS RELATED TO CHANNEL.
*******************************************************************************/
#define CHAN_SIZE 3                                   /* Size of the channel. */
/******************************************************************************/
/*            PARAMETERS RELATED TO GAUSSIAN NOISE GENERATOR.
*******************************************************************************/
#define PNS_SIZE 3                     /* Size of array used to generate PNS. */
#define SNR  (-1.00)                           /* Signal-to-noise ratio (dB). */
#define MAX_SNR 100.0                                    /* Maximum SNR (dB). */
#define NVALSIZE 4
#define URADIUS 0
#define UTHETA 1
#define GRAND_0 2
#define GRAND_1 3
/******************************************************************************/
/*                           STRUCTURE OF TRELLIS.
*******************************************************************************/
struct Trellis                                       /* Structure of trellis. */
{
 int *St_Path;                                   /* Surviving path of states. */
 int *Ip_Path;                                   /* Surviving path of inputs. */
 double *VA_Wght;                      /* Metrics associated with each state. */
 struct Trellis *Prv_Depth;                     /* Address of previous depth. */
 struct Trellis *Nxt_Depth;                         /* Address of next depth. */
};
/******************************************************************************/
/* PARAMETERS RELATED TO THE BUFFER AT TRANSMITTER. THIS IS USED TO STORE
   SYMBOLS SO THAT THEY CAN BE COMPARED WITH THE (DELAYED) SYMBOLS RELEASED
   BY THE RECEIVER, FOR THE PURPOSE OF ERROR CHECKING.
*******************************************************************************/
#define ERR_BUF_SIZE 64
/******************************************************************************/
/*                PARAMETERS RELATED TO VITERBI ALGORITHM.
*******************************************************************************/
#define THRESH_WGHT (1000000000000.0)                    /* Threshold weight. */
/******************************************************************************/
/*                          GENERAL DEFINITIONS.
*******************************************************************************/
#define TWO 2
#define ONE 1
#define ZERO 0
#define TRUE 1
#define FALSE 0
#define LEFT 0
#define RIGHT 1
#define MINUSONE (-1)
#define MINUSTWO (-2)
#define MAX_MEM_SIZE 16
#define ERROR 0.000001
#define PI 3.14159265358979323846
#define PIBY2 1.57079632679489661923
#define TWICEPI 6.28318530717958647692
/******************************************************************************/
