/******************************************************************************/
#include "teq3type.h"
#include "teq3ext.h"
/******************************************************************************/
/* Calculate the metric associated with each state for the current time
   instant.
*******************************************************************************/
int Calculate_Metric()
{
 if(Lst_Depth==NULL)
  Error_Mes(16);
 for(VA_P_St=ZERO;VA_P_St<Rx_States;VA_P_St++)
 {
  for(VA_Inp=ZERO;VA_Inp<C_Size;VA_Inp++)
  {
   VA_N_St=N_State[VA_P_St][VA_Inp];
   Get_Curr_Metric();
   Get_Survivor();
  }
 }
 return(0);
}
/******************************************************************************/
/*                     Get the current branch metric.
*******************************************************************************/
int Get_Curr_Metric()
{
 int samp_cnt;
 double rinp,iinp,rrxop,irxop,sum;
 Curr_Metric=0.0;
 for(samp_cnt=ZERO;samp_cnt<S_PER_SYM;samp_cnt++)
 {
  rinp=Re_VA_Ip[samp_cnt];
  iinp=Im_VA_Ip[samp_cnt];
  rrxop=Re_Rx_Op[samp_cnt][VA_P_St][VA_Inp];
  irxop=Im_Rx_Op[samp_cnt][VA_P_St][VA_Inp];
  sum=(rinp-rrxop)*(rinp-rrxop)+(iinp-irxop)*(iinp-irxop);
  Curr_Metric=Curr_Metric+sum;
  /*printf("VA inp Re:%lf Im:%lf\n",rinp,iinp);*/
 }
 /*printf("pst:%d inp:%d cur wght:%lf\n",VA_P_St,VA_Inp,Curr_Metric);*/
 return(0);
}
/******************************************************************************/
/*                   Update the survivor to the next state.
*******************************************************************************/
int Get_Survivor()
{
 double owght,nwght,ewght;
 struct Trellis *prev;
 prev=Lst_Depth->Prv_Depth;
 if(prev!=NULL)
  owght=prev->VA_Wght[VA_P_St];         /* Weight of state at previous depth. */
 else                         /* This happens when 'Lst_Depth' = 'Fst_Depth'. */
  owght=0.0;
 nwght=owght+Curr_Metric;            /* Prospective new metric at next state. */
 ewght=Lst_Depth->VA_Wght[VA_N_St];         /* Existing metric at next state. */
 if(nwght < ewght)
  Survivor_Info(nwght);              /* Store information about the survivor. */
 return(0);
}
/******************************************************************************/
/*                    Get survivor for the next state.
*******************************************************************************/
int Survivor_Info(nwght)
double nwght;
{
 Lst_Depth->VA_Wght[VA_N_St]=nwght;
 Lst_Depth->St_Path[VA_N_St]=VA_P_St;
 Lst_Depth->Ip_Path[VA_N_St]=VA_Inp;
 /*printf("PS:%d inp:%d NS:%d tot:%lf\n",VA_P_St,VA_Inp,VA_N_St,Total_Metric);*/
 return(0);
}
/******************************************************************************/
/*              Get the state with associated minimum metric.
*******************************************************************************/
int Get_Min_Metric()
{
 int cnt;
 double min;
 Min_Wght_St=ZERO;
 min=Lst_Depth->VA_Wght[ZERO];
 for(cnt=ONE;cnt<Rx_States;cnt++)
 {
  if(Lst_Depth->VA_Wght[cnt]<min)
  {
   min=Lst_Depth->VA_Wght[cnt];
   Min_Wght_St=cnt;
  }
 }
 return(0);
}
/******************************************************************************/
/*                           Print the metrics.
*******************************************************************************/
int Print_Metrics()
{
 int cnt;
 for(cnt=ZERO;cnt<Rx_States;cnt++)
  printf("Sym_Cnt:%d st:%d wght:%lf\n",Sym_Cnt,cnt,Lst_Depth->VA_Wght[cnt]);
 printf("State with max wght:%d\n",Min_Wght_St);
 return(0);
}
/******************************************************************************/
/*                Go backwards and get the last symbol in time.
*******************************************************************************/
int Get_Last_Symbol()
{
 int tstate,tip;
 struct Trellis *curr;
 curr=Lst_Depth;
 tstate=Min_Wght_St;
 while(curr!=NULL)
 {
  tip=curr->Ip_Path[tstate];
  tstate=curr->St_Path[tstate];
  curr=curr->Prv_Depth;
 }
 VA_Op_Sym_Num=tip;                             /* This is the symbol number. */
 return(0);
}
/******************************************************************************/
/*                    Normalize the metrics at 'Lst_Depth'.
*******************************************************************************/
int Normalize_Metrics()
{
 int cnt;
 double twght;
 twght=Lst_Depth->VA_Wght[Min_Wght_St];
 for(cnt=ZERO;cnt<Rx_States;cnt++)
  Lst_Depth->VA_Wght[cnt]=Lst_Depth->VA_Wght[cnt]-twght;
 return(0);
}
/******************************************************************************/
/*                       Reinitialize Linked List.
*******************************************************************************/
int Linked_List_Reinit()
{
 struct Trellis *tnext;
 tnext=Fst_Depth->Nxt_Depth;
 tnext->Prv_Depth=NULL;
 Lst_Depth->Nxt_Depth=Fst_Depth;
 Fst_Depth->Prv_Depth=Lst_Depth;
 Fst_Depth->Nxt_Depth=NULL;
 Lst_Depth=Fst_Depth;
 Fst_Depth=tnext;
 return(0);
}
/******************************************************************************/
/*               Reinitialize metrics at the last depth.
*******************************************************************************/
int Metric_Reinit()
{
 int cnt;
 for(cnt=ZERO;cnt<Rx_States;cnt++)
  Lst_Depth->VA_Wght[cnt]= THRESH_WGHT;
 return(0);
}
/******************************************************************************/
