/******************************************************************************/
/*   Copyright (C) 2005 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
#include "teq4type.h"
#include "teq4ext.h"
/******************************************************************************/
/*                   Get an S-random channel interleaver.
*******************************************************************************/
int Get_S_Rand_Channel_Iv_Map()
{
 int flag[CH_IV_SIZE],siv_flag[CH_IV_SIZE],siv[CH_IV_SIZE];
 Init_Siv_Flag(siv,flag,siv_flag);
 Get_Siv(siv,flag,siv_flag);
 Print_S_Rand_Ch_Iv_Map(siv,siv_flag);
 return(0);
}
/******************************************************************************/
/*                          Initialize the flag.
*******************************************************************************/
int Init_Siv_Flag(siv,flag,siv_flag)
int siv[],flag[],siv_flag[];
{
 int cnt;
 for(cnt=ZERO;cnt<CH_IV_SIZE;cnt++)
  siv[cnt]=flag[cnt]=siv_flag[cnt]=FALSE;
 return(0);
}
/******************************************************************************/
/*                       Get the S-random interleaver.
*******************************************************************************/
int Get_Siv(siv,flag,siv_flag)
int siv[],flag[],siv_flag[];
{
 int time,level;
 for(time=ZERO;time<CH_IV_SIZE;time++)
 {
  level=Level1_Check(time,siv,flag,siv_flag);
  if(level == TWO)
   Get_Any_Index(time,siv,flag,siv_flag);
 }
 return(0);
}
/******************************************************************************/
/*                 Do a level 1 search for a suitable index.
*******************************************************************************/
int Level1_Check(time,siv,flag,siv_flag)
int time,siv[],flag[],siv_flag[];
{
 int cnt1,inx,found,level;
 found=FALSE;
 for(cnt1=ZERO;(cnt1 < CH_IV_SIZE) && (found == FALSE);cnt1++)
 {
  if(flag[cnt1] == FALSE)
  {
   inx=Ch_Iv_Map[cnt1];
   found=Level1_Check_Index(time,inx,siv);
   if(found == TRUE)
    Update_Siv(ONE,inx,cnt1,time,siv,flag,siv_flag);
  }
 }
 if(found == FALSE)                                      /* Go to next level. */
  level=TWO;
 else
  level=ONE;
 return(level);
}
/******************************************************************************/
/*                       Verify if the index is valid.
*******************************************************************************/
int Level1_Check_Index(time,inx,siv)
int time,inx,siv[];
{
 int found,ctr,dist;
 found=TRUE;
 for(ctr=time-ONE;(ctr >= ZERO)&&(ctr > (time-S_VAL))&&(found == TRUE);ctr--)
 {
  dist=abs(siv[ctr]-inx);
  if(dist < S_VAL)
   found=FALSE;
 }
 return(found);
}
/******************************************************************************/
/*                  Get any index without any constraints.
*******************************************************************************/
int Get_Any_Index(time,siv,flag,siv_flag)
int time,siv[],flag[],siv_flag[];
{
 int cnt1,inx,found,level;
 found=FALSE;
 for(cnt1=ZERO;(cnt1 < CH_IV_SIZE) && (found == FALSE);cnt1++)
 {
  if(flag[cnt1] == FALSE)
  {
   inx=Ch_Iv_Map[cnt1];
   found=TRUE;
   Update_Siv(TWO,inx,cnt1,time,siv,flag,siv_flag);
  }
 }
 return(0);
}
/******************************************************************************/
/*                 Update the S-random interleaver array.
*******************************************************************************/
int Update_Siv(level,inx,cnt1,time,siv,flag,siv_flag)
int level,inx,cnt1,time,siv[],flag[],siv_flag[];
{
 siv[time]=inx;
 siv_flag[time]=level;
 flag[cnt1]=TRUE;
// printf("time:%d siv:%d cnt1:%d flag:%d\n",time,siv[time],cnt1,flag[cnt1]);
// fflush(stdout);
 return(0);
}
/******************************************************************************/
/*                Print the S-random channel interleaver map.
*******************************************************************************/
int Print_S_Rand_Ch_Iv_Map(siv,siv_flag)
int siv[],siv_flag[];
{
 int cnt,inx;
 printf("\t %d-random channel interleaver map.\n\n",S_VAL);
 for(cnt=ZERO;cnt<CH_IV_SIZE;cnt++)
 {
  inx=siv[cnt];
  Ch_Iv_Map[cnt]=inx;
  Ch_De_Iv_Map[inx]=cnt;
  printf("\t\t %d Iv. map Inx:%5d Map:%5d ",S_VAL,cnt,Ch_Iv_Map[cnt]);
  printf("Flag:%d\n",siv_flag[cnt]);
 }
 printf("\n");
 fflush(stdout);
 return(0);
}
/******************************************************************************/
