/******************************************************************************/
/*   Copyright (C) 2005 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
#include "teq5type.h"
#include "teq5ext.h"
/******************************************************************************/
/*    Compute the a posteriori probabilities for the channel MAP decoder.
*******************************************************************************/
int Compute_Ch_APP()
{
 Init_Alpha_Beta_Ch();
 Compute_Alpha_Beta_Ch();
 if(Outer_Cnt < (MAX_OUTER-ONE))
  Compute_APPLes_Ch();
 else
  Compute_Final_APPLes_Ch();
 return(0);
}
/******************************************************************************/
/*                          Initialize alpha and beta.
*******************************************************************************/
int Init_Alpha_Beta_Ch()
{
 int state;
 for(state=ZERO;state<NUM_CH_STATES;state++)
 {
  Ch_Alpha[ZERO][state]=0.0;
  Ch_Beta[CH_IV_SIZE][state]=0.0;
 }
 Ch_Alpha[ZERO][ZERO]=1.0;
 Ch_Beta[CH_IV_SIZE][Chan_F_State]=1.0;
 return(0);
}
/******************************************************************************/
/*                   Compute the alpha and beta values.
*******************************************************************************/
int Compute_Alpha_Beta_Ch()
{
 int time,state;
 for(time=ZERO;time<=CH_IV_SIZE-TWO;time++)
 {
  for(state=ZERO;state<NUM_CH_STATES;state++)
   Get_New_Ch_Alpha_Beta(time,state);
  Normalize_Ch_Alpha_Beta(time);
 }
 return(0);
}
/******************************************************************************/
/*      Compute the new alpha and beta values for a given state and time.
*******************************************************************************/
int Get_New_Ch_Alpha_Beta(time,state)
int time,state;
{
 int input,prv_st,nxt_st,st_ip;
 double grea,gima,greb,gimb,reproba,improba,reprobb,improbb;
 double prv_alpha,nxt_beta,alpha,beta;
 alpha=beta=0.0;
 for(input=ZERO;input<NUM_CH_INPUTS;input++)                    /* Data bits. */
 {
  prv_st=Ch_Trel_PS[state][input];
  nxt_st=Ch_Trel_NS[state][input];
  st_ip=Ch_Trel_Ip[state];                                 /* Input to state. */
  grea=Re_Ch_Gamma[time][prv_st][st_ip];                  /* Gamma for alpha. */
  gima=Im_Ch_Gamma[time][prv_st][st_ip];                  /* Gamma for alpha. */
  greb=Re_Ch_Gamma[CH_IV_SIZE-ONE-time][state][input];     /* Gamma for beta. */
  gimb=Im_Ch_Gamma[CH_IV_SIZE-ONE-time][state][input];     /* Gamma for beta. */
  reproba=Ch_Prob[TWO*time][st_ip%TWO];
  improba=Ch_Prob[TWO*time+ONE][st_ip/TWO];
  reprobb=Ch_Prob[TWO*(CH_IV_SIZE-ONE-time)][input%TWO];
  improbb=Ch_Prob[TWO*(CH_IV_SIZE-ONE-time)+ONE][input/TWO];
  prv_alpha=Ch_Alpha[time][prv_st];
  nxt_beta=Ch_Beta[CH_IV_SIZE-ONE-time+ONE][nxt_st];
  alpha=alpha+grea*gima*reproba*improba*prv_alpha;
  beta=beta+greb*gimb*reprobb*improbb*nxt_beta;
 }
 Ch_Alpha[time+ONE][state]=alpha;
 Ch_Beta[CH_IV_SIZE-ONE-time][state]=beta;
 return(0);
}
/******************************************************************************/
/*           Compute the APPs due to extrinsic information alone.
*******************************************************************************/
int Compute_APPLes_Ch()
{
 int time;
 double reapp[NUM_INPUTS],imapp[NUM_INPUTS];
 for(time=ZERO;time<CH_IV_SIZE;time++)
 {
  Get_APPLe_Ch(time,reapp,imapp);
  Get_Prob_Ch(time,reapp,imapp);
 }
 return(0);
}
/******************************************************************************/
/*   Get a posteriori probability of +1 and -1 due to extrinsic info alone.
*******************************************************************************/
int Get_APPLe_Ch(time,reapp,imapp)
int time;
double reapp[],imapp[];
{
 int state,input,nxt_state;
 double prv_alpha,nxt_beta,gre,gim;
 reapp[ZERO]=reapp[ONE]=imapp[ZERO]=imapp[ONE]=0.0;
 for(state=ZERO;state<NUM_CH_STATES;state++)
 {
  for(input=ZERO;input<NUM_CH_INPUTS;input++)
  {
   nxt_state=Ch_Trel_NS[state][input];
   prv_alpha=Ch_Alpha[time][state];
   nxt_beta=Ch_Beta[time+ONE][nxt_state];
   gre=Re_Ch_Gamma[time][state][input];
   gim=Im_Ch_Gamma[time][state][input];
   reapp[input%TWO]=reapp[input%TWO]+prv_alpha*nxt_beta*gre*gim;
   imapp[input/TWO]=imapp[input/TWO]+prv_alpha*nxt_beta*gre*gim;
  }
 }
 return(0);
}
/******************************************************************************/
/*                     Get the normalized probabilities.
*******************************************************************************/
int Get_Prob_Ch(time,reapp,imapp)
int time;
double reapp[],imapp[];
{
 int input;
 double resum,imsum;
 resum=imsum=0.0;
 for(input=ZERO;input<NUM_INPUTS;input++)
 {
  resum=resum+reapp[input];
  imsum=imsum+imapp[input];
 }
 for(input=ZERO;input<NUM_INPUTS;input++)
 {
  Ch_Prob[TWO*time][input]=reapp[input]/resum;
  Ch_Prob[TWO*time+ONE][input]=imapp[input]/imsum;
 }
 return(0);
}
/******************************************************************************/
