/******************************************************************************/
/*   Copyright (C) 2005 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
#include "teq5type.h"
#include "teq5ext.h"
/******************************************************************************/
/*         Compute the final APPs in the last iteration for decoder 1.
*******************************************************************************/
int Compute_Final_APPs_Dec1()
{
 int time;
 double fapp[NUM_INPUTS],parity[NUM_INPUTS];
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  Get_Final_APP_Dec1(time,fapp,parity);
  Get_Final_Prob_Dec1(time,fapp,parity);
 }
 return(0);
}
/******************************************************************************/
/*  Get the final unnormalized probabilities for the systematic bit and parity
 *  for decoder 1.
*******************************************************************************/
int Get_Final_APP_Dec1(time,fapp,parity)
int time;
double fapp[],parity[];
{
 int state,input,nxt_state,op;
 double ge,gi,prob,nxt_beta,prv_alpha;
 fapp[ZERO]=fapp[ONE]=parity[ZERO]=parity[ONE]=0.0;
 for(state=ZERO;state<NUM_STATES;state++)
 {
  for(input=ZERO;input<NUM_INPUTS;input++)
  {
   nxt_state=Trel_NS[state][input];
   op=Trel_Op[state][input];
   ge=Gamma_E_P1[time][op];                         /* Corresponds to parity. */
   gi=Gamma_I1[time][input];                          /* Corresponds to data. */
   prob=Prob[time][input];
   prv_alpha=Alpha[time][state];
   nxt_beta=Beta[time+ONE][nxt_state];
   fapp[input]=fapp[input]+ge*prv_alpha*nxt_beta;
   parity[op]=parity[op]+gi*prv_alpha*nxt_beta;
  }
 }
 return(0);
}
/******************************************************************************/
/*   Get the final normalized probabilities at the channel interleaver output.
*******************************************************************************/
int Get_Final_Prob_Dec1(time,fapp,parity)
int time;
double fapp[],parity[];
{
 int input,inx0,inx1;
 double sum0,sum1;
 inx0=Bit_Ch_Iv_Map[FOUR*time];
 inx1=Bit_Ch_Iv_Map[FOUR*time+ONE];
 sum0=sum1=0.0;
 for(input=ZERO;input<NUM_INPUTS;input++)
 {
  sum0=sum0+fapp[input];
  sum1=sum1+parity[input];
 }
 for(input=ZERO;input<NUM_INPUTS;input++)
 {
  Ch_Prob[inx0][input]=fapp[input]/sum0;
  Ch_Prob[inx1][input]=parity[input]/sum1;
 }
 return(0);
}
/******************************************************************************/
/*         Compute the final APPs in the last iteration for decoder 2.
*******************************************************************************/
int Compute_Final_APPs_Dec2()
{
 int time;
 double fapp[NUM_INPUTS],parity[NUM_INPUTS];
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  Get_Final_APP_Dec2(time,fapp,parity);
  Get_Final_Prob_Dec2(time,fapp,parity);
 }
 return(0);
}
/******************************************************************************/
/*  Get the final unnormalized probabilities for the systematic bit and parity
 *  for decoder 2.
*******************************************************************************/
int Get_Final_APP_Dec2(time,fapp,parity)
int time;
double fapp[],parity[];
{
 int state,input,nxt_state,op,iv_inx;
 double ge,gi,prob,nxt_beta,prv_alpha;
 fapp[ZERO]=fapp[ONE]=parity[ZERO]=parity[ONE]=0.0;
 iv_inx=De_Iv_Map[time];
 for(state=ZERO;state<NUM_STATES;state++)
 {
  for(input=ZERO;input<NUM_INPUTS;input++)
  {
   nxt_state=Trel_NS[state][input];
   op=Trel_Op[state][input];
   ge=Gamma_E_P2[time][op];
   gi=Gamma_I2[time][input];
   prob=Prob[iv_inx][input];
   prv_alpha=Alpha[time][state];
   nxt_beta=Beta[time+ONE][nxt_state];
   fapp[input]=fapp[input]+ge*prv_alpha*nxt_beta;
   parity[op]=parity[op]+gi*prv_alpha*nxt_beta;
  }
 }
 return(0);
}
/******************************************************************************/
/*   Get the final normalized probabilities at the channel interleaver output.
*******************************************************************************/
int Get_Final_Prob_Dec2(time,fapp,parity)
int time;
double fapp[],parity[];
{
 int input,inx0,inx1;
 double sum0,sum1;
 inx0=Bit_Ch_Iv_Map[FOUR*time+TWO];
 inx1=Bit_Ch_Iv_Map[FOUR*time+THREE];
 sum0=sum1=0.0;
 for(input=ZERO;input<NUM_INPUTS;input++)
 {
  sum0=sum0+fapp[input];
  sum1=sum1+parity[input];
 }
 for(input=ZERO;input<NUM_INPUTS;input++)
 {
  Ch_Prob[inx0][input]=fapp[input]/sum0;
  Ch_Prob[inx1][input]=parity[input]/sum1;
 }
 return(0);
}
/******************************************************************************/
