/******************************************************************************/
/*   Copyright (C) 2005 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
#include "teq5type.h"
#include "teq5ext.h"
/******************************************************************************/
int Simulation()
{
 for(Frame_Cnt=ZERO;Frame_Cnt<MAX_FRAMES;Frame_Cnt++)
 {
  Get_Frame();
  Get_Channel_Op();
  Add_Noise();
  Receive();
 }
 return(0);
}
/******************************************************************************/
/*                  Get the samples at the channel output.
*******************************************************************************/
int Get_Channel_Op()
{
 int time,inx0,inx1,bit0,bit1,ch_state;
 ch_state=ZERO;
 for(time=ZERO;time<BIT_CH_IV_SIZE;time=time+TWO)
 {
  inx0=Bit_Ch_De_Iv_Map[time];
  inx1=Bit_Ch_De_Iv_Map[time+ONE];
  bit0=Enc_Op[inx0];
  bit1=Enc_Op[inx1];
  Re_Chan_Op[time/TWO]=Re_Ch_Trel_Op[ch_state][TWO*bit1+bit0];
  Im_Chan_Op[time/TWO]=Im_Ch_Trel_Op[ch_state][TWO*bit1+bit0];
  ch_state=Ch_Trel_NS[ch_state][TWO*bit1+bit0];
 }
 Chan_F_State=ch_state;
 return(0);
}
/******************************************************************************/
/*                        Add white Gaussian noise.
*******************************************************************************/
int Add_Noise()
{
 int time;
 double erand48();
 for(time=ZERO;time<CH_IV_SIZE;time++)
 {
  N1_Val[URADIUS]=erand48(N1_Store);
  N1_Val[UTHETA]=erand48(N2_Store);
  My_Gauss(Variance,N1_Val);
  Re_Chan_Op[time]=Re_Chan_Op[time]+N1_Val[GRAND_0];
  Im_Chan_Op[time]=Im_Chan_Op[time]+N1_Val[GRAND_1];
 }
 return(0);
}
/******************************************************************************/
/*                         Receiver routines.
*******************************************************************************/
int Receive()
{
 Init_Chan_Dec();
 Init_Chan_Prob();
 for(Outer_Cnt=ZERO;Outer_Cnt<MAX_OUTER;Outer_Cnt++)
 {
  Compute_Ch_APP();
  Init_Gamma_I_E_Turbo_Dec();
  Init_App();
  Bcjr();
//  Get_Frame_Error();
 }
 Get_Bit_Errors();
 return(0);
}
/******************************************************************************/
/*                      Initialize the channel decoder.
*******************************************************************************/
int Init_Chan_Dec()
{
 int time,state,ip;
 double re,im;
 for(time=ZERO;time<CH_IV_SIZE;time++)
 {
  for(state=ZERO;state<NUM_CH_STATES;state++)
  {
   for(ip=ZERO;ip<NUM_CH_INPUTS;ip++)
   {
    re=Re_Chan_Op[time]-Re_Ch_Trel_Op[state][ip];
    im=Im_Chan_Op[time]-Im_Ch_Trel_Op[state][ip];
    re= -re*re*Inv_Twice_Noise_Var;
    im= -im*im*Inv_Twice_Noise_Var;
    Re_Ch_Gamma[time][state][ip]=exp(re);
    Im_Ch_Gamma[time][state][ip]=exp(im);
   }
  }
 }
 return(0);
}
/******************************************************************************/
/*            Get the a priori probabilities at the precoder input.
*******************************************************************************/
int Init_Chan_Prob()
{
 int time;
 for(time=ZERO;time<BIT_CH_IV_SIZE;time++)
  Ch_Prob[time][ZERO]=Ch_Prob[time][ONE]=0.5;
 return(0);
}
/******************************************************************************/
/*          Initialize the intrinsic and extrinsic values of gamma.
*******************************************************************************/
int Init_Gamma_I_E_Turbo_Dec()
{
 Init_Gamma_I_E();
// Init_Noisy_Gamma_I_E();
// Get_Ch_MAP_Op_Err();
 return(0);
}
/******************************************************************************/
/*  Initialize the intrinsic and extrinsic values of gamma for decoder 1 and
 *  2.
*******************************************************************************/
int Init_Gamma_I_E()
{
 int time,bit,inx0,inx1;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  for(bit=ZERO;bit<NUM_INPUTS;bit++)
  {
   inx0=Bit_Ch_Iv_Map[FOUR*time];
   inx1=Bit_Ch_Iv_Map[FOUR*time+ONE];
   Gamma_I1[time][bit]=Ch_Prob[inx0][bit];
   Gamma_E_P1[time][bit]=Ch_Prob[inx1][bit];
   inx0=Bit_Ch_Iv_Map[FOUR*time+TWO];
   inx1=Bit_Ch_Iv_Map[FOUR*time+THREE];
   Gamma_I2[time][bit]=Ch_Prob[inx0][bit];
   Gamma_E_P2[time][bit]=Ch_Prob[inx1][bit];
  }
 }
 return(0);
}
/******************************************************************************/
/*        Get the errors in a frame at the channel MAP detector output.
*******************************************************************************/
int Get_Ch_MAP_Op_Err()
{
 int time,bit_err,dect_bit,p1_err,p2_err,p1_bit,p2_bit;
 p1_err=p2_err=bit_err=ZERO;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  p1_bit=p2_bit=dect_bit=ZERO;
  if(Gamma_I1[time][ONE] > Gamma_I1[time][ZERO])
   dect_bit=ONE;
  if(dect_bit != Bits[time])
   bit_err++;
  if(Gamma_E_P1[time][ONE] > Gamma_E_P1[time][ZERO])
   p1_bit=ONE;
  if(p1_bit != Enc_Op[FOUR*time+ONE])
   p1_err++;
  if(Gamma_E_P2[time][ONE] > Gamma_E_P2[time][ZERO])
   p2_bit=ONE;
  if(p2_bit != Enc_Op[FOUR*time+THREE])
   p2_err++;
 }
 printf("fr:%d iperr:%d p1err:%d p2err:%d\n",Frame_Cnt,bit_err,p1_err,p2_err);
 return(0);
}
/******************************************************************************/
/*              Initialize the MAP decoder with noisy inputs.
*******************************************************************************/
int Init_Noisy_Gamma_I_E()
{
 int time;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  Init_Gamma_I(time);
  Init_Gamma_P1(time);
  Init_Gamma_P2(time);
 }
 return(0);
}
/******************************************************************************/
/*                         Get noisy intrinsic inputs.
*******************************************************************************/
int Init_Gamma_I(time)
int time;
{
 int bit;
 double sample0,sample1,x,y;
 double erand48();
 sample0=Bit_Map[Enc_Op[FOUR*time]];
 sample1=Bit_Map[Enc_Op[FOUR*time+TWO]];
 N1_Val[URADIUS]=erand48(N1_Store);
 N1_Val[UTHETA]=erand48(N2_Store);
 My_Gauss(Variance,N1_Val);
 sample0=sample0+N1_Val[GRAND_0];
 sample1=sample1+N1_Val[GRAND_1];
 for(bit=ZERO;bit<NUM_INPUTS;bit++)
 {
  x=sample0-Bit_Map[bit];
  y=sample1-Bit_Map[bit];
  x= -x*x*Inv_Twice_Noise_Var;
  y= -y*y*Inv_Twice_Noise_Var;
  Gamma_I1[time][bit]=exp(x);
  Gamma_I2[time][bit]=exp(y);
 }
 return(0);
}
/******************************************************************************/
/*                         Get noisy parity 1 inputs.
*******************************************************************************/
int Init_Gamma_P1(time)
int time;
{
 int bit;
 double sample,x;
 double erand48();
 sample=Bit_Map[Enc_Op[FOUR*time+ONE]];
 N1_Val[URADIUS]=erand48(N1_Store);
 N1_Val[UTHETA]=erand48(N2_Store);
 My_Gauss(Variance,N1_Val);
 sample=sample+N1_Val[GRAND_0];
 for(bit=ZERO;bit<NUM_INPUTS;bit++)
 {
  x=sample-Bit_Map[bit];
  x= -x*x*Inv_Twice_Noise_Var;
  Gamma_E_P1[time][bit]=exp(x);
 }
 return(0);
}
/******************************************************************************/
/*                         Get noisy parity 2 inputs.
*******************************************************************************/
int Init_Gamma_P2(time)
int time;
{
 int bit;
 double sample,x;
 double erand48();
 sample=Bit_Map[Enc_Op[FOUR*time+THREE]];
 N1_Val[URADIUS]=erand48(N1_Store);
 N1_Val[UTHETA]=erand48(N2_Store);
 My_Gauss(Variance,N1_Val);
 sample=sample+N1_Val[GRAND_0];
 for(bit=ZERO;bit<NUM_INPUTS;bit++)
 {
  x=sample-Bit_Map[bit];
  x= -x*x*Inv_Twice_Noise_Var;
  Gamma_E_P2[time][bit]=exp(x);
 }
 return(0);
}
/******************************************************************************/
/*                Initialize the a posteriori probabilities.
*******************************************************************************/
int Init_App()
{
 int time,input;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  for(input=ZERO;input<NUM_INPUTS;input++)
   Prob[time][input]=0.5;
 }
 return(0);
}
/******************************************************************************/
/*                          The BCJR algorithm.
*******************************************************************************/
int Bcjr()
{
 for(Inner_Cnt=ZERO;Inner_Cnt<MAX_INNER;Inner_Cnt++)
 {
  Dec2_Process();
  Dec1_Process();
 }
 return(0);
}
/******************************************************************************/
/*               Get the number of errors in the current frame.
*******************************************************************************/
int Get_Frame_Error()
{
 int time,bit,inx0,inx1,inx2,bit_err,dect_bit,p1_err,p2_err,p1_bit,p2_bit;
 p1_err=p2_err=bit_err=ZERO;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  inx0=Bit_Ch_Iv_Map[FOUR*time];
  inx1=Bit_Ch_Iv_Map[FOUR*time+ONE];
  inx2=Bit_Ch_Iv_Map[FOUR*time+THREE];
  p1_bit=p2_bit=dect_bit=ZERO;
  if(Ch_Prob[inx0][ONE] > Ch_Prob[inx0][ZERO])
   dect_bit=ONE;
  if(dect_bit != Enc_Op[FOUR*time])
   bit_err++;
  if(Ch_Prob[inx1][ONE] > Ch_Prob[inx1][ZERO])
   p1_bit=ONE;
  if(p1_bit != Enc_Op[FOUR*time+ONE])
   p1_err++;
  if(Ch_Prob[inx2][ONE] > Ch_Prob[inx2][ZERO])
   p2_bit=ONE;
  if(p2_bit != Enc_Op[FOUR*time+THREE])
   p2_err++;
 }
 printf("br:%d iperr:%d p1err:%d p2err:%d\n",Frame_Cnt,bit_err,p1_err,p2_err);
 return(0);
}
/******************************************************************************/
/*           Finally compute the number of bit errors in the frame.
*******************************************************************************/
int Get_Bit_Errors()
{
 int time,detected_bit,iv_inx;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  iv_inx=Bit_Ch_Iv_Map[FOUR*time];
  detected_bit=ONE;
  if(Ch_Prob[iv_inx][ZERO] > Ch_Prob[iv_inx][ONE])
   detected_bit=ZERO;
  if(detected_bit != Bits[time])
   Ber++;
 }
 return(0);
}
/******************************************************************************/
/*                       Print out the probabilities.
*******************************************************************************/
int Print_Prob()
{
 int time,act_bit;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  act_bit=Bits[time];
  printf("Print prob Frame:%d time:%d tx bit:%d ",Frame_Cnt,time,act_bit);
  printf("Prob 0:%lf  ",Prob[time][0]);
  printf("Prob 1:%lf\n",Prob[time][1]);
 }
 return(0);
}
/******************************************************************************/
