/******************************************************************************/
/*   Copyright (C) 2005 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
#include "teq8type.h"
#include "teq8ext.h"
/******************************************************************************/
int Eq_Training()
{
 Get_FB_Op();
 Get_Eq_Err();
 if((Frame_Cnt < (TRN_FRAMES-ONE)) && (IDEAL_EQ == ZERO))
  Update_Eq();
 Update_FB_Reg();
 if(Frame_Cnt==(TRN_FRAMES-ONE))
  Collect_Data();
 return(0);
}
/******************************************************************************/
/*                     Get the output of feedback part.
*******************************************************************************/
int Get_FB_Op()
{
 int cnt;
 Re_FB_Op=Im_FB_Op=0.0;
 for(cnt=ZERO;cnt<FB_SIZE;cnt++)
 {
  Re_FB_Op=Re_FB_Op+Re_FB[cnt]*Re_FB_Reg[cnt]-Im_FB[cnt]*Im_FB_Reg[cnt];
  Im_FB_Op=Im_FB_Op+Re_FB[cnt]*Im_FB_Reg[cnt]+Im_FB[cnt]*Re_FB_Reg[cnt];
 }
 return(0);
}
/******************************************************************************/
/*                         Get equalizer error.
*******************************************************************************/
int Get_Eq_Err()
{
 int bit,pty,cnt;
 double err_sq;
 bit=Eq_Buf[ZERO]%TWO;
 pty=Eq_Buf[ZERO]/TWO;
 if(IDEAL_EQ == ZERO)
 {
  Re_Eq_Err=Bit_Map[bit]-Re_Eq_Op-Re_FB_Op;
  Im_Eq_Err=Bit_Map[pty]-Im_Eq_Op-Im_FB_Op;
 }
 else
 {
  Re_Eq_Err=Bit_Map[bit]-Re_Eq_Op+Re_FB_Op;
  Im_Eq_Err=Bit_Map[pty]-Im_Eq_Op+Im_FB_Op;
 }
 err_sq=Re_Eq_Err*Re_Eq_Err+Im_Eq_Err*Im_Eq_Err;
 if(Frame_Cnt == (TRN_FRAMES-ONE))
  fprintf(fp3,"%lf\n",err_sq);
 return(0);
}
/******************************************************************************/
/*                Update the equalizer coefficients.
*******************************************************************************/
int Update_Eq()
{
 Update_FF_Eq();
 Update_FB_Eq();
 return(0);
}
/******************************************************************************/
/*                   Update the feedforward equalizer taps.
*******************************************************************************/
int Update_FF_Eq()
{
 int cnt;
 Ip_Pow=0.0;
 for(cnt=ZERO;cnt<EQ_SIZE;cnt++)
  Ip_Pow=Ip_Pow+Re_Eq_Reg[cnt]*Re_Eq_Reg[cnt]+Im_Eq_Reg[cnt]*Im_Eq_Reg[cnt];
 for(cnt=ZERO;cnt<EQ_SIZE;cnt++)
 {
  Re_Grad=Re_Eq_Err*Re_Eq_Reg[cnt]+Im_Eq_Err*Im_Eq_Reg[cnt];
  Im_Grad=Im_Eq_Err*Re_Eq_Reg[cnt]-Re_Eq_Err*Im_Eq_Reg[cnt];
  if(FIXED_STEP == ZERO)
  {
   Re_Eq[cnt]=Re_Eq[cnt]+Step*Re_Grad/Ip_Pow;
   Im_Eq[cnt]=Im_Eq[cnt]+Step*Im_Grad/Ip_Pow;
  }
  else
  {
   Re_Eq[cnt]=Re_Eq[cnt]+Step*Re_Grad/(double)(TWO*(EQ_SIZE+FB_SIZE));
   Im_Eq[cnt]=Im_Eq[cnt]+Step*Im_Grad/(double)(TWO*(EQ_SIZE+FB_SIZE));
  }
 }
 return(0);
}
/******************************************************************************/
/*                   Update the feedback equalizer taps.
*******************************************************************************/
int Update_FB_Eq()
{
 int cnt;
 Ip_Pow=0.0;
 for(cnt=ZERO;cnt<FB_SIZE;cnt++)
  Ip_Pow=Ip_Pow+Re_FB_Reg[cnt]*Re_FB_Reg[cnt]+Im_FB_Reg[cnt]*Im_FB_Reg[cnt];
 for(cnt=ZERO;cnt<FB_SIZE;cnt++)
 {
  Re_Grad=Re_Eq_Err*Re_FB_Reg[cnt]+Im_Eq_Err*Im_FB_Reg[cnt];
  Im_Grad=Im_Eq_Err*Re_FB_Reg[cnt]-Re_Eq_Err*Im_FB_Reg[cnt];
  if(FIXED_STEP == ZERO)
  {
   Re_FB[cnt]=Re_FB[cnt]+Step*Re_Grad/Ip_Pow;
   Im_FB[cnt]=Im_FB[cnt]+Step*Im_Grad/Ip_Pow;
  }
  else
  {
   Re_FB[cnt]=Re_FB[cnt]+Step*Re_Grad/(double)(TWO*(EQ_SIZE+FB_SIZE));
   Im_FB[cnt]=Im_FB[cnt]+Step*Im_Grad/(double)(TWO*(EQ_SIZE+FB_SIZE));
  }
 }
 return(0);
}
/******************************************************************************/
/*                      Update the feedback registers.
*******************************************************************************/
int Update_FB_Reg()
{
 int bit,pty,cnt;
 bit=Eq_Buf[ZERO]%TWO;
 pty=Eq_Buf[ZERO]/TWO;
 Right_Shift_Reg(Re_FB_Reg,Im_FB_Reg,FB_SIZE);
 Re_FB_Reg[ZERO]=Bit_Map[bit];
 Im_FB_Reg[ZERO]=Bit_Map[pty];
 for(cnt=ZERO;cnt<Eq_Buf_Inx-ONE;cnt++)
  Eq_Buf[cnt]=Eq_Buf[cnt+ONE];
 Eq_Buf_Inx--;
 if(Eq_Buf_Inx < ZERO)
  Error_Mes(12);
 return(0);
}
/******************************************************************************/
/*         Collect the data for updating the predictor coefficients.
*******************************************************************************/
int Collect_Data()
{
 if(Err_Reg_Inx < DATA_SIZE)
 {
  Re_Err_Reg[Err_Reg_Inx]=Re_Eq_Err;
  Im_Err_Reg[Err_Reg_Inx]=Im_Eq_Err;
  Err_Reg_Inx++;
 }
 if(Err_Reg_Inx == DATA_SIZE)
 {
  Compute_Eq_Var();
  Print_Norm_Eq();
  Print_Norm_FB();
  Err_Reg_Inx++;
 }
 return(0);
}
/******************************************************************************/
/*                   Compute the equalizer output variance.
*******************************************************************************/
int Compute_Eq_Var()
{
 int cnt;
 double var;
 var=0.0;
 for(cnt=ZERO;cnt<DATA_SIZE;cnt++)
  var=var+Re_Err_Reg[cnt]*Re_Err_Reg[cnt]+Im_Err_Reg[cnt]*Im_Err_Reg[cnt];
 var=0.5*var/(double)DATA_SIZE;
 Inv_Twice_Eq_Var=1.0/(2.0*var);
 printf("\t Eq. error variance:%lf.\n",var);
 printf("\t Inv. of twice the eq. error variance:%lf.\n\n",Inv_Twice_Eq_Var);
 return(0);
}
/******************************************************************************/
/*               Print the normalized equalizer coefficients.
*******************************************************************************/
int Print_Norm_Eq()
{
 int cnt;
 double energy,fac;
 energy=0.0;
 for(cnt=ZERO;cnt<EQ_SIZE;cnt++)
 {
  energy=energy+Re_Eq[cnt]*Re_Eq[cnt];
  energy=energy+Im_Eq[cnt]*Im_Eq[cnt];
 }
 fac=1.0/sqrt(energy);
 for(cnt=ZERO;cnt<EQ_SIZE;cnt++)
  fprintf(fp4,"%lf %lf\n",fac*Im_Eq[cnt],fac*Re_Eq[cnt]);
 return(0);
}
/******************************************************************************/
/*               Print the normalized feedback equalizer coefficients.
*******************************************************************************/
int Print_Norm_FB()
{
 int cnt;
 double energy,fac;
 energy=0.0;
 for(cnt=ZERO;cnt<FB_SIZE;cnt++)
 {
  energy=energy+Re_FB[cnt]*Re_FB[cnt];
  energy=energy+Im_FB[cnt]*Im_FB[cnt];
 }
 fac=1.0/sqrt(energy);
 for(cnt=ZERO;cnt<FB_SIZE;cnt++)
  fprintf(fp5,"%lf %lf\n",fac*Im_FB[cnt],fac*Re_FB[cnt]);
 return(0);
}
/******************************************************************************/
