/******************************************************************************/
/*   Copyright (C) 2005 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
FILE *fp1,*fp2,*fp3,*fp4,*fp5,*fp6;                         /* File pointers. */
/******************************************************************************/
/*                                BIT VARIABLES.
*******************************************************************************/
int Frame_Cnt;                                                /* Frame count. */
int Ber;                                                   /* Bit error rate. */
/******************************************************************************/
/*                             ENCODER VARIABLES.
*******************************************************************************/
int Enc_State;                                     /* State of outer encoder. */
int Enc1_State;                                    /* State of inner encoder. */
int Enc_Op[CH_IV_SIZE];                                       /* Encoder o/p. */
double P_Sym[CH_IV_SIZE];                   /* Parity symbols at encoder o/p. */
/******************************************************************************/
/*                       MAP TO CONSTELLATION VARIABLES.
*******************************************************************************/
double Bit_Map[CONSTELL_SIZE];                       /* Map to constellation. */
/******************************************************************************/
/*                 VARIABLES RELATED TO CHANNEL INTERLEAVER.
*******************************************************************************/
int Ch_Iv_Map[CH_IV_SIZE];                            /* Interleaver mapping. */
int Ch_De_Iv_Map[CH_IV_SIZE];                       /* Deinterleaver mapping. */
unsigned short Iv_Noise_Store[PN_SIZE];                 /* Stores PNS status. */
/******************************************************************************/
/*                       NOISE GENERATOR VARIABLES.
*******************************************************************************/
unsigned short N1_Store[PN_SIZE];                       /* Stores PNS status. */
unsigned short N2_Store[PN_SIZE];                       /* Stores PNS status. */
double Variance;                                           /* Noise variance. */
double Inv_Twice_Noise_Var;           /* Inverse of twice the noise variance. */
double N1_Val[NVALSIZE];                         /* Contains U & G rand. var. */
/******************************************************************************/
/*                    VARIABLES RELATED TO THE TRELLIS.
*******************************************************************************/
int Enc_N[ENC_SIZE];                           /* Stores encoder numer. taps. */
int Enc_D[ENC_SIZE];                           /* Stores encoder denom. taps. */
int Trel_NS[NUM_STATES][NUM_INPUTS];                           /* Next state. */
int Trel_PS[NUM_STATES][NUM_INPUTS];                           /* Prev state. */
int Trel_Op[NUM_STATES][NUM_INPUTS];                              /* Outputs. */
/******************************************************************************/
/*                  VARIABLES RELATED TO THE SUPER TRELLIS.
*******************************************************************************/
int STrel_NS[NUM_ST_STATES][NUM_INPUTS];          /* Super trellis nxt state. */
int STrel_PS[NUM_ST_STATES][NUM_INPUTS];          /* Super trellis prv state. */
int STrel_Ip[NUM_ST_STATES][NUM_INPUTS];         /* Inputs leading to states. */
int STrel_Ip_Array[NUM_ST_STATES][NUM_INPUTS][FB_SIZE_P1];             /* Ip. */
int STrel_Pt_Array[NUM_ST_STATES][NUM_INPUTS][FB_SIZE_P1];             /* Pt. */
/******************************************************************************/
/*                     VARIABLES RELATED TO THE CHANNEL.
*******************************************************************************/
double Re_Ch[CHAN_SIZE];                         /* Re. channel coefficients. */
double Im_Ch[CHAN_SIZE];                         /* Im. channel coefficients. */
double Re_Ch_Reg[CHAN_SIZE];                     /* Re. channel coefficients. */
double Im_Ch_Reg[CHAN_SIZE];                     /* Im. channel coefficients. */
double Re_Ch_Op;                                       /* Re. channel output. */
double Im_Ch_Op;                                       /* Im. channel output. */
/******************************************************************************/
/*                            EQUALIZER VARIABLES.
*******************************************************************************/
int Eq_Buf_Inx;                              /* Equalizer error buffer index. */
int Eq_Buf[EQ_BUF_SIZE];                           /* Equalizer error buffer. */
double Ip_Pow;                        /* Instantaneous equalizer input power. */
double Step;                                                    /* Step size. */
double Re_Grad;                                      /* Re. part of gradient. */
double Im_Grad;                                      /* Im. part of gradient. */
double Re_Eq_Op;                                        /* Re. equalizer o/p. */
double Im_Eq_Op;                                        /* Im. equalizer o/p. */
double Re_Eq_Err;                                     /* Re. equalizer error. */
double Im_Eq_Err;                                     /* Im. equalizer error. */
double Re_Eq[EQ_SIZE];                                /* Re. equalizer coeff. */
double Im_Eq[EQ_SIZE];                                /* Im. equalizer coeff. */
double Re_Eq_Reg[EQ_SIZE];                              /* Re. equalizer mem. */
double Im_Eq_Reg[EQ_SIZE];                              /* Im. equalizer mem. */
double Re_FB[FB_SIZE];                                    /* Re.feedback tap. */
double Im_FB[FB_SIZE];                                    /* Im.feedback tap. */
double Re_FB_Reg[FB_SIZE];                               /* Re. feedback reg. */
double Im_FB_Reg[FB_SIZE];                               /* Im. feedback reg. */
double Re_FB_Op;                                  /* Re. op of feedback part. */
double Im_FB_Op;                                  /* Im. op of feedback part. */
/******************************************************************************/
/*             VARIABLES RELATED TO THE EQUALIZER OUTPUT ERROR.
*******************************************************************************/
int Err_Reg_Inx;                                  /* Index to error register. */
double Inv_Twice_Eq_Var;                   /* Inv. of twice the eq. err. var. */
double Re_Err_Reg[DATA_SIZE];                       /* Re. eq. error samples. */
double Im_Err_Reg[DATA_SIZE];                       /* Im. eq. error samples. */
/******************************************************************************/
/*                            RECEIVED SYMBOLS.
*******************************************************************************/
double Rx_Re_Sym[CH_IV_SIZE];                                 /* Rx. re. sym. */
double Rx_Im_Sym[CH_IV_SIZE];                                 /* Rx. im. sym. */
/******************************************************************************/
/*                            BCJR VARIABLES.
*******************************************************************************/
double Alpha[FRAME_SIZE][NUM_STATES];                              /* Alphas. */
double Beta[FRAME_SIZE_P1][NUM_STATES];                             /* Betas. */
double Gamma_I[FRAME_SIZE][NUM_INPUTS];              /* Intrinsic info: data. */
double Gamma_P[FRAME_SIZE][NUM_INPUTS];            /* Extrinsic info: parity. */
double Prob[FRAME_SIZE][NUM_INPUTS];                   /* Data probabilities. */
/******************************************************************************/
/*                BCJR VARIABLES RELATED TO CHANNEL DECODER.
*******************************************************************************/
int Outer_Cnt;                                         /* Outer loop counter. */
double Ch_Alpha[CH_IV_SIZE][NUM_ST_STATES];                         /* Alpha. */
double Ch_Beta[CH_IV_SIZE_P1][NUM_ST_STATES];                        /* Beta. */
double Ch_Gamma[CH_IV_SIZE][NUM_ST_STATES][NUM_INPUTS];             /* Gamma. */
double Ch_Prob[CH_IV_SIZE][NUM_INPUTS];                     /* Probabilities. */
/******************************************************************************/
/*                LOG LIKELIHOOD RATIOS AND MUTUAL INFORMATION.
*******************************************************************************/
double LLR_Variance;                          /* Var. of noise added to LLRs. */
double Inv_Twice_LLR_Var;         /* Inverse of twice the LLR noise variance. */
int Valid_Iter[MAX_OUTER];                    /* Checks for valid iterations. */
double Ch_MI[MAX_OUTER];                             /* O/p MI of CDFE-STMAP. */
double CC_MI[MAX_OUTER];                             /* O/p MI of CC decoder. */
double Ch_Mean[MAX_OUTER];                /* Mean LLR at ch. MAP decoder o/p. */
double CC_Mean[MAX_OUTER];                /* Mean LLR at CC. MAP decoder o/p. */
double Ch_Var[MAX_OUTER];                                /* Ch. LLR variance. */
double CC_Var[MAX_OUTER];                                /* CC. LLR variance. */
double Ch_LLR[NUM_INPUTS][MAX_OUTER];                /* LLRs for the channel. */
double CC_LLR[NUM_INPUTS][MAX_OUTER];                     /* LLRs for the CC. */
/******************************************************************************/
/*                            HISTOGRAM PARAMETERS.
*******************************************************************************/
double Min_Ch_LLR[MAX_OUTER];                            /* Min. channel LLR. */
double Max_Ch_LLR[MAX_OUTER];                            /* Max. channel LLR. */
double Min_CC_LLR[MAX_OUTER];                                 /* Min. CC LLR. */
double Max_CC_LLR[MAX_OUTER];                                 /* Max. CC LLR. */
double Ch_Hist_Step[MAX_OUTER];               /* Channel histogram step size. */
double CC_Hist_Step[MAX_OUTER];                    /* CC histogram step size. */
int Ch_Freq[HIST_SIZE][MAX_OUTER];                 /* Channel histogram freq. */
int CC_Freq[HIST_SIZE][MAX_OUTER];                      /* CC histogram freq. */
/******************************************************************************/
