/******************************************************************************/
/*   Copyright (C) 2005 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
#include "teq8tjtype.h"
#include "teq8tjext.h"
/******************************************************************************/
/*   Compute the input and output mutual information between the bits and
 *   the LLRs.
*******************************************************************************/
int Compute_Mutual_Info()
{
 Compute_Mean_LLRs();
 Compute_Trajectory();
 return(0);
}
/******************************************************************************/
/*           Compute the mean value of the log-likelihood ratios.
*******************************************************************************/
int Compute_Mean_LLRs()
{
 int cnt;
 for(cnt=ZERO;cnt<MAX_OUTER;cnt++)
 {
  Ch_Mean[cnt]=Ch_LLR[ZERO][cnt]-Ch_LLR[ONE][cnt];
  CC_Mean[cnt]=CC_LLR[ZERO][cnt]-CC_LLR[ONE][cnt];
  Ch_Mean[cnt]=Ch_Mean[cnt]/(double)(SIM_FRAMES*CH_IV_SIZE);
  CC_Mean[cnt]=CC_Mean[cnt]/(double)(SIM_FRAMES*CH_IV_SIZE);
  Ch_Var[cnt]=2.0*Ch_Var[cnt]/(double)(SIM_FRAMES*CH_IV_SIZE);
  CC_Var[cnt]=2.0*CC_Var[cnt]/(double)(SIM_FRAMES*CH_IV_SIZE);
  printf("\t Iter:%d Validity:%d ",cnt,Valid_Iter[cnt]);
  printf("Final Ch. mean:%lf ",Ch_Mean[cnt]);
  printf("CC. mean:%lf\n",CC_Mean[cnt]);
  printf("\t Final Ch. var:%lf ",Ch_Var[cnt]);
  printf("CC. var:%lf\n",CC_Var[cnt]);
 }
 return(0);
}
/******************************************************************************/
/*                        Compute the trajectory.
*******************************************************************************/
int Compute_Trajectory()
{
 int cnt;
 for(cnt=ZERO;cnt<MAX_OUTER;cnt++)
 {
  if(Valid_Iter[cnt] == ONE)
  {
//   Compute_Ch_MI(cnt);
//   Compute_CC_MI(cnt);
   Compute_Hist_Output_Ch_MI(cnt);
   Compute_Hist_Output_CC_MI(cnt);
  }
 }
 return(0);
}
/******************************************************************************/
/*             Compute the o/p mutual information for CDFE-STMAP.
*******************************************************************************/
int Compute_Ch_MI(iter)
int iter;
{
 int bit;
 double mean,var,llim,ulim,step,x,y0,y1,numer,denom;
 mean=Ch_Mean[iter];
 var=Ch_Var[iter];
 llim= -mean-5.0*sqrt(var);
 ulim=  mean+5.0*sqrt(var);
 step=(ulim-llim)/(double)MAX_STEPS;
 Ch_MI[iter]=0.0;
 for(bit=ZERO;bit<NUM_INPUTS;bit++)
 {
  for(x=llim;x<ulim;x=x+step)
  {
   y0=(x-mean*Bit_Map[bit])*(x-mean*Bit_Map[bit])/(2.0*var);
   y1=(x+mean*Bit_Map[bit])*(x+mean*Bit_Map[bit])/(2.0*var);
   numer=exp(-y0);
   denom=exp(-y0)+exp(-y1);
   Ch_MI[iter]=Ch_MI[iter]+numer*log(2.0*numer/denom);
  }
 }
 Ch_MI[iter]=Ch_MI[iter]*0.5*step/(sqrt(2.0*MY_PI*var)*log(2.0));
 return(0);
}
/******************************************************************************/
/*           Compute the o/p mutual information for the CC decoder.
*******************************************************************************/
int Compute_CC_MI(iter)
int iter;
{
 int bit;
 double mean,var,llim,ulim,step,x,y0,y1,numer,denom;
 mean=CC_Mean[iter];
 var=CC_Var[iter];
 llim= -mean-5.0*sqrt(var);
 ulim=  mean+5.0*sqrt(var);
 step=(ulim-llim)/(double)MAX_STEPS;
 CC_MI[iter]=0.0;
 for(bit=ZERO;bit<NUM_INPUTS;bit++)
 {
  for(x=llim;x<ulim;x=x+step)
  {
   y0=(x-mean*Bit_Map[bit])*(x-mean*Bit_Map[bit])/(2.0*var);
   y1=(x+mean*Bit_Map[bit])*(x+mean*Bit_Map[bit])/(2.0*var);
   numer=exp(-y0);
   denom=exp(-y0)+exp(-y1);
   CC_MI[iter]=CC_MI[iter]+numer*log(2.0*numer/denom);
  }
 }
 CC_MI[iter]=CC_MI[iter]*0.5*step/(sqrt(2.0*MY_PI*var)*log(2.0));
 return(0);
}
/******************************************************************************/
