/******************************************************************************/
/*   Copyright (C) 2005 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
#include "teq8tjtype.h"
#include "teq8tjext.h"
/******************************************************************************/
/******************************************************************************/
/*   Compute the channel MAP decoder output mutual information using the
 *   histogram.
*******************************************************************************/
int Compute_Hist_Output_Ch_MI(iter)
int iter;
{
 Ch_MI[iter]=0.0;
 Get_Ch_MI_1(iter);
 Get_Ch_MI_2(iter);
 Ch_MI[iter]=Ch_MI[iter]/(log(2.0)*(double)(SIM_FRAMES*CH_IV_SIZE));
 return(0);
}
/******************************************************************************/
/*   Compute the channel MAP decoder output mutual information using the
 *   histogram when the bit is +1.
*******************************************************************************/
int Get_Ch_MI_1(iter)
int iter;
{
 int cnt,inx;
 double numer,denom,x,x1,min;
 min=Min_Ch_LLR[iter]+2.0*Ch_Mean[iter];
 for(cnt=ZERO;cnt<HIST_SIZE;cnt++)
 {
  if(Ch_Freq[cnt][iter] != ZERO)
  {
   numer=(double)Ch_Freq[cnt][iter];
   x=min+(double)cnt*Ch_Hist_Step[iter]+0.5*Ch_Hist_Step[iter];
   x1=x-Min_Ch_LLR[iter];
   if(fabs(Ch_Hist_Step[iter]) > ERROR)
    inx=(int)(x1/Ch_Hist_Step[iter]);
   else
    inx=ZERO;
   if(inx<HIST_SIZE)
    denom=(double)Ch_Freq[inx][iter];
   else
    denom=0.0;
   denom=denom+numer;
   Ch_MI[iter]=Ch_MI[iter]+numer*log(2.0*numer/denom);
  }
 }
 return(0);
}
/******************************************************************************/
/*   Compute the channel MAP decoder output mutual information using the
 *   histogram when the bit is -1.
*******************************************************************************/
int Get_Ch_MI_2(iter)
int iter;
{
 int cnt,inx;
 double numer,denom,x,x1,min;
 min=Min_Ch_LLR[iter];
 for(cnt=ZERO;cnt<HIST_SIZE;cnt++)
 {
  if(Ch_Freq[cnt][iter] != ZERO)
  {
   numer=(double)Ch_Freq[cnt][iter];
   x=min+(double)cnt*Ch_Hist_Step[iter]+0.5*Ch_Hist_Step[iter];
   x1=x-Min_Ch_LLR[iter]-2.0*Ch_Mean[iter];
   if(fabs(Ch_Hist_Step[iter]) > ERROR)
    inx=(int)(x1/Ch_Hist_Step[iter]);
   else
    inx=ZERO;
   if(inx >= ZERO)
    denom=(double)Ch_Freq[inx][iter];
   else
    denom=0.0;
   denom=denom+numer;
   Ch_MI[iter]=Ch_MI[iter]+numer*log(2.0*numer/denom);
  }
 }
 return(0);
}
/******************************************************************************/
/*   Compute the CC MAP decoder output mutual information using the
 *   histogram.
*******************************************************************************/
int Compute_Hist_Output_CC_MI(iter)
int iter;
{
 CC_MI[iter]=0.0;
 Get_CC_MI_1(iter);
 Get_CC_MI_2(iter);
 CC_MI[iter]=CC_MI[iter]/(log(2.0)*(double)(SIM_FRAMES*CH_IV_SIZE));
 return(0);
}
/******************************************************************************/
/*   Compute the CC MAP decoder output mutual information using the
 *   histogram when the bit is +1.
*******************************************************************************/
int Get_CC_MI_1(iter)
int iter;
{
 int cnt,inx;
 double numer,denom,x,x1,min;
 min=Min_CC_LLR[iter]+2.0*CC_Mean[iter];
 for(cnt=ZERO;cnt<HIST_SIZE;cnt++)
 {
  if(CC_Freq[cnt][iter] != ZERO)
  {
   numer=(double)CC_Freq[cnt][iter];
   x=min+(double)cnt*CC_Hist_Step[iter]+0.5*CC_Hist_Step[iter];
   x1=x-Min_CC_LLR[iter];
   if(fabs(CC_Hist_Step[iter]) > ERROR)
    inx=(int)(x1/CC_Hist_Step[iter]);
   else
    inx=ZERO;
   if(inx<HIST_SIZE)
    denom=(double)CC_Freq[inx][iter];
   else
    denom=0.0;
   denom=denom+numer;
   CC_MI[iter]=CC_MI[iter]+numer*log(2.0*numer/denom);
  }
 }
 return(0);
}
/******************************************************************************/
/*   Compute the CC MAP decoder output mutual information using the
 *   histogram when the bit is -1.
*******************************************************************************/
int Get_CC_MI_2(iter)
int iter;
{
 int cnt,inx;
 double numer,denom,x,x1,min;
 min=Min_CC_LLR[iter];
 for(cnt=ZERO;cnt<HIST_SIZE;cnt++)
 {
  if(CC_Freq[cnt][iter] != ZERO)
  {
   numer=CC_Freq[cnt][iter];
   x=min+(double)cnt*CC_Hist_Step[iter]+0.5*CC_Hist_Step[iter];
   x1=x-Min_CC_LLR[iter]-2.0*CC_Mean[iter];
   if(fabs(CC_Hist_Step[iter]) > ERROR)
    inx=(int)(x1/CC_Hist_Step[iter]);
   else
    inx=ZERO;
   if(inx >= ZERO)
    denom=(double)CC_Freq[inx][iter];
   else
    denom=0.0;
   denom=denom+numer;
   CC_MI[iter]=CC_MI[iter]+numer*log(2.0*numer/denom);
  }
 }
 return(0);
}
/******************************************************************************/
