/******************************************************************************/
/*   Copyright (C) 2005 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
#include "teq8tjtype.h"
#include "teq8tjext.h"
/******************************************************************************/
int Simulation()
{
 for(Frame_Cnt=ZERO;Frame_Cnt<MAX_FRAMES;Frame_Cnt++)
 {
  Get_Frame();
  if(APRIORI_GAUSSIAN == ONE)
   Add_LLR_Noise();
  if(Frame_Cnt > TWO)
   Step=0.05;
  Init_FB_Reg();
  ISI_Procedures();
  if(Frame_Cnt >= TRN_FRAMES)
   Receive();
  if(Frame_Cnt == MEAN_TIME)
   Compute_Mean();
 }
 Compute_Mutual_Info();
 return(0);
}
/******************************************************************************/
/*  Add white Gaussian noise to parity symbols for conversion to a priori
 *  probabilities.
*******************************************************************************/
int Add_LLR_Noise()
{
 int time;
 double erand48();
 for(time=ZERO;time<CH_IV_SIZE;time=time+TWO)
 {
  N1_Val[URADIUS]=erand48(N1_Store);
  N1_Val[UTHETA]=erand48(N2_Store);
  My_Gauss(LLR_Variance,N1_Val);
  P_Sym[time]=P_Sym[time]+N1_Val[GRAND_0];
  P_Sym[time+ONE]=P_Sym[time+ONE]+N1_Val[GRAND_1];
 }
 return(0);
}
/******************************************************************************/
/*                     Initialize the feedback register.
*******************************************************************************/
int Init_FB_Reg()
{
 int cnt;
 for(cnt=ZERO;cnt<FB_SIZE;cnt++)
  Re_FB_Reg[cnt]=Im_FB_Reg[cnt]=1.0;
 return(0);
}
/******************************************************************************/
/*                             ISI procedures.
*******************************************************************************/
int ISI_Procedures()
{
 int time;
 Enc1_State=ZERO;
 for(time=ZERO;time<RX_DATA_END;time++)
 {
  Get_Channel_Op(time);
  Add_Noise();
  Get_Eq_Op();
  if((time>=RX_DATA_START) && (Frame_Cnt<TRN_FRAMES))
   Eq_Training();
  else if((time >= RX_DATA_START) && (Frame_Cnt >= TRN_FRAMES))
   Get_Noisy_Symbols(time);
 }
 return(0);
}
/******************************************************************************/
/*                  Get the samples at the channel output.
*******************************************************************************/
int Get_Channel_Op(time)
int time;
{
 int iv_ip_time,bit;
 if(time < PREAMBLE)
  bit=ZERO;
 else if(time < TX_DATA_END)
 {
  iv_ip_time=Ch_De_Iv_Map[time-PREAMBLE];
  bit=Enc_Op[iv_ip_time];
 }
 else
  bit=ZERO;
 Right_Shift_Reg(Re_Ch_Reg,Im_Ch_Reg,CHAN_SIZE);
 Shift_In_New_Ch_Inp(bit,time);
 Compute_Ch_Op();
 return(0);
}
/******************************************************************************/
/*                        Shift in the new channel input.
*******************************************************************************/
int Shift_In_New_Ch_Inp(bit,time)
int bit,time;
{
 int pty;
 pty=Trel_Op[Enc1_State][bit];
 Enc1_State=Trel_NS[Enc1_State][bit];
 Re_Ch_Reg[ZERO]=Bit_Map[bit];
 Im_Ch_Reg[ZERO]=Bit_Map[pty];
 if((time >= PREAMBLE) && (time < TX_DATA_END) && (Frame_Cnt < TRN_FRAMES))
 {
  if(Eq_Buf_Inx == EQ_BUF_SIZE)
   Error_Mes(13);
  Eq_Buf[Eq_Buf_Inx]=TWO*pty+bit;
  Eq_Buf_Inx++;
 }
 return(0);
}
/******************************************************************************/
/*                       Compute the channel output.
*******************************************************************************/
int Compute_Ch_Op()
{
 int cnt;
 Re_Ch_Op=Im_Ch_Op=0.0;
 for(cnt=ZERO;cnt<CHAN_SIZE;cnt++)
 {
  Re_Ch_Op=Re_Ch_Op+Re_Ch[cnt]*Re_Ch_Reg[cnt];
  Re_Ch_Op=Re_Ch_Op-Im_Ch[cnt]*Im_Ch_Reg[cnt];
  Im_Ch_Op=Im_Ch_Op+Re_Ch[cnt]*Im_Ch_Reg[cnt];
  Im_Ch_Op=Im_Ch_Op+Im_Ch[cnt]*Re_Ch_Reg[cnt];
 }
 return(0);
}
/******************************************************************************/
/*                        Add white Gaussian noise.
*******************************************************************************/
int Add_Noise()
{
 double erand48();
 N1_Val[URADIUS]=erand48(N1_Store);
 N1_Val[UTHETA]=erand48(N2_Store);
 My_Gauss(Variance,N1_Val);
 Re_Ch_Op=Re_Ch_Op+N1_Val[GRAND_0];
 Im_Ch_Op=Im_Ch_Op+N1_Val[GRAND_1];
 return(0);
}
/******************************************************************************/
/*                          Get equalizer output.
*******************************************************************************/
int Get_Eq_Op()
{
 int cnt;
 Right_Shift_Reg(Re_Eq_Reg,Im_Eq_Reg,EQ_SIZE);
 Re_Eq_Reg[ZERO]=Re_Ch_Op;
 Im_Eq_Reg[ZERO]=Im_Ch_Op;
 Re_Eq_Op=Im_Eq_Op=0.0;
 for(cnt=ZERO;cnt<EQ_SIZE;cnt++)
 {
  Re_Eq_Op=Re_Eq_Op+Re_Eq_Reg[cnt]*Re_Eq[cnt]-Im_Eq_Reg[cnt]*Im_Eq[cnt];
  Im_Eq_Op=Im_Eq_Op+Re_Eq_Reg[cnt]*Im_Eq[cnt]+Im_Eq_Reg[cnt]*Re_Eq[cnt];
 }
 return(0);
}
/******************************************************************************/
/*                 Get noisy symbols from equalizer output.
*******************************************************************************/
int Get_Noisy_Symbols(time)
int time;
{
 Rx_Re_Sym[time-RX_DATA_START]=Re_Eq_Op;
 Rx_Im_Sym[time-RX_DATA_START]=Im_Eq_Op;
 return(0);
}
/******************************************************************************/
/*                         Receiver routines.
*******************************************************************************/
int Receive()
{
 Init_Ch_Dec();
 if(APRIORI_GAUSSIAN == ONE)
  Init_Ch_Prob_Gaussian();
 else
  Init_Ch_Prob();
 for(Outer_Cnt=ZERO;Outer_Cnt<MAX_OUTER;Outer_Cnt++)
 {
  Compute_Ch_APP();
  Compute_Ch_LLR();
  Init_Gamma_Turbo_Dec();
  Dec1_Process();
  Compute_CC_LLR();
 }
 Get_Bit_Errors();
 return(0);
}
/******************************************************************************/
/*                      Initialize the channel decoder.
*******************************************************************************/
int Init_Ch_Dec()
{
 int time,state,ip;
 for(time=ZERO;time<CH_IV_SIZE;time++)
 {
  for(state=ZERO;state<NUM_ST_STATES;state++)
  {
   for(ip=ZERO;ip<NUM_INPUTS;ip++)
   {
    if(time <FB_SIZE)
     Compute_Initial_Ch_Gamma(time,state,ip);
    else
     Compute_Ch_Gamma(time,state,ip);
   }
  }
 }
 return(0);
}
/******************************************************************************/
/*           Compute the initial gammas for the CDFE-MAP decoder.
*******************************************************************************/
int Compute_Initial_Ch_Gamma(time,state,ip)
int time,state,ip;
{
 int cnt,inx,pty,bit;
 double x,re_sym,im_sym,re_sum,im_sum,re_err,im_err;
 re_sum=im_sum=0.0;
 for(cnt=ONE;cnt<FB_SIZE_P1;cnt++)
 {
  inx=time-cnt;
  if(inx < ZERO)
   re_sym=im_sym=1.0;
  else
  {
   bit=STrel_Ip_Array[state][ip][cnt];
   pty=STrel_Pt_Array[state][ip][cnt];
   re_sym=Bit_Map[bit];
   im_sym=Bit_Map[pty];
  }
  re_sum=re_sum+re_sym*Re_FB[cnt-ONE]-im_sym*Im_FB[cnt-ONE];
  im_sum=im_sum+im_sym*Re_FB[cnt-ONE]+re_sym*Im_FB[cnt-ONE];
 }
 bit=STrel_Ip_Array[state][ip][ZERO];
 pty=STrel_Pt_Array[state][ip][ZERO];
 if(IDEAL_EQ == ZERO)
 {
  re_err=Bit_Map[bit]-re_sum-Rx_Re_Sym[time];
  im_err=Bit_Map[pty]-im_sum-Rx_Im_Sym[time];
 }
 else
 {
  re_err=Bit_Map[bit]+re_sum-Rx_Re_Sym[time];
  im_err=Bit_Map[pty]+im_sum-Rx_Im_Sym[time];
 }
 x=re_err*re_err+im_err*im_err;
 x= -x*Inv_Twice_Eq_Var;
 Ch_Gamma[time][state][ip]=exp(x);
 return(0);
}
/******************************************************************************/
/*             Compute the gammas for the CDFE-MAP decoder.
*******************************************************************************/
int Compute_Ch_Gamma(time,state,ip)
int time,state,ip;
{
 int cnt,pty,bit;
 double x,re_sym,im_sym,re_sum,im_sum,re_err,im_err;
 re_sum=im_sum=0.0;
 for(cnt=ONE;cnt<FB_SIZE_P1;cnt++)
 {
  bit=STrel_Ip_Array[state][ip][cnt];
  pty=STrel_Pt_Array[state][ip][cnt];
  re_sym=Bit_Map[bit];
  im_sym=Bit_Map[pty];
  re_sum=re_sum+re_sym*Re_FB[cnt-ONE]-im_sym*Im_FB[cnt-ONE];
  im_sum=im_sum+im_sym*Re_FB[cnt-ONE]+re_sym*Im_FB[cnt-ONE];
 }
 bit=STrel_Ip_Array[state][ip][ZERO];
 pty=STrel_Pt_Array[state][ip][ZERO];
 if(IDEAL_EQ == ZERO)
 {
  re_err=Bit_Map[bit]-re_sum-Rx_Re_Sym[time];
  im_err=Bit_Map[pty]-im_sum-Rx_Im_Sym[time];
 }
 else
 {
  re_err=Bit_Map[bit]+re_sum-Rx_Re_Sym[time];
  im_err=Bit_Map[pty]+im_sum-Rx_Im_Sym[time];
 }
 x=re_err*re_err+im_err*im_err;
 x= -x*Inv_Twice_Eq_Var;
 Ch_Gamma[time][state][ip]=exp(x);
 return(0);
}
/******************************************************************************/
/*   Get the a priori probabilities at the input of the channel MAP decoder.
 *   Here a priori probabilities are Gaussian.
*******************************************************************************/
int Init_Ch_Prob_Gaussian()
{
 int time;
 for(time=ZERO;time<FRAME_SIZE;time++)
  Init_Each_Ch_Prob(time);
 return(0);
}
/******************************************************************************/
/*                   Get noisy data and parity inputs.
*******************************************************************************/
int Init_Each_Ch_Prob(time)
int time;
{
 int bit,iv_inx_p0,iv_inx_p1;
 double x,y;
 iv_inx_p0=Ch_Iv_Map[TWO*time];
 iv_inx_p1=Ch_Iv_Map[TWO*time+ONE];
 for(bit=ZERO;bit<NUM_INPUTS;bit++)
 {
  x=P_Sym[TWO*time]-Bit_Map[bit];
  y=P_Sym[TWO*time+ONE]-Bit_Map[bit];
  x= -x*x*Inv_Twice_LLR_Var;
  y= -y*y*Inv_Twice_LLR_Var;
  Ch_Prob[iv_inx_p0][bit]=exp(x);
  Ch_Prob[iv_inx_p1][bit]=exp(y);
 }
 Normalize_Ch_Prob(iv_inx_p0,iv_inx_p1);
 return(0);
}
/******************************************************************************/
/*                Normalize the a priori channel probabilities.
*******************************************************************************/
int Normalize_Ch_Prob(iv_inx_p0,iv_inx_p1)
int iv_inx_p0,iv_inx_p1;
{
 int bit;
 double sum0,sum1;
 sum0=sum1=0.0;
 for(bit=ZERO;bit<NUM_INPUTS;bit++)
 {
  sum0=sum0+Ch_Prob[iv_inx_p0][bit];
  sum1=sum1+Ch_Prob[iv_inx_p1][bit];
 }
 for(bit=ZERO;bit<NUM_INPUTS;bit++)
 {
  Ch_Prob[iv_inx_p0][bit]=Ch_Prob[iv_inx_p0][bit]/sum0;
  Ch_Prob[iv_inx_p1][bit]=Ch_Prob[iv_inx_p1][bit]/sum1;
 }
 return(0);
}
/******************************************************************************/
/*   Get the a priori probabilities at the input of the channel MAP decoder.
 *   Here a priori probabilities are constants.
*******************************************************************************/
int Init_Ch_Prob()
{
 int time;
 for(time=ZERO;time<CH_IV_SIZE;time++)
  Ch_Prob[time][ZERO]=Ch_Prob[time][ONE]=0.5;
 return(0);
}
/******************************************************************************/
/*    Compute the log-likelihood ratios at the channel MAP decoder output.
*******************************************************************************/
int Compute_Ch_LLR()
{
 int time,iv_ip_time,bit;
 double llr,ub_llr;
 for(time=ZERO;time<CH_IV_SIZE;time++)
 {
  Get_LLR(time,&llr);
  iv_ip_time=Ch_De_Iv_Map[time];
  bit=Enc_Op[iv_ip_time];
  Ch_LLR[bit][Outer_Cnt]=Ch_LLR[bit][Outer_Cnt]+llr;
  if(Frame_Cnt <= MEAN_TIME)
   Update_Ch_Extrema(llr);
  if(Frame_Cnt > MEAN_TIME)
  {
   ub_llr=llr-Ch_Mean[Outer_Cnt]*Bit_Map[bit];
   Ch_Var[Outer_Cnt]=Ch_Var[Outer_Cnt]+ub_llr*ub_llr;
   Update_Ch_Hist(ub_llr);
  }
 }
 return(0);
}
/******************************************************************************/
/*               Update the maxima and minima of the channel LLR.
*******************************************************************************/
int Update_Ch_Extrema(llr)
double llr;
{
 if(llr < Min_Ch_LLR[Outer_Cnt])
  Min_Ch_LLR[Outer_Cnt]=llr;
 if(llr > Max_Ch_LLR[Outer_Cnt])
  Max_Ch_LLR[Outer_Cnt]=llr;
 return(0);
}
/******************************************************************************/
/*                      Update the channel LLR histogram.
*******************************************************************************/
int Update_Ch_Hist(ub_llr)
double ub_llr;
{
 int inx;
 double min;
 min=Min_Ch_LLR[Outer_Cnt]+Ch_Mean[Outer_Cnt];
 if(fabs(Ch_Hist_Step[Outer_Cnt]) > ERROR)
  inx=(int)((ub_llr-min)/Ch_Hist_Step[Outer_Cnt]);
 else
  inx=HIST_SIZE/TWO;
 if(inx < ZERO)
  inx=ZERO;
 else if(inx >= HIST_SIZE)
  inx=HIST_SIZE-ONE;
 Ch_Freq[inx][Outer_Cnt]++;
 return(0);
}
/******************************************************************************/
/*          Initialize the intrinsic and extrinsic values of gamma.
*******************************************************************************/
int Init_Gamma_Turbo_Dec()
{
 Init_Gamma_I_E();
// Get_Ch_MAP_Op_Err();
 return(0);
}
/******************************************************************************/
/*  Initialize the intrinsic and extrinsic values of gamma for decoder 1 and
 *  two.
*******************************************************************************/
int Init_Gamma_I_E()
{
 int time,bit,d_inx,p_inx;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  d_inx=Ch_Iv_Map[TWO*time];
  p_inx=Ch_Iv_Map[TWO*time+ONE];
  for(bit=ZERO;bit<NUM_INPUTS;bit++)
  {
   Gamma_I[time][bit]=Ch_Prob[d_inx][bit];
   Gamma_P[time][bit]=Ch_Prob[p_inx][bit];
  }
 }
 return(0);
}
/******************************************************************************/
/*        Get the errors in a frame at the channel MAP detector output.
*******************************************************************************/
int Get_Ch_MAP_Op_Err()
{
 int time,d_err,p_err,d_bit,p_bit;
 d_err=p_err=ZERO;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  d_bit=p_bit=ZERO;
  if(Gamma_I[time][ONE] > Gamma_I[time][ZERO])
   d_bit=ONE;
  if(d_bit != Enc_Op[TWO*time])
   d_err++;
  if(Gamma_P[time][ONE] > Gamma_P[time][ZERO])
   p_bit=ONE;
  if(p_bit != Enc_Op[TWO*time+ONE])
   p_err++;
 }
 printf("fr:%d derr:%d perr:%d\n",Frame_Cnt,d_err,p_err);
 return(0);
}
/******************************************************************************/
/*               Get the number of errors in the current frame.
*******************************************************************************/
int Get_Frame_Error()
{
 int time,d_inx,p_inx,d_err,p_err,d_bit,p_bit;
 d_err=p_err=ZERO;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  d_inx=Ch_Iv_Map[TWO*time];
  p_inx=Ch_Iv_Map[TWO*time+ONE];
  d_bit=p_bit=ZERO;
  if(Ch_Prob[d_inx][ONE] > Ch_Prob[d_inx][ZERO])
   d_bit=ONE;
  if(d_bit != Enc_Op[TWO*time])
   d_err++;
  if(Ch_Prob[p_inx][ONE] > Ch_Prob[p_inx][ZERO])
   p_bit=ONE;
  if(p_bit != Enc_Op[TWO*time+ONE])
   p_err++;
 }
 printf("br:%d derr:%d perr:%d\n",Frame_Cnt,d_err,p_err);
 return(0);
}
/******************************************************************************/
/*      Compute the log-likelihood ratios at the CC MAP decoder output.
*******************************************************************************/
int Compute_CC_LLR()
{
 int time,iv_ip_time,bit;
 double llr,ub_llr;
 for(time=ZERO;time<CH_IV_SIZE;time++)
 {
  Get_LLR(time,&llr);
  iv_ip_time=Ch_De_Iv_Map[time];
  bit=Enc_Op[iv_ip_time];
  CC_LLR[bit][Outer_Cnt]=CC_LLR[bit][Outer_Cnt]+llr;
  if(Frame_Cnt <= MEAN_TIME)
   Update_CC_Extrema(llr);
  if(Frame_Cnt > MEAN_TIME)
  {
   ub_llr=llr-CC_Mean[Outer_Cnt]*Bit_Map[bit];
   CC_Var[Outer_Cnt]=CC_Var[Outer_Cnt]+ub_llr*ub_llr;
   Update_CC_Hist(ub_llr);
  }
 }
 return(0);
}
/******************************************************************************/
/*                Update the maxima and minima of the CC LLR.
*******************************************************************************/
int Update_CC_Extrema(llr)
double llr;
{
 if(llr < Min_CC_LLR[Outer_Cnt])
  Min_CC_LLR[Outer_Cnt]=llr;
 if(llr > Max_CC_LLR[Outer_Cnt])
  Max_CC_LLR[Outer_Cnt]=llr;
 return(0);
}
/******************************************************************************/
/*                       Update the CC LLR histogram.
*******************************************************************************/
int Update_CC_Hist(ub_llr)
double ub_llr;
{
 int inx;
 double min;
 min=Min_CC_LLR[Outer_Cnt]+CC_Mean[Outer_Cnt];
 if(fabs(CC_Hist_Step[Outer_Cnt]) > ERROR)
  inx=(int)((ub_llr-min)/CC_Hist_Step[Outer_Cnt]);
 else
  inx=HIST_SIZE/TWO;
 if(inx < ZERO)
  inx=ZERO;
 else if(inx >= HIST_SIZE)
  inx=HIST_SIZE-ONE;
 CC_Freq[inx][Outer_Cnt]++;
 return(0);
}
/******************************************************************************/
/*               Check is all data in this iteration is valid.
*******************************************************************************/
int Check_Validity()
{
 int time;
 double diff;
 for(time=ZERO;time<CH_IV_SIZE;time++)
 {
  diff=Ch_Prob[time][ZERO]-Ch_Prob[time][ONE];
  diff=fabs(diff);
  if(fabs(1.0-diff) < ERROR)
   Valid_Iter[Outer_Cnt]=ZERO;
 }
 return(0);
}
/******************************************************************************/
/*                              Get the LLR.
*******************************************************************************/
int Get_LLR(time,llr_addr)
int time;
double *llr_addr;
{
 double prob;
 prob=Ch_Prob[time][ZERO];
 if((1.0-prob) < ERROR)
  *llr_addr=MAX_LLR;
 else if(prob < ERROR)
  *llr_addr= -MAX_LLR;
 else
  *llr_addr=log(Ch_Prob[time][ZERO]/Ch_Prob[time][ONE]);
 return(0);
}
/******************************************************************************/
/*                   Compute the mean value of the LLRs.
*******************************************************************************/
int Compute_Mean()
{
 int cnt;
 double ch_step,cc_step;
 for(cnt=ZERO;cnt<MAX_OUTER;cnt++)
 {
  Ch_Mean[cnt]=Ch_LLR[ZERO][cnt]-Ch_LLR[ONE][cnt];
  CC_Mean[cnt]=CC_LLR[ZERO][cnt]-CC_LLR[ONE][cnt];
  Ch_Mean[cnt]=2.0*Ch_Mean[cnt]/(double)(SIM_FRAMES*CH_IV_SIZE);
  CC_Mean[cnt]=2.0*CC_Mean[cnt]/(double)(SIM_FRAMES*CH_IV_SIZE);
  ch_step=Max_Ch_LLR[cnt]-Min_Ch_LLR[cnt]-2.0*Ch_Mean[cnt];
  Ch_Hist_Step[cnt]=ch_step/(double)HIST_SIZE;
  cc_step=Max_CC_LLR[cnt]-Min_CC_LLR[cnt]-2.0*CC_Mean[cnt];
  CC_Hist_Step[cnt]=cc_step/(double)HIST_SIZE;
  printf("\t Iter:%d Validity:%d ",cnt,Valid_Iter[cnt]);
  printf("Ch. mean:%lf CC. mean:%lf\n",Ch_Mean[cnt],CC_Mean[cnt]);
  printf("\t Hist. step Ch:%lf ",Ch_Hist_Step[cnt]);
  printf("CC:%lf\n\n",CC_Hist_Step[cnt]);
 }
 return(0);
}
/******************************************************************************/
/*           Finally compute the number of bit errors in the frame.
*******************************************************************************/
int Get_Bit_Errors()
{
 int time,detected_bit;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  detected_bit=ONE;
  if(Prob[time][ZERO] > Prob[time][ONE])
   detected_bit=ZERO;
  if(detected_bit != Enc_Op[TWO*time])
   Ber++;
 }
 return(0);
}
/******************************************************************************/
