/******************************************************************************/
/*   Copyright (C) 2005 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
#include "teq8tjtype.h"
#include "teq8tjext.h"
/******************************************************************************/
/*                         Convert decimal to binary.
*******************************************************************************/
int Decimal_to_Binary(num,array,array_size)
int num,array[],array_size;
{
 int cnt;
 for(cnt=ZERO;cnt<array_size;cnt++)
  array[cnt]=ZERO;
 cnt=ZERO;
 while(num>ZERO)
 {
  if(cnt == array_size)
   Error_Mes(1);
  array[cnt]=num&1;
  num=num>>1;
  cnt++;
 }
 return(0);
}
/******************************************************************************/
/*                    Convert binary array to decimal.
*******************************************************************************/
int Binary_to_Decimal(array,size)
int array[],size;
{
 int cnt,pow,sum;
 sum=ZERO;
 pow=1;
 for(cnt=ZERO;cnt<size;cnt++)
 {
  sum=sum+pow*array[cnt];
  pow=pow<<1;
 }
 return(sum);
}
/******************************************************************************/
/* Convert the state given in decimal into modulo-M and return the result in
 * the array 'stmary[]'.
*******************************************************************************/
int Decimal_To_M_ary(stcnt,stmary,size,M)
int stcnt,stmary[],size,M;
{
 int index;
 for(index=ZERO;index<size;index++)
  stmary[index]=ZERO;
 index=ZERO;
 while(stcnt>ZERO)
 {
  if(index == size)
   Error_Mes(1);
  stmary[index]=stcnt%M;
  stcnt=stcnt/M;
  index++;
 }
 return(0);
}
/******************************************************************************/
/*                       Convert M-ary to decimal.
*******************************************************************************/
int Mary_to_Decimal(stmary,size,M)
int stmary[],size,M;
{
 int cnt,power,decimal;
 power=ONE;
 decimal=ZERO;
 for(cnt=ZERO;cnt<size;cnt++)
 {
  decimal=decimal+stmary[cnt]*power;
  power=power*M;
 }
 return(decimal);
}
/******************************************************************************/
/* Fill the memory with the parity bits. Most recent input bit is
 * at ZERO. Most recent parity is also at ZERO.
*******************************************************************************/
int Fill_Mem(mod_num_inp,st_cnt,mod_ip_cnt,mem_size,ip_array,parity)
int mod_num_inp,st_cnt,mod_ip_cnt,mem_size,ip_array[],parity[];
{
 int cnt,inp,s_state;
 printf("\t Enc starting state:%d\n",st_cnt);
 for(cnt=mem_size-ONE;cnt>=ZERO;cnt--)
 {
  inp=ip_array[cnt];
  parity[cnt]=Trel_Op[st_cnt][inp];
  printf("\t Input:%d ",inp);
  printf("Parity:%d\n",parity[cnt]);
  st_cnt=Trel_NS[st_cnt][inp];
 }
 s_state=st_cnt*mod_num_inp+mod_ip_cnt;
 printf("\t Final encoder state:%d\n",st_cnt);
 printf("\t Final super trellis state:%d\n\n",s_state);
 return(st_cnt);
}
/******************************************************************************/
/*                      Normalize the alpha-beta values.
*******************************************************************************/
int Normalize_Alpha_Beta(time)
int time;
{
 int state;
 double suma,sumb;
 suma=sumb=0.0;
 for(state=ZERO;state<NUM_STATES;state++)
 {
  suma=suma+Alpha[time+ONE][state];
  sumb=sumb+Beta[FRAME_SIZE-ONE-time][state];
 }
 for(state=ZERO;state<NUM_STATES;state++)
 {
  Alpha[time+ONE][state]=Alpha[time+ONE][state]/suma;
  Beta[FRAME_SIZE-ONE-time][state]=Beta[FRAME_SIZE-ONE-time][state]/sumb;
 }
 return(0);
}
/******************************************************************************/
/*                  Normalize the channel alpha-beta values.
*******************************************************************************/
int Normalize_Ch_Alpha_Beta(time)
int time;
{
 int state;
 double suma,sumb;
 suma=sumb=0.0;
 for(state=ZERO;state<NUM_ST_STATES;state++)
 {
  suma=suma+Ch_Alpha[time+ONE][state];
  sumb=sumb+Ch_Beta[CH_IV_SIZE-ONE-time][state];
 }
 for(state=ZERO;state<NUM_ST_STATES;state++)
 {
  Ch_Alpha[time+ONE][state]=Ch_Alpha[time+ONE][state]/suma;
  Ch_Beta[CH_IV_SIZE-ONE-time][state]=Ch_Beta[CH_IV_SIZE-ONE-time][state]/sumb;
 }
 return(0);
}
/******************************************************************************/
/*                    Print the alpha and beta values.
*******************************************************************************/
int Print_Alpha_Beta()
{
 int cnt;
 for(cnt=ZERO;cnt<NUM_STATES;cnt++)
 {
  printf("\t Iter:%d St:%d Alpha:%lf ",Outer_Cnt,cnt,Alpha[PRINT_TIME][cnt]);
  printf(" Beta:%lf\n",Beta[PRINT_TIME+ONE][cnt]);
 }
 return(0);
}
/******************************************************************************/
/* Complex convolution. One 'xxip1[]' is real and 'xxip2[]' are complex.
   Input arrays: 'xxip1[ip1size]' is first array.
                 'xxip2[ip2size]' is second array.
   Output array: 'xxop[]'.
*******************************************************************************/
int Convolve(reip1,imip1,reip2,imip2,reop,imop,ip1size,ip2size)
int ip1size,ip2size;
double reip1[],imip1[],reip2[],imip2[],reop[],imop[];
{
 int opcnt,ipcnt,loop;
 for(opcnt=ZERO;opcnt<(ip1size+ip2size-1);opcnt++)
 {
  reop[opcnt]=imop[opcnt]=0.0;
  ipcnt=ZERO;
  loop=TRUE;
  while((ipcnt<ip1size)&&(loop==TRUE))
  {
   if((opcnt-ipcnt)<ZERO)
    loop=FALSE;
   else if((opcnt-ipcnt)>=ip2size)
    ipcnt=opcnt-ip2size+1;
   else
   {
    reop[opcnt]=reop[opcnt]+reip1[ipcnt]*reip2[opcnt-ipcnt];
    reop[opcnt]=reop[opcnt]-imip1[ipcnt]*imip2[opcnt-ipcnt];
    imop[opcnt]=imop[opcnt]+reip1[ipcnt]*imip2[opcnt-ipcnt];
    imop[opcnt]=imop[opcnt]+imip1[ipcnt]*reip2[opcnt-ipcnt];
    ipcnt++;
   }
  }
 }
 return(0);
}
/******************************************************************************/
/*             Copy a sequence in reverse complex conjugate order.
*******************************************************************************/
int Copy_Rev_Conjugate(re_orig,im_orig,re_copy,im_copy,size)
int size;
double re_orig[],im_orig[],re_copy[],im_copy[];
{
 int cnt;
 for(cnt=ZERO;cnt<size;cnt++)
 {
  re_copy[cnt]=  re_orig[size-ONE-cnt];
  im_copy[cnt]= -im_orig[size-ONE-cnt];
 }
 return(0);
}
/******************************************************************************/
/*                 Right shift the contents of the registers.
*******************************************************************************/
int Right_Shift_Reg(re_reg,im_reg,size)
int size;
double re_reg[],im_reg[];
{
 int cnt;
 for(cnt=size-ONE;cnt>ZERO;cnt--)
 {
  re_reg[cnt]=re_reg[cnt-ONE];
  im_reg[cnt]=im_reg[cnt-ONE];
 }
 return(0);
}
/******************************************************************************/
/*                Right shift the contents of the registers.
*******************************************************************************/
int Shift_Right(array,size)
int size;
double array[];
{
 int cnt;
 for(cnt=(size-ONE);cnt>ZERO;cnt--)
  array[cnt]=array[cnt-ONE];
 return(0);
}
/******************************************************************************/
/*  Convert uniform random variables to gaussian random variables. Refer to
 *  DSP by Proakis and Manolakis.
*******************************************************************************/
int My_Gauss(tvar,tnarry)
double tvar,tnarry[];
{
 double r,theta;
 r= -2.0*tvar*log(1.0-tnarry[URADIUS]);
 if(r < FRAC_ZERO)
  Error_Mes(10);
 r=sqrt(r);
 theta=2.0*MY_PI*tnarry[UTHETA];
 tnarry[GRAND_0]=r*cos(theta);
 tnarry[GRAND_1]=r*sin(theta);
 return(0);
}
/******************************************************************************/
/*                 Check for memory allocation failure.
*******************************************************************************/
int Ck_Calloc(tmp)
void *tmp;
{
 if(tmp==NULL)
  Error_Mes(5);
 return(0);
}
/******************************************************************************/
/*                             Error message.
*******************************************************************************/
int Error_Mes(num)
int num;
{
 fprintf(stdout,"\t Simulation error. Exiting with code %d.\n\n",num);
 exit(num);
 return(0);
}
/******************************************************************************/
