/******************************************************************************/
/*   Copyright (C) 2005 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
#include "teq9type.h"
#include "teq9ext.h"
/******************************************************************************/
/*                     Do processing for decoder 1.
*******************************************************************************/
int Dec1_Process()
{
 Init_Alpha_Beta_Dec1();
 Compute_Alpha_Beta_Dec1();
 if(Outer_Cnt < (MAX_OUTER-ONE))
  Compute_APPLes_Dec1();
 else
  Compute_Final_APPs_Dec1();
 return(0);
}
/******************************************************************************/
/*                          Initialize alpha and beta.
*******************************************************************************/
int Init_Alpha_Beta_Dec1()
{
 int state;
 for(state=ZERO;state<NUM_STATES;state++)
  Alpha[ZERO][state]=Beta[FRAME_SIZE][state]=1.0;
// Alpha[ZERO][ZERO]=1.0;
// Beta[FRAME_SIZE][Enc_State]=1.0;
 return(0);
}
/******************************************************************************/
/*                   Compute the alpha and beta values.
*******************************************************************************/
int Compute_Alpha_Beta_Dec1()
{
 int time,state;
 for(time=ZERO;time<=FRAME_SIZE-TWO;time++)
 {
  for(state=ZERO;state<NUM_STATES;state++)
   Get_New_Alpha_Beta_Dec1(time,state);
  Normalize_Alpha_Beta(time);
 }
 return(0);
}
/******************************************************************************/
/*      Compute the new alpha and beta values for a given state and time.
*******************************************************************************/
int Get_New_Alpha_Beta_Dec1(time,state)
int time,state;
{
 int input,prv_st,nxt_st,opa,opb;
 double gia,gea,gib,geb,prv_alpha,nxt_beta,alpha,beta;
 alpha=beta=0.0;
 for(input=ZERO;input<NUM_INPUTS;input++)                       /* Data bits. */
 {
  prv_st=Trel_PS[state][input];
  nxt_st=Trel_NS[state][input];
  opa=Trel_Op[prv_st][input];                             /* Get encoded bit. */
  opb=Trel_Op[state][input];                              /* Get encoded bit. */
  gia=Gamma_I[time][input];                       /* Corresponds to parity 0. */
  gea=Gamma_P[time][opa];                         /* Corresponds to parity 1. */
  gib=Gamma_I[FRAME_SIZE-ONE-time][input];        /* Corresponds to parity 1. */
  geb=Gamma_P[FRAME_SIZE-ONE-time][opb];          /* Corresponds to parity 1. */
  prv_alpha=Alpha[time][prv_st];
  nxt_beta=Beta[FRAME_SIZE-ONE-time+ONE][nxt_st];
  alpha=alpha+gia*gea*prv_alpha;
  beta=beta+gib*geb*nxt_beta;
 }
 Alpha[time+ONE][state]=alpha;
 Beta[FRAME_SIZE-ONE-time][state]=beta;
 return(0);
}
/******************************************************************************/
/*           Compute the APPs due to extrinsic information alone.
*******************************************************************************/
int Compute_APPLes_Dec1()
{
 int time;
 double data[NUM_INPUTS],parity[NUM_INPUTS];
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  Get_APPLe_Dec1(time,data,parity);
  Get_Prob_Dec1(time,data,parity);
 }
 return(0);
}
/******************************************************************************/
/*   Get a posteriori probability of +1 and -1 due to extrinsic info alone.
*******************************************************************************/
int Get_APPLe_Dec1(time,data,parity)
int time;
double data[],parity[];
{
 int state,input,nxt_state,op;
 double gi,ge,prv_alpha,nxt_beta;
 for(input=ZERO;input<NUM_INPUTS;input++)
  data[input]=parity[input]=0.0;
 for(state=ZERO;state<NUM_STATES;state++)
 {
  for(input=ZERO;input<NUM_INPUTS;input++)
  {
   nxt_state=Trel_NS[state][input];                          /* 0 maps to +1. */
   op=Trel_Op[state][input];
   gi=Gamma_I[time][input];                       /* Corresponds to parity 0. */
   ge=Gamma_P[time][op];                          /* Corresponds to parity 0. */
   prv_alpha=Alpha[time][state];
   nxt_beta=Beta[time+ONE][nxt_state];
   parity[op]=parity[op]+gi*prv_alpha*nxt_beta;
   data[input]=data[input]+ge*prv_alpha*nxt_beta;
  }
 }
 return(0);
}
/******************************************************************************/
/*                     Get the normalized probabilities.
*******************************************************************************/
int Get_Prob_Dec1(time,data,parity)
int time;
double data[],parity[];
{
 int input,d_inx,p_inx;
 double sum0,sum1;
 sum0=sum1=0.0;
 d_inx=Ch_Iv_Map[TWO*time];
 p_inx=Ch_Iv_Map[TWO*time+ONE];
 for(input=ZERO;input<NUM_INPUTS;input++)
 {
  sum0=sum0+data[input];
  sum1=sum1+parity[input];
 }
 for(input=ZERO;input<NUM_INPUTS;input++)
 {
  Ch_Prob[d_inx][input]=data[input]/sum0;
  Ch_Prob[p_inx][input]=parity[input]/sum1;
  Prob[time][input]=data[input]/sum0;
 }
 return(0);
}
/******************************************************************************/
