/******************************************************************************/
/*   Copyright (C) 2005 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
#include "teq9type.h"
#include "teq9ext.h"
/******************************************************************************/
/*                          Get the trellis.
*******************************************************************************/
int Get_Super_Trellis()
{
 int num_ip,p_state,p_input,index[NUM_ST_STATES];
 int ip_array[PRED_MEM_SIZE],parity[PRED_MEM_SIZE],new_st;
 Init_S_Index(index);
 num_ip=MOD_ST_NUM_INPUTS;
 for(p_state=ZERO;p_state<NUM_STATES;p_state++)
 {
  for(p_input=ZERO;p_input<MOD_ST_NUM_INPUTS;p_input++)
  {
   Decimal_to_Binary(p_input,ip_array,PRED_MEM_SIZE);
   new_st=Fill_Mem(num_ip,p_state,p_input,PRED_MEM_SIZE,ip_array,parity);
   Get_S_Op(new_st,p_input,ip_array,parity);
   Get_S_Next_State(new_st,p_input,ip_array,index);
  }
 }
 Print_S_Trellis();
 return(0);
}
/******************************************************************************/
/*                   Initialize the indices to each state.
*******************************************************************************/
int Init_S_Index(index)
int index[];
{
 int cnt;
 for(cnt=ZERO;cnt<NUM_ST_STATES;cnt++)
  index[cnt]=ZERO;
 return(0);
}
/******************************************************************************/
/*                          Get the parity bit.
*******************************************************************************/
int Get_S_Op(new_st,p_input,ip_array,parity)
int new_st,p_input,ip_array[],parity[];
{
 int cnt,bit,enc_bit,s_state,ip;
 s_state=new_st*MOD_ST_NUM_INPUTS+p_input;
 for(ip=ZERO;ip<NUM_INPUTS;ip++)
 {
  enc_bit=Trel_Op[new_st][ip];
  STrel_Ip_Array[s_state][ip][ZERO]=ip;
  STrel_Pt_Array[s_state][ip][ZERO]=enc_bit;
  for(cnt=ZERO;cnt<PRED_MEM_SIZE;cnt++)
  {
   STrel_Ip_Array[s_state][ip][cnt+ONE]=ip_array[cnt];
   STrel_Pt_Array[s_state][ip][cnt+ONE]=parity[cnt];
  }
 }
 return(0);
}
/******************************************************************************/
/*                           Get the next state.
*******************************************************************************/
int Get_S_Next_State(new_st,p_input,ip_array,index)
int new_st,p_input,ip_array[],index[];
{
 int pred_nxt_state,enc_state,cnt,s_state,ip,new_s_state;
 int inx,tip[PRED_MEM_SIZE];
 for(ip=ZERO;ip<NUM_INPUTS;ip++)
 {
  enc_state=Trel_NS[new_st][ip];
  tip[ZERO]=ip;
  for(cnt=ZERO;cnt<PRED_MEM_SIZE-ONE;cnt++)
   tip[cnt+ONE]=ip_array[cnt];
  pred_nxt_state=Binary_to_Decimal(tip,PRED_MEM_SIZE);
  s_state=new_st*MOD_ST_NUM_INPUTS+p_input;
  new_s_state=enc_state*MOD_ST_NUM_INPUTS+pred_nxt_state;
  inx=index[new_s_state]++;
  if(inx >= NUM_INPUTS)
   Error_Mes(1);
  STrel_NS[s_state][ip]=new_s_state;
  STrel_PS[new_s_state][inx]=s_state;
  STrel_Ip[new_s_state][inx]=ip;
 }
 return(0);
}
/******************************************************************************/
/*                          Print the super trellis.
*******************************************************************************/
int Print_S_Trellis()
{
 int p_state,p_input,cnt;
 printf("\t Channel trellis:\n\n");
 for(p_state=ZERO;p_state<NUM_ST_STATES;p_state++)
 {
  for(p_input=ZERO;p_input<NUM_INPUTS;p_input++)
  {
   printf("\t St:%3d Ip:%d ",p_state,p_input);
   printf("N. St:%3d ",STrel_NS[p_state][p_input]);
   printf("Prv. St:%3d ",STrel_PS[p_state][p_input]);
   printf("Input to st:%d\n",STrel_Ip[p_state][p_input]);
   printf("\t\t Input array:");
   for(cnt=ZERO;cnt<PRED_SIZE;cnt++)
    printf("%d",STrel_Ip_Array[p_state][p_input][cnt]);
   printf("\n");
   printf("\t\t Parity array:");
   for(cnt=ZERO;cnt<PRED_SIZE;cnt++)
    printf("%d",STrel_Pt_Array[p_state][p_input][cnt]);
   printf("\n");
  }
 }
 printf("\n");
 return(0);
}
/******************************************************************************/
