/******************************************************************************/
/*   Copyright (C) 2005 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
#include "teqdfsetype.h"
#include "teqdfseext.h"
/******************************************************************************/
/*    Compute the a posteriori probabilities for the channel MAP decoder.
*******************************************************************************/
int Compute_Ch_APP()
{
 Init_Alpha_Beta_Ch();
 Compute_Alpha_Beta_Ch();
 Compute_APPLes_Ch();
 return(0);
}
/******************************************************************************/
/*                          Initialize alpha and beta.
*******************************************************************************/
int Init_Alpha_Beta_Ch()
{
 int state;
 for(state=ZERO;state<RDU_CH_STATES;state++)
  Ch_Alpha[ZERO][state]=Ch_Beta[CH_IV_SIZE][state]=1.0;
 return(0);
}
/******************************************************************************/
/*                   Compute the alpha and beta values.
*******************************************************************************/
int Compute_Alpha_Beta_Ch()
{
 int time,state;
 for(time=ZERO;time<=CH_IV_SIZE-TWO;time++)
 {
  for(state=ZERO;state<RDU_CH_STATES;state++)
  Ch_Alpha[time+ONE][state]=Ch_Beta[CH_IV_SIZE-ONE-time][state]=0.0;
  for(state=ZERO;state<NUM_CH_STATES;state++)
   Get_New_Ch_Alpha_Beta(time,state);
  Normalize_Ch_Alpha_Beta(time);
 }
 return(0);
}
/******************************************************************************/
/*      Compute the new alpha and beta values for a given state and time.
*******************************************************************************/
int Get_New_Ch_Alpha_Beta(time,state)
int time,state;
{
 int input,prv_st,nxt_st,st_ip,mod_prv_st,mod_nxt_st,mod_st;
 double grea,gima,greb,gimb,proba,probb,prv_alpha,nxt_beta,alpha,beta;
 alpha=beta=0.0;
 mod_st=Get_Reduced_State(state);
 for(input=ZERO;input<NUM_INPUTS;input++)                       /* Data bits. */
 {
  prv_st=Ch_Trel_PS[state][input];
  nxt_st=Ch_Trel_NS[state][input];
  mod_prv_st=Get_Reduced_State(prv_st);
  mod_nxt_st=Get_Reduced_State(nxt_st);
  st_ip=Ch_Trel_Ip[state][input];                          /* Input to state. */
  grea=Re_Ch_Gamma[time][prv_st][st_ip];                  /* Gamma for alpha. */
  gima=Im_Ch_Gamma[time][prv_st][st_ip];                  /* Gamma for alpha. */
  greb=Re_Ch_Gamma[CH_IV_SIZE-ONE-time][state][input];     /* Gamma for beta. */
  gimb=Im_Ch_Gamma[CH_IV_SIZE-ONE-time][state][input];     /* Gamma for beta. */
  proba=Ch_Prob[time][st_ip];
  probb=Ch_Prob[CH_IV_SIZE-ONE-time][input];
  prv_alpha=Ch_Alpha[time][mod_prv_st];
  nxt_beta=Ch_Beta[CH_IV_SIZE-ONE-time+ONE][mod_nxt_st];
  alpha=alpha+grea*gima*proba*prv_alpha;
  beta=beta+greb*gimb*probb*nxt_beta;
 }
 Ch_Alpha[time+ONE][mod_st]=Ch_Alpha[time+ONE][mod_st]+alpha;
 Ch_Beta[CH_IV_SIZE-ONE-time][mod_st]=Ch_Beta[CH_IV_SIZE-ONE-time][mod_st]+beta;
 return(0);
}
/******************************************************************************/
/*           Compute the APPs due to extrinsic information alone.
*******************************************************************************/
int Compute_APPLes_Ch()
{
 int time;
 double app[NUM_INPUTS];
 for(time=ZERO;time<CH_IV_SIZE;time++)
 {
  Get_APPLe_Ch(time,app);
  Get_Prob_Ch(time,app);
 }
 return(0);
}
/******************************************************************************/
/*   Get a posteriori probability of +1 and -1 due to extrinsic info alone.
*******************************************************************************/
int Get_APPLe_Ch(time,app)
int time;
double app[];
{
 int state,input,nxt_state,mod_st,mod_nxt_st;
 double prv_alpha,nxt_beta,regamma,imgamma;
 app[ZERO]=app[ONE]=0.0;
 for(state=ZERO;state<NUM_CH_STATES;state++)
 {
  mod_st=Get_Reduced_State(state);
  for(input=ZERO;input<NUM_INPUTS;input++)
  {
   nxt_state=Ch_Trel_NS[state][input];
   mod_nxt_st=Get_Reduced_State(nxt_state);
   prv_alpha=Ch_Alpha[time][mod_st];
   nxt_beta=Ch_Beta[time+ONE][mod_nxt_st];
   regamma=Re_Ch_Gamma[time][state][input];
   imgamma=Im_Ch_Gamma[time][state][input];
   app[input]=app[input]+prv_alpha*nxt_beta*regamma*imgamma;
  }
 }
 return(0);
}
/******************************************************************************/
/*                     Get the normalized probabilities.
*******************************************************************************/
int Get_Prob_Ch(time,app)
int time;
double app[];
{
 int input;
 double sum;
 sum=0.0;
 for(input=ZERO;input<NUM_INPUTS;input++)
  sum=sum+app[input];
 for(input=ZERO;input<NUM_INPUTS;input++)
  Ch_Prob[time][input]=app[input]/sum;
 return(0);
}
/******************************************************************************/
