/******************************************************************************/
/*   Copyright (C) 2005 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
#include "teqdfsetype.h"
#include "teqdfseext.h"
/******************************************************************************/
int Simulation()
{
 for(Frame_Cnt=ZERO;Frame_Cnt<MAX_FRAMES;Frame_Cnt++)
 {
  Get_Frame();
  Get_Channel_Op();
  Add_Noise();
  Receive();
 }
 return(0);
}
/******************************************************************************/
/*                  Get the samples at the channel output.
*******************************************************************************/
int Get_Channel_Op()
{
 int time,iv_ip_time,bit,ch_state;
 ch_state=ZERO;
 for(time=ZERO;time<CH_IV_SIZE;time++)
 {
  iv_ip_time=Ch_De_Iv_Map[time];
  bit=Enc_Op[iv_ip_time];
  Re_Chan_Op[time]=Re_Ch_Trel_Op[ch_state][bit];
  Im_Chan_Op[time]=Im_Ch_Trel_Op[ch_state][bit];
  ch_state=Ch_Trel_NS[ch_state][bit];
 }
 Chan_F_State=ch_state;
 return(0);
}
/******************************************************************************/
/*                        Add white Gaussian noise.
*******************************************************************************/
int Add_Noise()
{
 int time;
 double erand48();
 for(time=ZERO;time<CH_IV_SIZE;time++)
 {
  N1_Val[URADIUS]=erand48(N1_Store);
  N1_Val[UTHETA]=erand48(N2_Store);
  My_Gauss(Variance,N1_Val);
  Re_Chan_Op[time]=Re_Chan_Op[time]+N1_Val[GRAND_0];
  Im_Chan_Op[time]=Im_Chan_Op[time]+N1_Val[GRAND_1];
 }
 return(0);
}
/******************************************************************************/
/*                         Receiver routines.
*******************************************************************************/
int Receive()
{
 Init_Chan_Dec();
 Init_Chan_Prob();
 for(Outer_Cnt=ZERO;Outer_Cnt<MAX_OUTER;Outer_Cnt++)
 {
  Compute_Ch_APP();
  Init_Gamma_Turbo_Dec();
  Dec1_Process();
//  Get_Frame_Error();
 }
 Get_Bit_Errors();
 return(0);
}
/******************************************************************************/
/*                      Initialize the channel decoder.
*******************************************************************************/
int Init_Chan_Dec()
{
 int time,state,ip;
 double x,y;
 for(time=ZERO;time<CH_IV_SIZE;time++)
 {
  for(state=ZERO;state<NUM_CH_STATES;state++)
  {
   for(ip=ZERO;ip<NUM_INPUTS;ip++)
   {
    x=Re_Chan_Op[time]-Re_Ch_Trel_Op[state][ip];
    y=Im_Chan_Op[time]-Im_Ch_Trel_Op[state][ip];
    x= -x*x*Inv_Twice_Noise_Var;
    y= -y*y*Inv_Twice_Noise_Var;
    Re_Ch_Gamma[time][state][ip]=exp(x);
    Im_Ch_Gamma[time][state][ip]=exp(y);
   }
  }
 }
 return(0);
}
/******************************************************************************/
/*            Get the a priori probabilities at the precoder input.
*******************************************************************************/
int Init_Chan_Prob()
{
 int time;
 for(time=ZERO;time<CH_IV_SIZE;time++)
  Ch_Prob[time][ZERO]=Ch_Prob[time][ONE]=0.5;
 return(0);
}
/******************************************************************************/
/*          Initialize the intrinsic and extrinsic values of gamma.
*******************************************************************************/
int Init_Gamma_Turbo_Dec()
{
 Init_Gamma_I_E();
// Init_Noisy_Gamma_I_E();
// Get_Ch_MAP_Op_Err();
 return(0);
}
/******************************************************************************/
/*  Initialize the intrinsic and extrinsic values of gamma for decoder 1 and
 *  two.
*******************************************************************************/
int Init_Gamma_I_E()
{
 int time,bit,d_inx,p_inx;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  d_inx=Ch_Iv_Map[TWO*time];
  p_inx=Ch_Iv_Map[TWO*time+ONE];
  for(bit=ZERO;bit<NUM_INPUTS;bit++)
  {
   Gamma_I[time][bit]=Ch_Prob[d_inx][bit];
   Gamma_P[time][bit]=Ch_Prob[p_inx][bit];
  }
 }
 return(0);
}
/******************************************************************************/
/*        Get the errors in a frame at the channel MAP detector output.
*******************************************************************************/
int Get_Ch_MAP_Op_Err()
{
 int time,d_err,p_err,d_bit,p_bit;
 d_err=p_err=ZERO;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  d_bit=p_bit=ZERO;
  if(Gamma_I[time][ONE] > Gamma_I[time][ZERO])
   d_bit=ONE;
  if(d_bit != Enc_Op[TWO*time])
   d_err++;
  if(Gamma_P[time][ONE] > Gamma_P[time][ZERO])
   p_bit=ONE;
  if(p_bit != Enc_Op[TWO*time+ONE])
   p_err++;
 }
 printf("fr:%d derr:%d perr:%d\n",Frame_Cnt,d_err,p_err);
 return(0);
}
/******************************************************************************/
/*              Initialize the MAP decoder with noisy inputs.
*******************************************************************************/
int Init_Noisy_Gamma_I_E()
{
 int time;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  Init_Gamma_P0(time);
  Init_Gamma_P1(time);
 }
 return(0);
}
/******************************************************************************/
/*                         Get noisy parity 1 inputs.
*******************************************************************************/
int Init_Gamma_P0(time)
int time;
{
 int bit;
 double sample,x;
 double erand48();
 sample=Bit_Map[Enc_Op[TWO*time]];
 N1_Val[URADIUS]=erand48(N1_Store);
 N1_Val[UTHETA]=erand48(N2_Store);
 My_Gauss(Variance,N1_Val);
 sample=sample+N1_Val[GRAND_0];
 for(bit=ZERO;bit<NUM_INPUTS;bit++)
 {
  x=sample-Bit_Map[bit];
  x= -x*x*Inv_Twice_Noise_Var;
  Gamma_I[time][bit]=exp(x);
 }
 return(0);
}
/******************************************************************************/
/*                         Get noisy parity 2 inputs.
*******************************************************************************/
int Init_Gamma_P1(time)
int time;
{
 int bit;
 double sample,x;
 double erand48();
 sample=Bit_Map[Enc_Op[TWO*time+ONE]];
 N1_Val[URADIUS]=erand48(N1_Store);
 N1_Val[UTHETA]=erand48(N2_Store);
 My_Gauss(Variance,N1_Val);
 sample=sample+N1_Val[GRAND_0];
 for(bit=ZERO;bit<NUM_INPUTS;bit++)
 {
  x=sample-Bit_Map[bit];
  x= -x*x*Inv_Twice_Noise_Var;
  Gamma_P[time][bit]=exp(x);
 }
 return(0);
}
/******************************************************************************/
/*               Get the number of errors in the current frame.
*******************************************************************************/
int Get_Frame_Error()
{
 int time,d_inx,p_inx,d_err,p_err,d_bit,p_bit;
 d_err=p_err=ZERO;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  d_inx=Ch_Iv_Map[TWO*time];
  p_inx=Ch_Iv_Map[TWO*time+ONE];
  d_bit=p_bit=ZERO;
  if(Ch_Prob[d_inx][ONE] > Ch_Prob[d_inx][ZERO])
   d_bit=ONE;
  if(d_bit != Enc_Op[TWO*time])
   d_err++;
  if(Ch_Prob[p_inx][ONE] > Ch_Prob[p_inx][ZERO])
   p_bit=ONE;
  if(p_bit != Enc_Op[TWO*time+ONE])
   p_err++;
 }
 printf("br:%d derr:%d perr:%d\n",Frame_Cnt,d_err,p_err);
 return(0);
}
/******************************************************************************/
/*           Finally compute the number of bit errors in the frame.
*******************************************************************************/
int Get_Bit_Errors()
{
 int time,detected_bit;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  detected_bit=ONE;
  if(Prob[time][ZERO] > Prob[time][ONE])
   detected_bit=ZERO;
  if(detected_bit != Bits[time])
   Ber++;
 }
 return(0);
}
/******************************************************************************/
