/******************************************************************************/
/*   Copyright (C) 2005 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
#include "teqdfsetype.h"
#include "teqdfseext.h"
/******************************************************************************/
/*                         Convert decimal to binary.
*******************************************************************************/
int Decimal_to_Binary(num,array,array_size)
int num,array[],array_size;
{
 int cnt;
 for(cnt=ZERO;cnt<array_size;cnt++)
  array[cnt]=ZERO;
 cnt=ZERO;
 while(num>ZERO)
 {
  if(cnt == array_size)
   Error_Mes(1);
  array[cnt]=num&1;
  num=num>>1;
  cnt++;
 }
 return(0);
}
/******************************************************************************/
/*                    Convert binary array to decimal.
*******************************************************************************/
int Binary_to_Decimal(array,size)
int array[],size;
{
 int cnt,pow,sum;
 sum=ZERO;
 pow=1;
 for(cnt=ZERO;cnt<size;cnt++)
 {
  sum=sum+pow*array[cnt];
  pow=pow<<1;
 }
 return(sum);
}
/******************************************************************************/
/* Convert the state given in decimal into modulo-M and return the result in
 * the array 'stmary[]'.
*******************************************************************************/
int Decimal_To_M_ary(stcnt,stmary,size,M)
int stcnt,stmary[],size,M;
{
 int index;
 for(index=ZERO;index<size;index++)
  stmary[index]=ZERO;
 index=ZERO;
 while(stcnt>ZERO)
 {
  if(index == size)
   Error_Mes(1);
  stmary[index]=stcnt%M;
  stcnt=stcnt/M;
  index++;
 }
 return(0);
}
/******************************************************************************/
/*                       Convert M-ary to decimal.
*******************************************************************************/
int Mary_to_Decimal(stmary,size,M)
int stmary[],size,M;
{
 int cnt,power,decimal;
 power=ONE;
 decimal=ZERO;
 for(cnt=ZERO;cnt<size;cnt++)
 {
  decimal=decimal+stmary[cnt]*power;
  power=power*M;
 }
 return(decimal);
}
/******************************************************************************/
/*                      Normalize the alpha-beta values.
*******************************************************************************/
int Normalize_Alpha_Beta(time)
int time;
{
 int state;
 double suma,sumb;
 suma=sumb=0.0;
 for(state=ZERO;state<NUM_STATES;state++)
 {
  suma=suma+Alpha[time+ONE][state];
  sumb=sumb+Beta[FRAME_SIZE-ONE-time][state];
 }
 for(state=ZERO;state<NUM_STATES;state++)
 {
  Alpha[time+ONE][state]=Alpha[time+ONE][state]/suma;
  Beta[FRAME_SIZE-ONE-time][state]=Beta[FRAME_SIZE-ONE-time][state]/sumb;
 }
 return(0);
}
/******************************************************************************/
/*                  Normalize the channel alpha-beta values.
*******************************************************************************/
int Normalize_Ch_Alpha_Beta(time)
int time;
{
 int state;
 double suma,sumb;
 suma=sumb=0.0;
 for(state=ZERO;state<RDU_CH_STATES;state++)
 {
  suma=suma+Ch_Alpha[time+ONE][state];
  sumb=sumb+Ch_Beta[CH_IV_SIZE-ONE-time][state];
 }
 for(state=ZERO;state<RDU_CH_STATES;state++)
 {
  Ch_Alpha[time+ONE][state]=Ch_Alpha[time+ONE][state]/suma;
  Ch_Beta[CH_IV_SIZE-ONE-time][state]=Ch_Beta[CH_IV_SIZE-ONE-time][state]/sumb;
 }
 return(0);
}
/******************************************************************************/
/*                    Print the alpha and beta values.
*******************************************************************************/
int Print_Alpha_Beta()
{
 int cnt;
 for(cnt=ZERO;cnt<NUM_STATES;cnt++)
 {
  printf("\t Iter:%d St:%d Alpha:%lf ",Inner_Cnt,cnt,Alpha[PRINT_TIME][cnt]);
  printf(" Beta:%lf\n",Beta[PRINT_TIME+ONE][cnt]);
 }
 return(0);
}
/******************************************************************************/
/*                         Get the reduced state.
*******************************************************************************/
int Get_Reduced_State(state)
int state;
{
 int enc_st,reduced_st,ch_st;
 enc_st=state/MOD_NUM_INPUTS;
 ch_st=state%MOD_NUM_INPUTS;
 reduced_st=enc_st*MOD1_NUM_INPUTS+ch_st%MOD1_NUM_INPUTS;
 if(reduced_st >= RDU_CH_STATES)
  Error_Mes(11);
 return(reduced_st);
}
/******************************************************************************/
/*                Right shift the contents of the registers.
*******************************************************************************/
int Shift_Right(array,size)
int size;
double array[];
{
 int cnt;
 for(cnt=(size-ONE);cnt>ZERO;cnt--)
  array[cnt]=array[cnt-ONE];
 return(0);
}
/******************************************************************************/
/*  Convert uniform random variables to gaussian random variables. Refer to
 *  DSP by Proakis and Manolakis.
*******************************************************************************/
int My_Gauss(tvar,tnarry)
double tvar,tnarry[];
{
 double r,theta;
 r= -2.0*tvar*log(1.0-tnarry[URADIUS]);
 if(r < FRAC_ZERO)
  Error_Mes(10);
 r=sqrt(r);
 theta=2.0*MY_PI*tnarry[UTHETA];
 tnarry[GRAND_0]=r*cos(theta);
 tnarry[GRAND_1]=r*sin(theta);
 return(0);
}
/******************************************************************************/
/*                 Check for memory allocation failure.
*******************************************************************************/
int Ck_Calloc(tmp)
void *tmp;
{
 if(tmp==NULL)
  Error_Mes(5);
 return(0);
}
/******************************************************************************/
/*                             Error message.
*******************************************************************************/
int Error_Mes(num)
int num;
{
 fprintf(stdout,"\t Simulation error. Exiting with code %d.\n\n",num);
 exit(num);
 return(0);
}
/******************************************************************************/
