/******************************************************************************/
/*   Copyright (C) 2005 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
#include "teqex2type.h"
#include "teqex2ext.h"
/******************************************************************************/
/*                     Do processing for decoder 1.
*******************************************************************************/
int Dec1_Process()
{
 Init_Alpha_Beta_Dec1();
 Compute_Alpha_Beta_Dec1();
 Compute_APPLes_Dec1();
 return(0);
}
/******************************************************************************/
/*                          Initialize alpha and beta.
*******************************************************************************/
int Init_Alpha_Beta_Dec1()
{
 int state;
 for(state=ZERO;state<NUM_STATES;state++)
  Alpha[ZERO][state]=Beta[FRAME_SIZE][state]=1.0;
 return(0);
}
/******************************************************************************/
/*                   Compute the alpha and beta values.
*******************************************************************************/
int Compute_Alpha_Beta_Dec1()
{
 int time,state;
 for(time=ZERO;time<=FRAME_SIZE-TWO;time++)
 {
  for(state=ZERO;state<NUM_STATES;state++)
   Get_New_Alpha_Beta_Dec1(time,state);
  Normalize_Alpha_Beta(time);
 }
 return(0);
}
/******************************************************************************/
/*      Compute the new alpha and beta values for a given state and time.
*******************************************************************************/
int Get_New_Alpha_Beta_Dec1(time,state)
int time,state;
{
 int input,prv_st,nxt_st,opa,opb,ip;
 double gea0,gea1,geb0,geb1,proba,probb,prv_alpha,nxt_beta,alpha,beta;
 alpha=beta=0.0;
 for(input=ZERO;input<NUM_INPUTS;input++)                       /* Data bits. */
 {
  ip=Trel_Ip[state];
  prv_st=Trel_PS[state][input];
  nxt_st=Trel_NS[state][input];
  opa=Trel_Op[prv_st][ip];                             /* Get encoded bit. */
  opb=Trel_Op[state][input];                              /* Get encoded bit. */
  gea0=Gamma_P0[time][opa%TWO];                   /* Corresponds to parity 0. */
  gea1=Gamma_P1[time][opa/TWO];                   /* Corresponds to parity 1. */
  geb0=Gamma_P0[FRAME_SIZE-ONE-time][opb%TWO];    /* Corresponds to parity 1. */
  geb1=Gamma_P1[FRAME_SIZE-ONE-time][opb/TWO];    /* Corresponds to parity 1. */
  proba=Prob[time][ip];
  probb=Prob[FRAME_SIZE-ONE-time][input];
  prv_alpha=Alpha[time][prv_st];
  nxt_beta=Beta[FRAME_SIZE-ONE-time+ONE][nxt_st];
  alpha=alpha+gea0*gea1*proba*prv_alpha;
  beta=beta+geb0*geb1*probb*nxt_beta;
 }
 Alpha[time+ONE][state]=alpha;
 Beta[FRAME_SIZE-ONE-time][state]=beta;
 return(0);
}
/******************************************************************************/
/*           Compute the APPs due to extrinsic information alone.
*******************************************************************************/
int Compute_APPLes_Dec1()
{
 int time;
 double data[NUM_INPUTS],p0[NUM_INPUTS],p1[NUM_INPUTS];
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  Get_APPLe_Dec1(time,p0,p1);
  Get_Prob_Dec1(time,p0,p1);
 }
 return(0);
}
/******************************************************************************/
/*   Get a posteriori probability of +1 and -1 due to extrinsic info alone.
*******************************************************************************/
int Get_APPLe_Dec1(time,p0,p1)
int time;
double p0[],p1[];
{
 int state,input,nxt_state,op;
 double ge0,ge1,prv_alpha,nxt_beta;
 for(input=ZERO;input<NUM_INPUTS;input++)
  p0[input]=p1[input]=0.0;
 for(state=ZERO;state<NUM_STATES;state++)
 {
  for(input=ZERO;input<NUM_INPUTS;input++)
  {
   nxt_state=Trel_NS[state][input];                          /* 0 maps to +1. */
   op=Trel_Op[state][input];
   ge0=Gamma_P0[time][op%TWO];                    /* Corresponds to parity 0. */
   ge1=Gamma_P1[time][op/TWO];                    /* Corresponds to parity 0. */
   prv_alpha=Alpha[time][state];
   nxt_beta=Beta[time+ONE][nxt_state];
   p0[op%TWO]=p0[op%TWO]+ge1*prv_alpha*nxt_beta;
   p1[op/TWO]=p1[op/TWO]+ge0*prv_alpha*nxt_beta;
  }
 }
 return(0);
}
/******************************************************************************/
/*                     Get the normalized probabilities.
*******************************************************************************/
int Get_Prob_Dec1(time,p0,p1)
int time;
double p0[],p1[];
{
 int input,p0_inx,p1_inx;
 double sum0,sum1,sum_data;
 sum0=sum1=sum_data=0.0;
 p0_inx=Ch_Iv_Map[TWO*time];
 p1_inx=Ch_Iv_Map[TWO*time+ONE];
 for(input=ZERO;input<NUM_INPUTS;input++)
 {
  sum0=sum0+p0[input];
  sum1=sum1+p1[input];
 }
 for(input=ZERO;input<NUM_INPUTS;input++)
 {
  Ch_Prob[p0_inx][input]=p0[input]/sum0;
  Ch_Prob[p1_inx][input]=p1[input]/sum1;
 }
 return(0);
}
/******************************************************************************/
