/******************************************************************************/
/*   Copyright (C) 2005 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
#include "teqex2type.h"
#include "teqex2ext.h"
/******************************************************************************/
int Simulation()
{
 for(Snr=MIN_SNR;Snr<MAX_SNR;Snr=Snr+SNR_STEP)
 {
  Get_LLR_Variance();
  Init_LLR();
  for(Frame_Cnt=ZERO;Frame_Cnt<MAX_FRAMES;Frame_Cnt++)
  {
   Get_Frame();
   Get_Channel_Op();
   Add_Noise();
   Add_LLR_Noise();
   Receive();
   if(Frame_Cnt == MEAN_TIME)
    Compute_Mean();
  }
  Compute_Mutual_Info();
 }
 return(0);
}
/******************************************************************************/
/*                        Get the noise variance.
*******************************************************************************/
int Get_LLR_Variance()
{
 LLR_Variance=1.0/pow(10.0,0.1*Snr);
 printf("\t LLR Noise variance:%lf.\n\n",LLR_Variance);
 Inv_Twice_LLR_Var=1.0/(2.0*LLR_Variance);
 return(0);
}
/******************************************************************************/
/*                 Initialize the log-likelihood ratios.
*******************************************************************************/
int Init_LLR()
{
 int cnt;
 for(cnt=ZERO;cnt<NUM_INPUTS;cnt++)
  Ch_LLR[cnt]=CC_LLR[cnt]=0.0;
 Ch_Var=CC_Var=0.0;
 Min_Ch_LLR=Min_CC_LLR=MAX_MAG;
 Max_Ch_LLR=Max_CC_LLR=MIN_MAG;
 for(cnt=ZERO;cnt<HIST_SIZE;cnt++)
  Ch_Freq[cnt]=CC_Freq[cnt]=0.0;
 return(0);
}
/******************************************************************************/
/*                  Get the samples at the channel output.
*******************************************************************************/
int Get_Channel_Op()
{
 int time,iv_ip_time,bit,ch_state;
 ch_state=ZERO;
 for(time=ZERO;time<CH_IV_SIZE;time++)
 {
  iv_ip_time=Ch_De_Iv_Map[time];
  bit=Enc_Op[iv_ip_time];
  Chan_Op[time]=Ch_Trel_Op[ch_state][bit];
  ch_state=Ch_Trel_NS[ch_state][bit];
 }
 Chan_F_State=ch_state;
 return(0);
}
/******************************************************************************/
/*                        Add white Gaussian noise.
*******************************************************************************/
int Add_Noise()
{
 int time;
 double erand48();
 for(time=ZERO;time<CH_IV_SIZE;time=time+TWO)
 {
  N1_Val[URADIUS]=erand48(N1_Store);
  N1_Val[UTHETA]=erand48(N2_Store);
  My_Gauss(Variance,N1_Val);
  Chan_Op[time]=Chan_Op[time]+N1_Val[GRAND_0];
  Chan_Op[time+ONE]=Chan_Op[time+ONE]+N1_Val[GRAND_1];
 }
 return(0);
}
/******************************************************************************/
/*  Add white Gaussian noise to parity symbols for conversion to a priori
 *  probabilities.
*******************************************************************************/
int Add_LLR_Noise()
{
 int time;
 double erand48();
 for(time=ZERO;time<CH_IV_SIZE;time=time+TWO)
 {
  N1_Val[URADIUS]=erand48(N1_Store);
  N1_Val[UTHETA]=erand48(N2_Store);
  My_Gauss(LLR_Variance,N1_Val);
  P_Sym[time]=P_Sym[time]+N1_Val[GRAND_0];
  P_Sym[time+ONE]=P_Sym[time+ONE]+N1_Val[GRAND_1];
 }
 return(0);
}
/******************************************************************************/
/*                         Receiver routines.
*******************************************************************************/
int Receive()
{
 Init_Chan_Dec();
 Init_Gamma_Turbo_Dec();
 Compute_Ch_APP();
 Compute_Ch_LLR();
 Init_App();
 Dec1_Process();
 Compute_CC_LLR();
 return(0);
}
/******************************************************************************/
/*                      Initialize the channel decoder.
*******************************************************************************/
int Init_Chan_Dec()
{
 int time,state,ip;
 double x;
 for(time=ZERO;time<CH_IV_SIZE;time++)
 {
  for(state=ZERO;state<NUM_CH_STATES;state++)
  {
   for(ip=ZERO;ip<NUM_INPUTS;ip++)
   {
    x=Chan_Op[time]-Ch_Trel_Op[state][ip];
    x= -x*x*Inv_Twice_Noise_Var;
    Ch_Gamma[time][state][ip]=exp(x);
   }
  }
 }
 return(0);
}
/******************************************************************************/
/*    Compute the log-likelihood ratios at the channel MAP decoder output.
*******************************************************************************/
int Compute_Ch_LLR()
{
 int time,iv_ip_time,bit;
 double llr,ub_llr;
 for(time=ZERO;time<CH_IV_SIZE;time++)
 {
  llr=log(Ch_Prob[time][ZERO]/Ch_Prob[time][ONE]);
  iv_ip_time=Ch_De_Iv_Map[time];
  bit=Enc_Op[iv_ip_time];
  Ch_LLR[bit]=Ch_LLR[bit]+llr;
  if(Frame_Cnt <= MEAN_TIME)
   Update_Ch_Extrema(llr);
  if(Frame_Cnt > MEAN_TIME)
  {
   ub_llr=llr-Ch_Mean*Bit_Map[bit];
   Ch_Var=Ch_Var+ub_llr*ub_llr;
   Update_Ch_Hist(ub_llr);
  }
 }
 return(0);
}
/******************************************************************************/
/*               Update the maxima and minima of the channel LLR.
*******************************************************************************/
int Update_Ch_Extrema(llr)
double llr;
{
 if(llr < Min_Ch_LLR)
  Min_Ch_LLR=llr;
 if(llr > Max_Ch_LLR)
  Max_Ch_LLR=llr;
 return(0);
}
/******************************************************************************/
/*                      Update the channel LLR histogram.
*******************************************************************************/
int Update_Ch_Hist(ub_llr)
double ub_llr;
{
 int inx;
 double min;
 min=Min_Ch_LLR+Ch_Mean;
 inx=(int)((ub_llr-min)/Ch_Hist_Step);
 if(inx < ZERO)
  inx=ZERO;
 if(inx >= HIST_SIZE)
  inx=HIST_SIZE-ONE;
 Ch_Freq[inx]++;
 return(0);
}
/******************************************************************************/
/*              Initialize the MAP decoder with noisy inputs.
*******************************************************************************/
int Init_Gamma_Turbo_Dec()
{
 int time;
 for(time=ZERO;time<FRAME_SIZE;time++)
  Init_Gamma_P0_P1(time);
 return(0);
}
/******************************************************************************/
/*                   Get noisy parity 1 and parity 2 inputs.
*******************************************************************************/
int Init_Gamma_P0_P1(time)
int time;
{
 int bit,iv_inx_p0,iv_inx_p1;
 double x,y;
 iv_inx_p0=Ch_Iv_Map[TWO*time];
 iv_inx_p1=Ch_Iv_Map[TWO*time+ONE];
 for(bit=ZERO;bit<NUM_INPUTS;bit++)
 {
  x=P_Sym[TWO*time]-Bit_Map[bit];
  y=P_Sym[TWO*time+ONE]-Bit_Map[bit];
  x= -x*x*Inv_Twice_LLR_Var;
  y= -y*y*Inv_Twice_LLR_Var;
  Ch_Prob[iv_inx_p0][bit]=Gamma_P0[time][bit]=exp(x);
  Ch_Prob[iv_inx_p1][bit]=Gamma_P1[time][bit]=exp(y);
 }
 Normalize_Ch_Prob(iv_inx_p0,iv_inx_p1);
 return(0);
}
/******************************************************************************/
/*                Normalize the a priori channel probabilities.
*******************************************************************************/
int Normalize_Ch_Prob(iv_inx_p0,iv_inx_p1)
int iv_inx_p0,iv_inx_p1;
{
 int bit;
 double sum0,sum1;
 sum0=sum1=0.0;
 for(bit=ZERO;bit<NUM_INPUTS;bit++)
 {
  sum0=sum0+Ch_Prob[iv_inx_p0][bit];
  sum1=sum1+Ch_Prob[iv_inx_p1][bit];
 }
 for(bit=ZERO;bit<NUM_INPUTS;bit++)
 {
  Ch_Prob[iv_inx_p0][bit]=Ch_Prob[iv_inx_p0][bit]/sum0;
  Ch_Prob[iv_inx_p1][bit]=Ch_Prob[iv_inx_p1][bit]/sum1;
 }
 return(0);
}
/******************************************************************************/
/*                Initialize the a posteriori probabilities.
*******************************************************************************/
int Init_App()
{
 int time,input;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  for(input=ZERO;input<NUM_INPUTS;input++)
   Prob[time][input]=0.5;
 }
 return(0);
}
/******************************************************************************/
/*      Compute the log-likelihood ratios at the CC MAP decoder output.
*******************************************************************************/
int Compute_CC_LLR()
{
 int time,iv_ip_time,bit;
 double llr,ub_llr;
 for(time=ZERO;time<CH_IV_SIZE;time++)
 {
  llr=log(Ch_Prob[time][ZERO]/Ch_Prob[time][ONE]);
  iv_ip_time=Ch_De_Iv_Map[time];
  bit=Enc_Op[iv_ip_time];
  CC_LLR[bit]=CC_LLR[bit]+llr;
  if(Frame_Cnt <= MEAN_TIME)
   Update_CC_Extrema(llr);
  if(Frame_Cnt > MEAN_TIME)
  {
   ub_llr=llr-CC_Mean*Bit_Map[bit];
   CC_Var=CC_Var+ub_llr*ub_llr;
   Update_CC_Hist(ub_llr);
  }
 }
 return(0);
}
/******************************************************************************/
/*                Update the maxima and minima of the CC LLR.
*******************************************************************************/
int Update_CC_Extrema(llr)
double llr;
{
 if(llr < Min_CC_LLR)
  Min_CC_LLR=llr;
 if(llr > Max_CC_LLR)
  Max_CC_LLR=llr;
 return(0);
}
/******************************************************************************/
/*                       Update the CC LLR histogram.
*******************************************************************************/
int Update_CC_Hist(ub_llr)
double ub_llr;
{
 int inx;
 double min;
 min=Min_CC_LLR+CC_Mean;
 inx=(int)((ub_llr-min)/CC_Hist_Step);
 if(inx < ZERO)
  inx=ZERO;
 if(inx >= HIST_SIZE)
  inx=HIST_SIZE-ONE;
 CC_Freq[inx]++;
 return(0);
}
/******************************************************************************/
/*                   Compute the mean value of the LLRs.
*******************************************************************************/
int Compute_Mean()
{
 Ch_Mean=2.0*(Ch_LLR[ZERO]-Ch_LLR[ONE])/(double)(MAX_FRAMES*CH_IV_SIZE);
 CC_Mean=2.0*(CC_LLR[ZERO]-CC_LLR[ONE])/(double)(MAX_FRAMES*CH_IV_SIZE);
 Ch_Hist_Step=(Max_Ch_LLR-Min_Ch_LLR-2.0*Ch_Mean)/(double)HIST_SIZE;
 CC_Hist_Step=(Max_CC_LLR-Min_CC_LLR-2.0*CC_Mean)/(double)HIST_SIZE;
 printf("\t Int Ch. mean:%lf CC. mean:%lf\n",Ch_Mean,CC_Mean);
 printf("\t Hist step Ch:%lf CC:%lf\n",Ch_Hist_Step,CC_Hist_Step);
 return(0);
}
/******************************************************************************/
