/******************************************************************************/
/*   Copyright (C) 2005 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
#include "teqex6type.h"
#include "teqex6ext.h"
/******************************************************************************/
/*                          Get the trellis.
*******************************************************************************/
int Get_Channel_Trellis()
{
 int p_state,p_input,index[NUM_CH_STATES];
 int ip_array[CHAN_MEM_SIZE],parity[CHAN_MEM_SIZE],new_st;
 Init_Ch_Index(index);
 for(p_state=ZERO;p_state<NUM_STATES;p_state++)
 {
  for(p_input=ZERO;p_input<MOD_NUM_INPUTS;p_input++)
  {
   Decimal_to_Binary(p_input,ip_array,CHAN_MEM_SIZE);
   new_st=Fill_Mem(p_state,p_input,CHAN_MEM_SIZE,ip_array,parity);
   Get_Ch_Op(new_st,p_input,ip_array,parity);
   Get_Ch_Next_State(new_st,p_input,ip_array,index);
  }
 }
 Print_Ch_Trellis();
 return(0);
}
/******************************************************************************/
/*                   Initialize the indices to each state.
*******************************************************************************/
int Init_Ch_Index(index)
int index[];
{
 int cnt;
 for(cnt=ZERO;cnt<NUM_CH_STATES;cnt++)
  index[cnt]=ZERO;
 return(0);
}
/******************************************************************************/
/* Fill the memory with the parity bits. Most recent input bit is
 * at ZERO. Most recent parity is also at ZERO.
*******************************************************************************/
int Fill_Mem(st_cnt,mod_ip_cnt,mem_size,ip_array,parity)
int st_cnt,mod_ip_cnt,mem_size,ip_array[],parity[];
{
 int cnt,inp,s_state;
 printf("\t Enc starting state:%d\n",st_cnt);
 for(cnt=mem_size-ONE;cnt>=ZERO;cnt--)
 {
  inp=ip_array[cnt];
  parity[cnt]=Trel_Op[st_cnt][inp];
  printf("\t Input:%d ",inp);
  printf("Parity:%d\n",parity[cnt]);
  st_cnt=Trel_NS[st_cnt][inp];
 }
 s_state=st_cnt*MOD_NUM_INPUTS+mod_ip_cnt;
 printf("\t Final encoder state:%d\n",st_cnt);
 printf("\t Final super trellis state:%d\n\n",s_state);
 return(st_cnt);
}
/******************************************************************************/
/*                          Get the parity bit.
*******************************************************************************/
int Get_Ch_Op(new_st,p_input,ip_array,parity)
int new_st,p_input,ip_array[],parity[];
{
 int cnt,bit,enc_bit,s_state,ip;
 double reop,imop;
 for(ip=ZERO;ip<NUM_INPUTS;ip++)
 {
  enc_bit=Trel_Op[new_st][ip];
  reop=Chan_Tap[ZERO]*Bit_Map[ip];
  imop=Chan_Tap[ZERO]*Bit_Map[enc_bit];
  for(cnt=ZERO;cnt<CHAN_MEM_SIZE;cnt++)
  {
   bit=ip_array[cnt];
   enc_bit=parity[cnt];
   reop=reop+Chan_Tap[cnt+ONE]*Bit_Map[bit];
   imop=imop+Chan_Tap[cnt+ONE]*Bit_Map[enc_bit];
  }
  s_state=new_st*MOD_NUM_INPUTS+p_input;
  Re_Ch_Trel_Op[s_state][ip]=reop;
  Im_Ch_Trel_Op[s_state][ip]=imop;
 }
 return(0);
}
/******************************************************************************/
/*                           Get the next state.
*******************************************************************************/
int Get_Ch_Next_State(new_st,p_input,ip_array,index)
int new_st,p_input,ip_array[],index[];
{
 int chan_nxt_state,enc_state,cnt,s_state,ip,new_s_state;
 int inx,tip[CHAN_MEM_SIZE];
 for(ip=ZERO;ip<NUM_INPUTS;ip++)
 {
  enc_state=Trel_NS[new_st][ip];
  tip[ZERO]=ip;
  for(cnt=ZERO;cnt<CHAN_MEM_SIZE-ONE;cnt++)
   tip[cnt+ONE]=ip_array[cnt];
  chan_nxt_state=Binary_to_Decimal(tip,CHAN_MEM_SIZE);
  s_state=new_st*MOD_NUM_INPUTS+p_input;
  new_s_state=enc_state*MOD_NUM_INPUTS+chan_nxt_state;
  inx=index[new_s_state]++;
  if(inx >= NUM_INPUTS)
   Error_Mes(1);
  Ch_Trel_NS[s_state][ip]=new_s_state;
  Ch_Trel_PS[new_s_state][inx]=s_state;
  Ch_Trel_Ip[new_s_state][inx]=ip;
 }
 return(0);
}
/******************************************************************************/
/*                          Print the channel trellis.
*******************************************************************************/
int Print_Ch_Trellis()
{
 int p_state,p_input;
 printf("\t Channel trellis:\n\n");
 for(p_state=ZERO;p_state<NUM_CH_STATES;p_state++)
 {
  for(p_input=ZERO;p_input<NUM_INPUTS;p_input++)
  {
   printf("\t St:%3d Ip:%d ",p_state,p_input);
   printf("N. St:%3d ",Ch_Trel_NS[p_state][p_input]);
   printf("Prv. St:%3d ",Ch_Trel_PS[p_state][p_input]);
   printf("Input to st:%d\n",Ch_Trel_Ip[p_state][p_input]);
   printf("\t\t Re. Op:%+lf ",Re_Ch_Trel_Op[p_state][p_input]);
   printf("Im. Op:%+lf\n",Im_Ch_Trel_Op[p_state][p_input]);
  }
 }
 printf("\n");
 return(0);
}
/******************************************************************************/
