/******************************************************************************/
/*   Copyright (C) 2005 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
#include "teqex6type.h"
#include "teqex6ext.h"
/******************************************************************************/
/*   Compute the input and output mutual information between the bits and
 *   the LLRs.
*******************************************************************************/
int Compute_Mutual_Info()
{
 Compute_Input_MI();
 Compute_Mean_LLRs();
// Compute_Output_Ch_MI();
// Compute_Output_CC_MI();
 Compute_Hist_Output_Ch_MI();
 Compute_Hist_Output_CC_MI();
 Print_Results();
 if(fabs(Snr-MIN_SNR) < ERROR)
 {
  Print_Hist();
  Print_Ch_Gauss();
  Print_CC_Gauss();
 }
 return(0);
}
/******************************************************************************/
/*                    Compute the input mutual information.
*******************************************************************************/
int Compute_Input_MI()
{
 int bit;
 double mean,var,llim,ulim,step,x,y0,y1,numer,denom;
 mean=2.0/LLR_Variance;
 var=4.0/LLR_Variance;
 llim= -mean-5.0*sqrt(var);
 ulim=  mean+5.0*sqrt(var);
 step=(ulim-llim)/(double)MAX_STEPS;
 Ip_MI=0.0;
 for(bit=ZERO;bit<NUM_INPUTS;bit++)
 {
  for(x=llim;x<ulim;x=x+step)
  {
   y0=(x-mean*Bit_Map[bit])*(x-mean*Bit_Map[bit])/(2.0*var);
   y1=(x+mean*Bit_Map[bit])*(x+mean*Bit_Map[bit])/(2.0*var);
   numer=exp(-y0);
   denom=exp(-y0)+exp(-y1);
   Ip_MI=Ip_MI+numer*log(2.0*numer/denom);
  }
 }
 Ip_MI=Ip_MI*0.5*step/(sqrt(2.0*MY_PI*var)*log(2.0));
 return(0);
}
/******************************************************************************/
/*           Compute the mean value of the log-likelihood ratios.
*******************************************************************************/
int Compute_Mean_LLRs()
{
 Ch_Mean=(Ch_LLR[ZERO]-Ch_LLR[ONE])/(double)(MAX_FRAMES*CH_IV_SIZE);
 CC_Mean=(CC_LLR[ZERO]-CC_LLR[ONE])/(double)(MAX_FRAMES*CH_IV_SIZE);
 Ch_Var=2.0*Ch_Var/(double)(MAX_FRAMES*CH_IV_SIZE);
 CC_Var=2.0*CC_Var/(double)(MAX_FRAMES*CH_IV_SIZE);
 printf("\t Ch. mean:%lf CC. mean:%lf\n",Ch_Mean,CC_Mean);
 printf("\t Ch. var:%lf CC var:%lf\n\n",Ch_Var,CC_Var);
 return(0);
}
/******************************************************************************/
/*   Compute the channel MAP decoder output mutual information using the
 *   histogram.
*******************************************************************************/
int Compute_Hist_Output_Ch_MI()
{
 Ch_Op_MI=0.0;
 Get_Ch_MI_1();
 Get_Ch_MI_2();
 Ch_Op_MI=Ch_Op_MI/(log(2.0)*(double)(MAX_FRAMES*CH_IV_SIZE));
 return(0);
}
/******************************************************************************/
/*   Compute the channel MAP decoder output mutual information using the
 *   histogram when the bit is +1.
*******************************************************************************/
int Get_Ch_MI_1()
{
 int cnt,inx;
 double numer,denom,x,x1,min;
 min=Min_Ch_LLR+2.0*Ch_Mean;
 for(cnt=ZERO;cnt<HIST_SIZE;cnt++)
 {
  if(Ch_Freq[cnt] != ZERO)
  {
   numer=(double)Ch_Freq[cnt];
   x=min+(double)cnt*Ch_Hist_Step+0.5*Ch_Hist_Step;
   x1=x-Min_Ch_LLR;
   inx=(int)(x1/Ch_Hist_Step);
   if(inx<HIST_SIZE)
    denom=(double)Ch_Freq[inx];
   else
    denom=0.0;
   denom=denom+numer;
   Ch_Op_MI=Ch_Op_MI+numer*log(2.0*numer/denom);
  }
 }
 return(0);
}
/******************************************************************************/
/*   Compute the channel MAP decoder output mutual information using the
 *   histogram when the bit is -1.
*******************************************************************************/
int Get_Ch_MI_2()
{
 int cnt,inx;
 double numer,denom,x,x1,min;
 min=Min_Ch_LLR;
 for(cnt=ZERO;cnt<HIST_SIZE;cnt++)
 {
  if(Ch_Freq[cnt] != ZERO)
  {
   numer=(double)Ch_Freq[cnt];
   x=min+(double)cnt*Ch_Hist_Step+0.5*Ch_Hist_Step;
   x1=x-Min_Ch_LLR-2.0*Ch_Mean;
   inx=(int)(x1/Ch_Hist_Step);
   if(inx >= ZERO)
    denom=(double)Ch_Freq[inx];
   else
    denom=0.0;
   denom=denom+numer;
   Ch_Op_MI=Ch_Op_MI+numer*log(2.0*numer/denom);
  }
 }
 return(0);
}
/******************************************************************************/
/*   Compute the CC MAP decoder output mutual information using the
 *   histogram.
*******************************************************************************/
int Compute_Hist_Output_CC_MI()
{
 CC_Op_MI=0.0;
 Get_CC_MI_1();
 Get_CC_MI_2();
 CC_Op_MI=CC_Op_MI/(log(2.0)*(double)(MAX_FRAMES*CH_IV_SIZE));
 return(0);
}
/******************************************************************************/
/*   Compute the CC MAP decoder output mutual information using the
 *   histogram when the bit is +1.
*******************************************************************************/
int Get_CC_MI_1()
{
 int cnt,inx;
 double numer,denom,x,x1,min;
 min=Min_CC_LLR+2.0*CC_Mean;
 for(cnt=ZERO;cnt<HIST_SIZE;cnt++)
 {
  if(CC_Freq[cnt] != ZERO)
  {
   numer=(double)CC_Freq[cnt];
   x=min+(double)cnt*CC_Hist_Step+0.5*CC_Hist_Step;
   x1=x-Min_CC_LLR;
   inx=(int)(x1/CC_Hist_Step);
   if(inx<HIST_SIZE)
    denom=(double)CC_Freq[inx];
   else
    denom=0.0;
   denom=denom+numer;
   CC_Op_MI=CC_Op_MI+numer*log(2.0*numer/denom);
  }
 }
 return(0);
}
/******************************************************************************/
/*   Compute the CC MAP decoder output mutual information using the
 *   histogram when the bit is -1.
*******************************************************************************/
int Get_CC_MI_2()
{
 int cnt,inx;
 double numer,denom,x,x1,min;
 min=Min_CC_LLR;
 for(cnt=ZERO;cnt<HIST_SIZE;cnt++)
 {
  if(CC_Freq[cnt] != ZERO)
  {
   numer=CC_Freq[cnt];
   x=min+(double)cnt*CC_Hist_Step+0.5*CC_Hist_Step;
   x1=x-Min_CC_LLR-2.0*CC_Mean;
   inx=(int)(x1/CC_Hist_Step);
   if(inx >= ZERO)
    denom=(double)CC_Freq[inx];
   else
    denom=0.0;
   denom=denom+numer;
   CC_Op_MI=CC_Op_MI+numer*log(2.0*numer/denom);
  }
 }
 return(0);
}
/******************************************************************************/
/*                          Print the results.
*******************************************************************************/
int Print_Results()
{
 fprintf(fp3,"%lf %lf\n",Ip_MI,Ch_Op_MI);
 fprintf(fp4,"%lf %lf\n",CC_Op_MI,Ip_MI);
 return(0);
}
/******************************************************************************/
/*                           Print the histogram.
*******************************************************************************/
int Print_Hist()
{
 int cnt;
 double ch_x,ch_y,cc_x,cc_y,a;
 a=(double)(MAX_FRAMES*CH_IV_SIZE);
 for(cnt=ZERO;cnt<HIST_SIZE;cnt++)
 {
  ch_x=Min_Ch_LLR+Ch_Mean+(double)cnt*Ch_Hist_Step+0.5*Ch_Hist_Step;
  cc_x=Min_CC_LLR+CC_Mean+(double)cnt*CC_Hist_Step+0.5*CC_Hist_Step;
  ch_y=2.0*(double)Ch_Freq[cnt]/(Ch_Hist_Step*a);
  cc_y=2.0*(double)CC_Freq[cnt]/(CC_Hist_Step*a);
  fprintf(fp5,"%lf %lf\n",ch_x,ch_y);
  fprintf(fp6,"%lf %lf\n",cc_x,cc_y);
 }
 return(0);
}
/******************************************************************************/
/*   Print the Gaussian distribution with zero mean and given variance
 *   corresponding to the channel LLRs.
*******************************************************************************/
int Print_Ch_Gauss()
{
 double mean,var,llim,ulim,step,x,y;
 mean=Ch_Mean;
 var=Ch_Var;
 llim= -mean-5.0*sqrt(var);
 ulim=  mean+5.0*sqrt(var);
 step=(ulim-llim)/(double)MAX_STEPS;
 for(x=llim;x<ulim;x=x+step)
 {
  y= -x*x/(2.0*var);
  y=exp(y)/sqrt(2.0*MY_PI*var);
  fprintf(fp7,"%lf %lf\n",x,y);
 }
 return(0);
}
/******************************************************************************/
/*   Print the Gaussian distribution with zero mean and given variance
 *   corresponding to the CC LLRs.
*******************************************************************************/
int Print_CC_Gauss()
{
 double mean,var,llim,ulim,step,x,y;
 mean=CC_Mean;
 var=CC_Var;
 llim= -mean-5.0*sqrt(var);
 ulim=  mean+5.0*sqrt(var);
 step=(ulim-llim)/(double)MAX_STEPS;
 for(x=llim;x<ulim;x=x+step)
 {
  y= -x*x/(2.0*var);
  y=exp(y)/sqrt(2.0*MY_PI*var);
  fprintf(fp8,"%lf %lf\n",x,y);
 }
 return(0);
}
/******************************************************************************/
