/******************************************************************************/
/*   Copyright (C) 2005 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
#include "teqex8type.h"
#include "teqex8ext.h"
/******************************************************************************/
/*    Compute the a posteriori probabilities for the predictive MAP decoder.
*******************************************************************************/
int Compute_Ch_APP()
{
 Init_Alpha_Beta_Ch();
 Compute_Alpha_Beta_Ch();
 Compute_APPLes_Ch();
 return(0);
}
/******************************************************************************/
/*                          Initialize alpha and beta.
*******************************************************************************/
int Init_Alpha_Beta_Ch()
{
 int state;
 for(state=ZERO;state<NUM_ST_STATES;state++)
 {
  Ch_Alpha[ZERO][state]=1.0;
  Ch_Beta[CH_IV_SIZE][state]=1.0;
 }
// Ch_Alpha[ZERO][ZERO]=1.0;
// Ch_Beta[CH_IV_SIZE][Chan_F_State]=1.0;
 return(0);
}
/******************************************************************************/
/*                   Compute the alpha and beta values.
*******************************************************************************/
int Compute_Alpha_Beta_Ch()
{
 int time,state;
 for(time=ZERO;time<=CH_IV_SIZE-TWO;time++)
 {
  for(state=ZERO;state<NUM_ST_STATES;state++)
   Get_New_Ch_Alpha_Beta(time,state);
  Normalize_Ch_Alpha_Beta(time);
 }
 return(0);
}
/******************************************************************************/
/*      Compute the new alpha and beta values for a given state and time.
*******************************************************************************/
int Get_New_Ch_Alpha_Beta(time,state)
int time,state;
{
 int input,prv_st,nxt_st,st_ip;
 double ga,gb,proba,probb,prv_alpha,nxt_beta,alpha,beta;
 alpha=beta=0.0;
 for(input=ZERO;input<NUM_INPUTS;input++)                       /* Data bits. */
 {
  prv_st=STrel_PS[state][input];
  nxt_st=STrel_NS[state][input];
  st_ip=STrel_Ip[state][input];                            /* Input to state. */
  ga=Ch_Gamma[time][prv_st][st_ip];                       /* Gamma for alpha. */
  gb=Ch_Gamma[CH_IV_SIZE-ONE-time][state][input];          /* Gamma for beta. */
  proba=Ch_Prob[time][st_ip];
  probb=Ch_Prob[CH_IV_SIZE-ONE-time][input];
  prv_alpha=Ch_Alpha[time][prv_st];
  nxt_beta=Ch_Beta[CH_IV_SIZE-ONE-time+ONE][nxt_st];
  alpha=alpha+ga*proba*prv_alpha;
  beta=beta+gb*probb*nxt_beta;
 }
 Ch_Alpha[time+ONE][state]=alpha;
 Ch_Beta[CH_IV_SIZE-ONE-time][state]=beta;
 return(0);
}
/******************************************************************************/
/*           Compute the APPs due to extrinsic information alone.
*******************************************************************************/
int Compute_APPLes_Ch()
{
 int time;
 double app[NUM_INPUTS];
 for(time=ZERO;time<CH_IV_SIZE;time++)
 {
  Get_APPLe_Ch(time,app);
  Get_Prob_Ch(time,app);
 }
 return(0);
}
/******************************************************************************/
/*   Get a posteriori probability of +1 and -1 due to extrinsic info alone.
*******************************************************************************/
int Get_APPLe_Ch(time,app)
int time;
double app[];
{
 int state,input,nxt_state;
 double prv_alpha,nxt_beta,gamma;
 app[ZERO]=app[ONE]=0.0;
 for(state=ZERO;state<NUM_ST_STATES;state++)
 {
  for(input=ZERO;input<NUM_INPUTS;input++)
  {
   nxt_state=STrel_NS[state][input];
   prv_alpha=Ch_Alpha[time][state];
   nxt_beta=Ch_Beta[time+ONE][nxt_state];
   gamma=Ch_Gamma[time][state][input];
   app[input]=app[input]+prv_alpha*nxt_beta*gamma;
  }
 }
 return(0);
}
/******************************************************************************/
/*                     Get the normalized probabilities.
*******************************************************************************/
int Get_Prob_Ch(time,app)
int time;
double app[];
{
 int input;
 double sum;
 sum=0.0;
 for(input=ZERO;input<NUM_INPUTS;input++)
  sum=sum+app[input];
 for(input=ZERO;input<NUM_INPUTS;input++)
  Ch_Prob[time][input]=app[input]/sum;
 return(0);
}
/******************************************************************************/
