/******************************************************************************/
/*   Copyright (C) 2005 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
#include "teqex8type.h"
#include "teqex8ext.h"
/******************************************************************************/
/*                   Initialize simulation variables.
*******************************************************************************/
int Sim_Init()
{
 Get_Noise_Variance();
 Noise_Gen_Init();
 Iv_Noise_Gen_Init();
 Eq_Init();
 if(IDEAL_EQ == ONE)
 {
  Ideal_FF_Eq_Init();
  Ideal_FB_Eq_Init();
 }
 Chan_Init();
 return(0);
}
/******************************************************************************/
/*                        Get the noise variance.
*******************************************************************************/
int Get_Noise_Variance()
{
 Variance=2.0/pow(10.0,0.1*SNR_B);
 printf("\t Noise variance:%lf.\n\n",Variance);
 Inv_Twice_Noise_Var=1.0/(2.0*Variance);
 return(0);
}
/******************************************************************************/
/*                     Initialize the noise generator.
*******************************************************************************/
int Noise_Gen_Init()
{
 int cnt;
 long int random(void);
 for(cnt=ZERO;cnt<PN_SIZE;cnt++)
 {
  N1_Store[cnt]=(unsigned short)random();
  N2_Store[cnt]=(unsigned short)random();
 }
 return(0);
}
/******************************************************************************/
/*                Initialize the interleaver noise generator.
*******************************************************************************/
int Iv_Noise_Gen_Init()
{
 int cnt;
 long int random(void);
 for(cnt=ZERO;cnt<PN_SIZE;cnt++)
  Iv_Noise_Store[cnt]=(unsigned short)random();
 return(0);
}
/******************************************************************************/
/*                    Initialize equalizer parameters.
*******************************************************************************/
int Eq_Init()
{
 int cnt;
 for(cnt=ZERO;cnt<EQ_SIZE;cnt++)
  Re_Eq[cnt]=Im_Eq[cnt]=Re_Eq_Reg[cnt]=Im_Eq_Reg[cnt]=0.0;
 for(cnt=ZERO;cnt<FB_SIZE;cnt++)
  Re_FB[cnt]=Im_FB[cnt]=Re_FB_Reg[cnt]=Im_FB_Reg[cnt]=0.0;
 Step=0.5;
 Err_Reg_Inx=Eq_Buf_Inx=ZERO;
 return(0);
}
/******************************************************************************/
/*          Initialize the feedforward part of the ideal equalizer.
*******************************************************************************/
int Ideal_FF_Eq_Init()
{
 int cnt;
 double re_tmp,im_tmp;
 FILE *tfp;
 tfp=fopen("../opt_eq/opt_cdfe_ff.dat","r");
 if(tfp == NULL)
  Error_Mes(7);
 cnt=ZERO;
 while(fscanf(tfp,"%lf %lf",&im_tmp,&re_tmp) != EOF)
 {
  if(cnt == EQ_SIZE)
   Error_Mes(14);
  Im_Eq[cnt]=im_tmp;
  Re_Eq[cnt]=re_tmp;
  cnt++;
 }
 if(cnt < EQ_SIZE)
  Error_Mes(15);
 fclose(tfp);
 return(0);
}
/******************************************************************************/
/*          Initialize the feedback part of the ideal equalizer.
*******************************************************************************/
int Ideal_FB_Eq_Init()
{
 int cnt;
 double re_tmp,im_tmp;
 FILE *tfp;
 tfp=fopen("../opt_eq/opt_fb.dat","r");
 if(tfp == NULL)
  Error_Mes(7);
 cnt=ZERO;
 while(fscanf(tfp,"%lf %lf",&im_tmp,&re_tmp) != EOF)
 {
  if(cnt == FB_SIZE)
   Error_Mes(14);
  Im_FB[cnt]=im_tmp;
  Re_FB[cnt]=re_tmp;
  cnt++;
 }
 if(cnt < FB_SIZE)
  Error_Mes(15);
 fclose(tfp);
 return(0);
}
/******************************************************************************/
/*                          Initialize the channel.
*******************************************************************************/
int Chan_Init()
{
 int cnt;
 for(cnt=ZERO;cnt<CHAN_SIZE;cnt++)
  Re_Ch_Reg[cnt]=Im_Ch_Reg[cnt]=0.0;
 return(0);
}
/******************************************************************************/
