/******************************************************************************/
/*   Copyright (C) 2005 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
#include "teqex8type.h"
#include "teqex8ext.h"
/******************************************************************************/
int Simulation()
{
 Train_The_Equalizer();
 Get_The_EXIT_Chart();
 return(0);
}
/******************************************************************************/
/*                        Train the equalizer.
*******************************************************************************/
int Train_The_Equalizer()
{
 for(Frame_Cnt=ZERO;Frame_Cnt<TRN_FRAMES;Frame_Cnt++)
 {
  Get_Frame();
  if(Frame_Cnt > TWO)
   Step=0.05;
  Init_FB_Reg();
  ISI_Procedures();
 }
 return(0);
}
/******************************************************************************/
/*                     Initialize the feedback register.
*******************************************************************************/
int Init_FB_Reg()
{
 int cnt;
 for(cnt=ZERO;cnt<FB_SIZE;cnt++)
  Re_FB_Reg[cnt]=Im_FB_Reg[cnt]=1.0;
 return(0);
}
/******************************************************************************/
/*                             ISI procedures.
*******************************************************************************/
int ISI_Procedures()
{
 int time,bit,pty;
 Enc1_State=ZERO;
 for(time=ZERO;time<RX_DATA_END;time++)
 {
  bit=Get_Channel_Ip(time);
  pty=Trel_Op[Enc1_State][bit];
  Enc1_State=Trel_NS[Enc1_State][bit];
  if((time >= PREAMBLE) && (time < TX_DATA_END))
   Store_In_Eq_Buf(bit,pty);
  Get_Channel_Op(bit,pty);
  Add_Noise();
  Get_Eq_Op();
  if(time>=RX_DATA_START)
   Eq_Training();
 }
 return(0);
}
/******************************************************************************/
/*                          Get the channel input.
*******************************************************************************/
int Get_Channel_Ip(time)
int time;
{
 int iv_ip_time,bit;
 if(time < PREAMBLE)
  bit=ZERO;
 else if(time < TX_DATA_END)
 {
  iv_ip_time=Ch_De_Iv_Map[time-PREAMBLE];
  bit=Enc_Op[iv_ip_time];
 }
 else
  bit=ZERO;
 return(bit);
}
/******************************************************************************/
/*                       Store in equalizer buffer.
*******************************************************************************/
int Store_In_Eq_Buf(bit,pty)
int bit,pty;
{
 if(Eq_Buf_Inx == EQ_BUF_SIZE)
  Error_Mes(13);
 Eq_Buf[Eq_Buf_Inx]=TWO*pty+bit;
 Eq_Buf_Inx++;
 return(0);
}
/******************************************************************************/
/*                  Get the samples at the channel output.
*******************************************************************************/
int Get_Channel_Op(bit,pty)
int bit,pty;
{
 int cnt;
 Re_Ch_Op=Im_Ch_Op=0.0;
 Right_Shift_Reg(Re_Ch_Reg,Im_Ch_Reg,CHAN_SIZE);
 Re_Ch_Reg[ZERO]=Bit_Map[bit];
 Im_Ch_Reg[ZERO]=Bit_Map[pty];
 for(cnt=ZERO;cnt<CHAN_SIZE;cnt++)
 {
  Re_Ch_Op=Re_Ch_Op+Re_Ch[cnt]*Re_Ch_Reg[cnt];
  Re_Ch_Op=Re_Ch_Op-Im_Ch[cnt]*Im_Ch_Reg[cnt];
  Im_Ch_Op=Im_Ch_Op+Re_Ch[cnt]*Im_Ch_Reg[cnt];
  Im_Ch_Op=Im_Ch_Op+Im_Ch[cnt]*Re_Ch_Reg[cnt];
 }
 return(0);
}
/******************************************************************************/
/*                        Add white Gaussian noise.
*******************************************************************************/
int Add_Noise()
{
 double erand48();
 N1_Val[URADIUS]=erand48(N1_Store);
 N1_Val[UTHETA]=erand48(N2_Store);
 My_Gauss(Variance,N1_Val);
 Re_Ch_Op=Re_Ch_Op+N1_Val[GRAND_0];
 Im_Ch_Op=Im_Ch_Op+N1_Val[GRAND_1];
 return(0);
}
/******************************************************************************/
/*                          Get equalizer output.
*******************************************************************************/
int Get_Eq_Op()
{
 int cnt;
 Right_Shift_Reg(Re_Eq_Reg,Im_Eq_Reg,EQ_SIZE);
 Re_Eq_Reg[ZERO]=Re_Ch_Op;
 Im_Eq_Reg[ZERO]=Im_Ch_Op;
 Re_Eq_Op=Im_Eq_Op=0.0;
 for(cnt=ZERO;cnt<EQ_SIZE;cnt++)
 {
  Re_Eq_Op=Re_Eq_Op+Re_Eq_Reg[cnt]*Re_Eq[cnt]-Im_Eq_Reg[cnt]*Im_Eq[cnt];
  Im_Eq_Op=Im_Eq_Op+Re_Eq_Reg[cnt]*Im_Eq[cnt]+Im_Eq_Reg[cnt]*Re_Eq[cnt];
 }
 return(0);
}
/******************************************************************************/
